"use strict";

var _test = require("test");
var _assert = require("assert");
var _nodeStream = require("node:stream");
var _encryptor = require("./_encryptor");
var _crypto = _interopRequireDefault(require("crypto"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const algorithms = ['none', 'aes-256-cbc', 'aes-256-gcm'];
function streamToBuffer(stream) {
  return new Promise(resolve => {
    const bufs = [];
    stream.on('data', function (d) {
      bufs.push(d);
    });
    stream.on('end', function () {
      resolve(Buffer.concat(bufs));
    });
  });
}
algorithms.forEach(algorithm => {
  (0, _test.describe)(`test algorithm ${algorithm}`, () => {
    const key = algorithm === 'none' ? undefined : '73c1838d7d8a6088ca2317fb5f29cd91';
    const encryptor = (0, _encryptor._getEncryptor)(algorithm, key);
    const buffer = _crypto.default.randomBytes(1024 * 1024 + 1);
    (0, _test.it)('handle buffer', () => {
      const encrypted = encryptor.encryptData(buffer);
      if (algorithm !== 'none') {
        _assert.strict.equal(encrypted.equals(buffer), false);
        _assert.strict.notEqual(encrypted.length, buffer.length);
      }
      const decrypted = encryptor.decryptData(encrypted);
      _assert.strict.equal(decrypted.equals(buffer), true);
    });
    (0, _test.it)('handle stream', async () => {
      const stream = _nodeStream.Readable.from(buffer);
      stream.length = buffer.length;
      const encrypted = encryptor.encryptStream(stream);
      if (algorithm !== 'none') {
        _assert.strict.equal(encrypted.length, undefined);
      }
      const decrypted = encryptor.decryptStream(encrypted);
      const decryptedBuffer = await streamToBuffer(decrypted);
      _assert.strict.equal(decryptedBuffer.equals(buffer), true);
    });
  });
});
//# sourceMappingURL=_encryptor.test.js.map