"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _execa = _interopRequireDefault(require("execa"));
var _fsExtra = _interopRequireDefault(require("fs-extra"));
var _promiseToolbox = require("promise-toolbox");
var _path = require("path");
var _os = require("os");
var _local = _interopRequireDefault(require("./local"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const sudoExeca = (command, args, opts) => (0, _execa.default)('sudo', [command, ...args], opts);
class MountHandler extends _local.default {
  constructor(remote, {
    mountsDir = (0, _path.join)((0, _os.tmpdir)(), 'xo-fs-mounts'),
    useSudo = false,
    ...opts
  } = {}, params) {
    super(remote, opts);
    this._execa = useSudo ? sudoExeca : _execa.default;
    this._keeper = undefined;
    this._params = {
      ...params,
      options: [params.options, remote.options ?? params.defaultOptions].filter(_ => _ !== undefined).join(',')
    };
    this._realPath = (0, _path.join)(mountsDir, remote.id || Math.random().toString(36).slice(2));
  }
  async _forget() {
    const keeper = this._keeper;
    if (keeper === undefined) {
      return;
    }
    this._keeper = undefined;
    await _fsExtra.default.close(keeper);
    await _promiseToolbox.ignoreErrors.call(this._execa('umount', [this._getRealPath()], {
      env: {
        LANG: 'C'
      }
    }));
  }
  _getRealPath() {
    return this._realPath;
  }
  async _sync() {
    {
      const keeper = this._keeper;
      if (keeper !== undefined) {
        this._keeper = undefined;
        _promiseToolbox.ignoreErrors.call(_fsExtra.default.close(keeper));
      }
    }
    const realPath = this._getRealPath();
    await _fsExtra.default.ensureDir(realPath);
    try {
      const {
        type,
        device,
        options,
        env
      } = this._params;
      await this._execa('mount', ['-o', options, '-t', type, device, realPath], {
        env: {
          LANG: 'C',
          ...env
        }
      });
    } catch (error) {
      try {
        await this._execa('findmnt', [realPath], {
          stdio: 'ignore'
        });
      } catch (_) {
        throw error;
      }
    }
    const keeperPath = `${realPath}/.keeper_${Math.random().toString(36).slice(2)}`;
    this._keeper = await _fsExtra.default.open(keeperPath, 'w');
    _promiseToolbox.ignoreErrors.call(_fsExtra.default.unlink(keeperPath));
  }
}
exports.default = MountHandler;
//# sourceMappingURL=_mount.js.map