"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "DEFAULT_ENCRYPTION_ALGORITHM", {
  enumerable: true,
  get: function () {
    return _encryptor.DEFAULT_ENCRYPTION_ALGORITHM;
  }
});
Object.defineProperty(exports, "UNENCRYPTED_ALGORITHM", {
  enumerable: true,
  get: function () {
    return _encryptor.UNENCRYPTED_ALGORITHM;
  }
});
exports.getSyncedHandler = exports.getHandler = void 0;
Object.defineProperty(exports, "isLegacyEncryptionAlgorithm", {
  enumerable: true,
  get: function () {
    return _encryptor.isLegacyEncryptionAlgorithm;
  }
});
var _execa = _interopRequireDefault(require("execa"));
var _xoRemoteParser = require("xo-remote-parser");
var _local = _interopRequireDefault(require("./local"));
var _nfs = _interopRequireDefault(require("./nfs"));
var _s = _interopRequireDefault(require("./s3"));
var _smb = _interopRequireDefault(require("./smb"));
var _encryptor = require("./_encryptor");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const HANDLERS = {
  file: _local.default,
  nfs: _nfs.default,
  s3: _s.default
};
try {
  _execa.default.sync('mount.cifs', ['-V']);
  HANDLERS.smb = _smb.default;
} catch (_) {}
const getHandler = (remote, ...rest) => {
  const Handler = HANDLERS[(0, _xoRemoteParser.parse)(remote.url).type];
  if (!Handler) {
    throw new Error('Unhandled remote type');
  }
  return new Handler(remote, ...rest);
};
exports.getHandler = getHandler;
const getSyncedHandler = async (...opts) => {
  const handler = getHandler(...opts);
  await handler.sync();
  return {
    dispose: () => handler.forget(),
    value: handler
  };
};
exports.getSyncedHandler = getSyncedHandler;
//# sourceMappingURL=index.js.map