"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _clientS = require("@aws-sdk/client-s3");
var _libStorage = require("@aws-sdk/lib-storage");
var _nodeHttpHandler = require("@aws-sdk/node-http-handler");
var _middlewareApplyBodyChecksum = require("@aws-sdk/middleware-apply-body-checksum");
var _http = require("http");
var _https = require("https");
var _retry = _interopRequireDefault(require("promise-toolbox/retry"));
var _log = require("@xen-orchestra/log");
var _decorateWith = require("@vates/decorate-with");
var _stream = require("stream");
var _xoRemoteParser = require("xo-remote-parser");
var _copyStreamToBuffer = _interopRequireDefault(require("./_copyStreamToBuffer.js"));
var _guessAwsRegion = _interopRequireDefault(require("./_guessAwsRegion.js"));
var _abstract = _interopRequireDefault(require("./abstract"));
var _path = require("./path");
var _asyncEach = require("@vates/async-each");
var _dec, _class;
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) { var desc = {}; Object.keys(descriptor).forEach(function (key) { desc[key] = descriptor[key]; }); desc.enumerable = !!desc.enumerable; desc.configurable = !!desc.configurable; if ('value' in desc || desc.initializer) { desc.writable = true; } desc = decorators.slice().reverse().reduce(function (desc, decorator) { return decorator(target, property, desc) || desc; }, desc); if (context && desc.initializer !== void 0) { desc.value = desc.initializer ? desc.initializer.call(context) : void 0; desc.initializer = undefined; } if (desc.initializer === void 0) { Object.defineProperty(target, property, desc); desc = null; } return desc; }
const MAX_PART_SIZE = 1024 * 1024 * 1024 * 5;
const {
  warn
} = (0, _log.createLogger)('xo:fs:s3');
let S3Handler = (_dec = (0, _decorateWith.decorateWith)(_retry.default.wrap, {
  delays: [100, 200, 500, 1000, 2000],
  when: e => {
    var _e$$metadata;
    return ((_e$$metadata = e.$metadata) === null || _e$$metadata === void 0 ? void 0 : _e$$metadata.httpStatusCode) === 500;
  },
  onRetry(error) {
    warn('retrying writing file', {
      attemptNumber: this.attemptNumber,
      delay: this.delay,
      error,
      file: this.arguments[0]
    });
  }
}), (_class = class S3Handler extends _abstract.default {
  constructor(remote, _opts) {
    super(remote);
    const {
      allowUnauthorized,
      host,
      path,
      username,
      password,
      protocol,
      region = (0, _guessAwsRegion.default)(host)
    } = (0, _xoRemoteParser.parse)(remote.url);
    this._s3 = new _clientS.S3Client({
      apiVersion: '2006-03-01',
      endpoint: `${protocol}://${host}`,
      forcePathStyle: true,
      credentials: {
        accessKeyId: username,
        secretAccessKey: password
      },
      tls: protocol === 'https',
      region,
      requestHandler: new _nodeHttpHandler.NodeHttpHandler({
        socketTimeout: 600000,
        httpAgent: new _http.Agent({
          keepAlive: true
        }),
        httpsAgent: new _https.Agent({
          rejectUnauthorized: !allowUnauthorized,
          keepAlive: true
        })
      })
    });
    this._s3.middlewareStack.use((0, _middlewareApplyBodyChecksum.getApplyMd5BodyChecksumPlugin)(this._s3.config));
    const parts = (0, _path.split)(path);
    this._bucket = parts.shift();
    this._dir = (0, _path.join)(...parts);
  }
  get type() {
    return 's3';
  }
  _makeCopySource(path) {
    return (0, _path.join)(this._bucket, this._dir, path);
  }
  _makeKey(file) {
    return (0, _path.join)(this._dir, file);
  }
  _makePrefix(dir) {
    const prefix = (0, _path.join)(this._dir, dir, '/');
    if (prefix !== './') {
      return prefix;
    }
  }
  _createParams(file) {
    return {
      Bucket: this._bucket,
      Key: this._makeKey(file)
    };
  }
  async _multipartCopy(oldPath, newPath) {
    const size = await this._getSize(oldPath);
    const CopySource = this._makeCopySource(oldPath);
    const multipartParams = await this._s3.send(new _clientS.CreateMultipartUploadCommand({
      ...this._createParams(newPath)
    }));
    try {
      const parts = [];
      let start = 0;
      while (start < size) {
        const partNumber = parts.length + 1;
        const upload = await this._s3.send(new _clientS.UploadPartCopyCommand({
          ...multipartParams,
          CopySource,
          CopySourceRange: `bytes=${start}-${Math.min(start + MAX_PART_SIZE, size) - 1}`,
          PartNumber: partNumber
        }));
        parts.push({
          ETag: upload.CopyPartResult.ETag,
          PartNumber: partNumber
        });
        start += MAX_PART_SIZE;
      }
      await this._s3.send(new _clientS.CompleteMultipartUploadCommand({
        ...multipartParams,
        MultipartUpload: {
          Parts: parts
        }
      }));
    } catch (e) {
      await this._s3.send(new _clientS.AbortMultipartUploadCommand(multipartParams));
      throw e;
    }
  }
  async _copy(oldPath, newPath) {
    const CopySource = this._makeCopySource(oldPath);
    try {
      await this._s3.send(new _clientS.CopyObjectCommand({
        ...this._createParams(newPath),
        CopySource
      }));
    } catch (e) {
      if (e.name === 'EntityTooLarge') {
        return this._multipartCopy(oldPath, newPath);
      }
      if (e.name === 'NoSuchKey') {
        const error = new Error(`ENOENT: no such file or directory '${oldPath}'`);
        error.cause = e;
        error.code = 'ENOENT';
        error.path = oldPath;
        throw error;
      }
      throw e;
    }
  }
  async _isNotEmptyDir(path) {
    var _result$Contents;
    const result = await this._s3.send(new _clientS.ListObjectsV2Command({
      Bucket: this._bucket,
      MaxKeys: 1,
      Prefix: this._makePrefix(path)
    }));
    return ((_result$Contents = result.Contents) === null || _result$Contents === void 0 ? void 0 : _result$Contents.length) > 0;
  }
  async _isFile(path) {
    try {
      await this._s3.send(new _clientS.HeadObjectCommand(this._createParams(path)));
      return true;
    } catch (error) {
      if (error.name === 'NotFound') {
        return false;
      }
      throw error;
    }
  }
  async _outputStream(path, input, {
    validator
  }) {
    const Body = new _stream.PassThrough();
    (0, _stream.pipeline)(input, Body, () => {});
    const upload = new _libStorage.Upload({
      client: this._s3,
      params: {
        ...this._createParams(path),
        Body
      }
    });
    await upload.done();
    if (validator !== undefined) {
      try {
        await validator.call(this, path);
      } catch (error) {
        await this.unlink(path);
        throw error;
      }
    }
  }
  async _writeFile(file, data, options) {
    return this._s3.send(new _clientS.PutObjectCommand({
      ...this._createParams(file),
      Body: data
    }));
  }
  async _createReadStream(path, options) {
    try {
      return (await this._s3.send(new _clientS.GetObjectCommand(this._createParams(path)))).Body;
    } catch (e) {
      if (e.name === 'NoSuchKey') {
        const error = new Error(`ENOENT: no such file '${path}'`);
        error.code = 'ENOENT';
        error.path = path;
        throw error;
      }
      throw e;
    }
  }
  async _unlink(path) {
    await this._s3.send(new _clientS.DeleteObjectCommand(this._createParams(path)));
    if (await this._isNotEmptyDir(path)) {
      const error = new Error(`EISDIR: illegal operation on a directory, unlink '${path}'`);
      error.code = 'EISDIR';
      error.path = path;
      throw error;
    }
  }
  async _list(dir) {
    let NextContinuationToken;
    const uniq = new Set();
    const Prefix = this._makePrefix(dir);
    do {
      const result = await this._s3.send(new _clientS.ListObjectsV2Command({
        Bucket: this._bucket,
        Prefix,
        Delimiter: '/',
        ContinuationToken: NextContinuationToken
      }));
      if (result.IsTruncated) {
        warn(`need pagination to browse the directory ${dir} completely`);
        NextContinuationToken = result.NextContinuationToken;
      } else {
        NextContinuationToken = undefined;
      }
      for (const entry of result.CommonPrefixes ?? []) {
        uniq.add((0, _path.basename)(entry.Prefix));
      }
      for (const entry of result.Contents ?? []) {
        uniq.add((0, _path.basename)(entry.Key));
      }
    } while (NextContinuationToken !== undefined);
    return [...uniq];
  }
  async _mkdir(path) {
    if (await this._isFile(path)) {
      const error = new Error(`ENOTDIR: file already exists, mkdir '${path}'`);
      error.code = 'ENOTDIR';
      error.path = path;
      throw error;
    }
  }
  async _rename(oldPath, newPath) {
    await this.copy(oldPath, newPath);
    await this._s3.send(new _clientS.DeleteObjectCommand(this._createParams(oldPath)));
  }
  async _getSize(file) {
    if (typeof file !== 'string') {
      file = file.fd;
    }
    const result = await this._s3.send(new _clientS.HeadObjectCommand(this._createParams(file)));
    return +result.ContentLength;
  }
  async _read(file, buffer, position = 0) {
    if (typeof file !== 'string') {
      file = file.fd;
    }
    const params = this._createParams(file);
    params.Range = `bytes=${position}-${position + buffer.length - 1}`;
    try {
      const result = await this._s3.send(new _clientS.GetObjectCommand(params));
      const bytesRead = await (0, _copyStreamToBuffer.default)(result.Body, buffer);
      return {
        bytesRead,
        buffer
      };
    } catch (e) {
      if (e.name === 'NoSuchKey') {
        if (await this._isNotEmptyDir(file)) {
          const error = new Error(`${file} is a directory`);
          error.code = 'EISDIR';
          error.path = file;
          throw error;
        }
      }
      throw e;
    }
  }
  async _rmdir(path) {
    if (await this._isNotEmptyDir(path)) {
      const error = new Error(`ENOTEMPTY: directory not empty, rmdir '${path}`);
      error.code = 'ENOTEMPTY';
      error.path = path;
      throw error;
    }
  }
  async _rmtree(path) {
    let NextContinuationToken;
    const Prefix = this._makePrefix(path);
    do {
      const result = await this._s3.send(new _clientS.ListObjectsV2Command({
        Bucket: this._bucket,
        Prefix,
        ContinuationToken: NextContinuationToken
      }));
      NextContinuationToken = result.IsTruncated ? result.NextContinuationToken : undefined;
      await (0, _asyncEach.asyncEach)(result.Contents ?? [], async ({
        Key
      }) => {
        await this._s3.send(new _clientS.DeleteObjectCommand({
          Bucket: this._bucket,
          Key
        }));
      }, {
        concurrency: 16
      });
    } while (NextContinuationToken !== undefined);
  }
  async _openFile(path, flags) {
    return path;
  }
  async _closeFile(fd) {}
  useVhdDirectory() {
    return true;
  }
}, (_applyDecoratedDescriptor(_class.prototype, "_writeFile", [_dec], Object.getOwnPropertyDescriptor(_class.prototype, "_writeFile"), _class.prototype)), _class));
exports.default = S3Handler;
//# sourceMappingURL=s3.js.map