"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _assert = _interopRequireDefault(require("assert"));
var _getFromMap = _interopRequireDefault(require("../../util/get-from-map"));
var _action = _interopRequireDefault(require("../action/action"));
var _openflow = _interopRequireDefault(require("../openflow-11"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const SIZES = {
  [_openflow.default.actionType.output]: _openflow.default.sizes.actionOutput,
  [_openflow.default.actionType.group]: _openflow.default.sizes.actionGroup,
  [_openflow.default.actionType.setVlanId]: _openflow.default.sizes.actionVlanId,
  [_openflow.default.actionType.setVlanPcp]: _openflow.default.sizes.actionVlanPcp,
  [_openflow.default.actionType.setDlSrc]: _openflow.default.sizes.actionDlAddr,
  [_openflow.default.actionType.setDlDst]: _openflow.default.sizes.actionDlAddr,
  [_openflow.default.actionType.setNwSrc]: _openflow.default.sizes.actionNwAddr,
  [_openflow.default.actionType.setNwDst]: _openflow.default.sizes.actionNwAddr,
  [_openflow.default.actionType.setNwTos]: _openflow.default.sizes.actionNwTos,
  [_openflow.default.actionType.setNwEcn]: _openflow.default.sizes.actionNwEcn,
  [_openflow.default.actionType.setTpSrc]: _openflow.default.sizes.actionTpPort,
  [_openflow.default.actionType.setTpDst]: _openflow.default.sizes.actionTpPort,
  [_openflow.default.actionType.copyTtlOut]: _openflow.default.sizes.actionHeader,
  [_openflow.default.actionType.copyTtlIn]: _openflow.default.sizes.actionHeader,
  [_openflow.default.actionType.setMplsLabel]: _openflow.default.sizes.actionMplsLabel,
  [_openflow.default.actionType.setMplsTc]: _openflow.default.sizes.actionMplsTc,
  [_openflow.default.actionType.setMplsTtl]: _openflow.default.sizes.actionMplsTtl,
  [_openflow.default.actionType.decMplsTtl]: _openflow.default.sizes.actionMplsTtl,
  [_openflow.default.actionType.pushVlan]: _openflow.default.sizes.actionPush,
  [_openflow.default.actionType.popVlan]: _openflow.default.sizes.actionHeader,
  [_openflow.default.actionType.pushMpls]: _openflow.default.sizes.actionPush,
  [_openflow.default.actionType.popMpls]: _openflow.default.sizes.actionPopMpls,
  [_openflow.default.actionType.setQueue]: _openflow.default.sizes.actionSetQueue,
  [_openflow.default.actionType.setNwTtl]: _openflow.default.sizes.actionNwTtl,
  [_openflow.default.actionType.decNwTtl]: _openflow.default.sizes.actionNwTtl
};
const TYPES = [_openflow.default.instructionType.clearActions, _openflow.default.instructionType.writeActions, _openflow.default.instructionType.applyActions];
const OFFSETS = _openflow.default.offsets.instructionActions;
const PAD_LENGTH = 4;
var _default = {
  pack: (object, buffer = undefined, offset = 0) => {
    const {
      type
    } = object;
    (0, _assert.default)(TYPES.includes(type));
    object.len = _openflow.default.sizes.instructionActions;
    const {
      actions = []
    } = object;
    actions.forEach(action => {
      (0, _assert.default)(Object.values(_openflow.default.actionType).includes(action.type));
      object.len += (0, _getFromMap.default)(SIZES, action.type, `Invalid action type: ${action.type}`);
    });
    buffer = buffer !== undefined ? buffer : Buffer.alloc(object.len);
    buffer.writeUInt16BE(type, offset + OFFSETS.type);
    buffer.writeUInt16BE(object.len, offset + OFFSETS.len);
    buffer.fill(0, offset + OFFSETS.pad, offset + OFFSETS.pad + PAD_LENGTH);
    let actionOffset = offset + OFFSETS.actions;
    actions.forEach(action => {
      _action.default.pack(action, buffer, actionOffset);
      actionOffset += SIZES[action.type];
    });
  },
  unpack: (buffer = undefined, offset = 0) => {
    const type = buffer.readUInt16BE(offset + OFFSETS.type);
    (0, _assert.default)(TYPES.includes(type));
    const object = {
      type
    };
    object.len = buffer.readUInt16BE(offset + OFFSETS.len);
    if (type === _openflow.default.instructionType.clearActions) {
      return object;
    }
    object.actions = [];
    let actionOffset = offset + OFFSETS.actions;
    while (actionOffset < object.len) {
      const action = _action.default.unpack(buffer, actionOffset);
      actionOffset += action.len;
      object.actions.push(action);
    }
    return object;
  }
};
exports.default = _default;
//# sourceMappingURL=actions.js.map