"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _assert = _interopRequireDefault(require("assert"));
var _getFromMap = _interopRequireDefault(require("../../util/get-from-map"));
var _header = _interopRequireDefault(require("./header"));
var _openflow = _interopRequireDefault(require("../openflow-11"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const ERROR_CODE = {
  [_openflow.default.errorType.helloFailed]: _openflow.default.helloFailedCode,
  [_openflow.default.errorType.badRequest]: _openflow.default.badRequestCode,
  [_openflow.default.errorType.badAction]: _openflow.default.badActionCode,
  [_openflow.default.errorType.badInstruction]: _openflow.default.badInstructionCode,
  [_openflow.default.errorType.badMatch]: _openflow.default.badMatchCode,
  [_openflow.default.errorType.flowModFailed]: _openflow.default.flowModFailedCode,
  [_openflow.default.errorType.groupModFailed]: _openflow.default.groupModFailedCode,
  [_openflow.default.errorType.portModFailed]: _openflow.default.portModFailedCode,
  [_openflow.default.errorType.tableModFailed]: _openflow.default.tableModFailedCode,
  [_openflow.default.errorType.queueOpFailed]: _openflow.default.queueOpFailedCode,
  [_openflow.default.errorType.switchConfigFailed]: _openflow.default.switchConfigFailedCode
};
const OFFSETS = _openflow.default.offsets.errorMsg;
var _default = {
  pack: (object, buffer = undefined, offset = 0) => {
    const {
      header,
      type,
      code,
      data
    } = object;
    (0, _assert.default)(header.type === _openflow.default.type.error);
    const errorCodes = (0, _getFromMap.default)(ERROR_CODE, type, `Invalid error type: ${type}`);
    (0, _assert.default)(Object.values(errorCodes).includes(code));
    object.length = _openflow.default.sizes.errorMsg;
    if (data !== undefined) {
      object.length += data.length;
    }
    buffer = buffer !== undefined ? buffer : Buffer.alloc(object.length);
    _header.default.pack(header, buffer, offset + OFFSETS.header);
    buffer.writeUInt16BE(type, offset + OFFSETS.type);
    buffer.writeUInt16BE(code, offset + OFFSETS.code);
    if (data !== undefined) {
      data.copy(buffer, offset + OFFSETS.data, 0, data.length);
    }
    return buffer;
  },
  unpack: (buffer, offset = 0) => {
    const header = _header.default.unpack(buffer, offset + OFFSETS.header);
    (0, _assert.default)(header.type === _openflow.default.type.error);
    const type = buffer.readUInt16BE(offset + OFFSETS.type);
    const errorCodes = (0, _getFromMap.default)(ERROR_CODE, type, `Invalid error type: ${type}`);
    const code = buffer.readUInt16BE(offset + OFFSETS.code);
    (0, _assert.default)(Object.values(errorCodes).includes(code));
    const object = {
      header,
      type,
      code
    };
    const dataSize = header.length - _openflow.default.sizes.errorMsg;
    if (dataSize > 0) {
      object.data = Buffer.alloc(dataSize);
      buffer.copy(object.data, 0, offset + OFFSETS.data, offset + OFFSETS.data + dataSize);
    }
    return object;
  }
};
exports.default = _default;
//# sourceMappingURL=error.js.map