"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _assert = _interopRequireDefault(require("assert"));
var _header = _interopRequireDefault(require("./header"));
var _port = _interopRequireDefault(require("../struct/port"));
var _openflow = _interopRequireDefault(require("../openflow-11"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const OFFSETS = _openflow.default.offsets.switchFeatures;
const PAD_LENGTH = 3;
var _default = {
  pack: object => {
    const {
      header,
      datapath_id: did,
      n_buffers: nBufs,
      n_tables: nTables,
      capabilities,
      reserved,
      ports
    } = object;
    (0, _assert.default)(header.type === _openflow.default.type.featuresReply);
    header.length = _openflow.default.sizes.switchFeatures + ports.length * _openflow.default.sizes.port;
    const buffer = Buffer.alloc(header.length);
    _header.default.pack(header, buffer, OFFSETS.header);
    buffer.writeBigUInt64BE(did, OFFSETS.datapathId);
    buffer.writeUInt32BE(nBufs, OFFSETS.nBuffers);
    buffer.writeUInt8(nTables, OFFSETS.nTables);
    buffer.fill(0, OFFSETS.pad, OFFSETS.pad + PAD_LENGTH);
    buffer.writeUInt32BE(capabilities, OFFSETS.capabilities);
    buffer.writeUInt32BE(reserved, OFFSETS.reserved);
    let portsOffset = 0;
    ports.forEach(port => {
      _port.default.pack(port, buffer, OFFSETS.ports + portsOffset++ * _openflow.default.sizes.port);
    });
    return buffer;
  },
  unpack: (buffer, offset = 0) => {
    const header = _header.default.unpack(buffer, offset + OFFSETS.header);
    (0, _assert.default)(header.type === _openflow.default.type.featuresReply);
    const object = {
      header
    };
    object.datapath_id = buffer.toString('hex', offset + OFFSETS.datapathId, offset + OFFSETS.datapathId + 8);
    object.n_buffers = buffer.readUInt32BE(offset + OFFSETS.nBuffers);
    object.n_tables = buffer.readUInt8(offset + OFFSETS.nTables);
    object.capabilities = buffer.readUInt32BE(offset + OFFSETS.capabilities);
    object.reserved = buffer.readUInt32BE(offset + OFFSETS.reserved);
    object.ports = [];
    const nPorts = (header.length - _openflow.default.sizes.switchFeatures) / _openflow.default.sizes.port;
    for (let i = 0; i < nPorts; ++i) {
      object.ports.push(_port.default.unpack(buffer, offset + OFFSETS.ports + i * _openflow.default.sizes.port));
    }
    return object;
  }
};
exports.default = _default;
//# sourceMappingURL=features-reply.js.map