"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _assert = _interopRequireDefault(require("assert"));
var _getFromMap = _interopRequireDefault(require("../../util/get-from-map"));
var _instruction = _interopRequireDefault(require("../instruction/instruction"));
var _uintHelper = _interopRequireDefault(require("../../util/uint-helper"));
var _header = _interopRequireDefault(require("./header"));
var _openflow = _interopRequireDefault(require("../openflow-11"));
var _match = _interopRequireDefault(require("../struct/match/match"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const INSTRUCTION_SIZE = {
  [_openflow.default.instructionType.goToTable]: _openflow.default.sizes.instructionWriteMetadata,
  [_openflow.default.instructionType.writeMetadata]: _openflow.default.sizes.instructionGotoTable,
  [_openflow.default.instructionType.clearActions]: _openflow.default.sizes.instructionActions,
  [_openflow.default.instructionType.writeActions]: _openflow.default.sizes.instructionActions,
  [_openflow.default.instructionType.applyActions]: _openflow.default.sizes.instructionActions
};
const ACTION_SIZE = {
  [_openflow.default.actionType.output]: _openflow.default.sizes.actionOutput,
  [_openflow.default.actionType.group]: _openflow.default.sizes.actionGroup,
  [_openflow.default.actionType.setVlanId]: _openflow.default.sizes.actionVlanId,
  [_openflow.default.actionType.setVlanPcp]: _openflow.default.sizes.actionVlanPcp,
  [_openflow.default.actionType.setDlSrc]: _openflow.default.sizes.actionDlAddr,
  [_openflow.default.actionType.setDlDst]: _openflow.default.sizes.actionDlAddr,
  [_openflow.default.actionType.setNwSrc]: _openflow.default.sizes.actionNwAddr,
  [_openflow.default.actionType.setNwDst]: _openflow.default.sizes.actionNwAddr,
  [_openflow.default.actionType.setNwTos]: _openflow.default.sizes.actionNwTos,
  [_openflow.default.actionType.setNwEcn]: _openflow.default.sizes.actionNwEcn,
  [_openflow.default.actionType.setTpSrc]: _openflow.default.sizes.actionTpPort,
  [_openflow.default.actionType.setTpDst]: _openflow.default.sizes.actionTpPort,
  [_openflow.default.actionType.copyTtlOut]: _openflow.default.sizes.actionHeader,
  [_openflow.default.actionType.copyTtlIn]: _openflow.default.sizes.actionHeader,
  [_openflow.default.actionType.setMplsLabel]: _openflow.default.sizes.actionMplsLabel,
  [_openflow.default.actionType.setMplsTc]: _openflow.default.sizes.actionMplsTc,
  [_openflow.default.actionType.setMplsTtl]: _openflow.default.sizes.actionMplsTtl,
  [_openflow.default.actionType.decMplsTtl]: _openflow.default.sizes.actionMplsTtl,
  [_openflow.default.actionType.pushVlan]: _openflow.default.sizes.actionPush,
  [_openflow.default.actionType.popVlan]: _openflow.default.sizes.actionHeader,
  [_openflow.default.actionType.pushMpls]: _openflow.default.sizes.actionPush,
  [_openflow.default.actionType.popMpls]: _openflow.default.sizes.actionPopMpls,
  [_openflow.default.actionType.setQueue]: _openflow.default.sizes.actionSetQueue,
  [_openflow.default.actionType.setNwTtl]: _openflow.default.sizes.actionNwTtl,
  [_openflow.default.actionType.decNwTtl]: _openflow.default.sizes.actionNwTtl
};
const OFFSETS = _openflow.default.offsets.flowMod;
const COOKIE_LENGTH = 8;
const PAD_LENGTH = 2;
var _default = {
  pack: (object, buffer = undefined, offset = 0) => {
    const {
      header,
      cookie,
      cookie_mask,
      table_id = 0,
      command,
      idle_timeout = 0,
      hard_timeout = 0,
      priority = _openflow.default.defaultPriority,
      buffer_id = 0xffffffff,
      out_port = _openflow.default.port.any,
      out_group = _openflow.default.group.any,
      flags = 0,
      match,
      instructions = []
    } = object;
    header.length = _openflow.default.sizes.flowMod;
    instructions.forEach(instruction => {
      header.length += (0, _getFromMap.default)(INSTRUCTION_SIZE, instruction.type, `Invalid instruction type: ${instruction.type}`);
      const {
        actions = []
      } = instruction;
      actions.forEach(action => {
        header.length += (0, _getFromMap.default)(ACTION_SIZE, action.type, `Invalid instruction type: ${action.type}`);
      });
    });
    buffer = buffer !== undefined ? buffer : Buffer.alloc(header.length);
    _header.default.pack(header, buffer, offset + OFFSETS.header);
    if (cookie !== undefined) {
      if (cookie_mask !== undefined) {
        cookie_mask.copy(buffer, offset + OFFSETS.cookieMask);
      } else {
        buffer.fill(0x00, offset + OFFSETS.cookie_mask, offset + OFFSETS.cookieMask + COOKIE_LENGTH);
      }
      cookie.copy(buffer, offset + OFFSETS.cookie);
    } else {
      buffer.fill(0x00, offset + OFFSETS.cookie, offset + OFFSETS.cookie + COOKIE_LENGTH);
      buffer.fill(0xff, offset + OFFSETS.cookieMask, offset + OFFSETS.cookieMask + COOKIE_LENGTH);
    }
    buffer.writeUInt8(table_id, offset + OFFSETS.tableId);
    (0, _assert.default)(Object.values(_openflow.default.flowModCommand).includes(command));
    buffer.writeUInt8(command, offset + OFFSETS.command);
    buffer.writeUInt16BE(idle_timeout, offset + OFFSETS.idleTimeout);
    buffer.writeUInt16BE(hard_timeout, offset + OFFSETS.hardTimeout);
    buffer.writeUInt16BE(priority, offset + OFFSETS.priority);
    buffer.writeUInt32BE(buffer_id, offset + OFFSETS.bufferId);
    buffer.writeUInt32BE(out_port, offset + OFFSETS.outPort);
    buffer.writeUInt32BE(out_group, offset + OFFSETS.outGroup);
    buffer.writeUInt16BE(flags, offset + OFFSETS.flags);
    buffer.fill(0, offset + OFFSETS.pad, offset + OFFSETS.pad + PAD_LENGTH);
    _match.default.pack(match, buffer, offset + OFFSETS.match);
    let instructionOffset = offset + OFFSETS.instructions;
    instructions.forEach(instruction => {
      _instruction.default.pack(instruction, buffer, instructionOffset);
      instructionOffset += instruction.len;
    });
    return buffer;
  },
  unpack: (buffer, offset = 0) => {
    const header = _header.default.unpack(buffer, offset + OFFSETS.header);
    (0, _assert.default)(header.type === _openflow.default.type.flowMod);
    const object = {
      header
    };
    object.cookie = Buffer.alloc(COOKIE_LENGTH);
    buffer.copy(object.cookie, 0, offset + OFFSETS.cookie, offset + OFFSETS.cookie + COOKIE_LENGTH);
    if (!_uintHelper.default.isUInt64None([buffer.readUInt32BE(offset + OFFSETS.cookieMask), buffer.readUInt32BE(offset + OFFSETS.cookieMask + COOKIE_LENGTH / 2)])) {
      object.cookie_mask = Buffer.alloc(COOKIE_LENGTH);
      buffer.copy(object.cookie_mask, 0, offset + OFFSETS.cookieMask, offset + OFFSETS.cookieMask + COOKIE_LENGTH);
    }
    object.table_id = buffer.readUInt8(offset + OFFSETS.tableId);
    object.command = buffer.readUInt8(offset + OFFSETS.command);
    (0, _assert.default)(Object.values(_openflow.default.flowModCommand).includes(object.command));
    object.idle_timeout = buffer.readUInt16BE(offset + OFFSETS.idleTimeout);
    object.hard_timeout = buffer.readUInt16BE(offset + OFFSETS.hardTimeout);
    object.priority = buffer.readUInt16BE(offset + OFFSETS.priority);
    object.buffer_id = buffer.readUInt32BE(offset + OFFSETS.bufferId);
    object.out_port = buffer.readUInt32BE(offset + OFFSETS.outPort);
    object.out_group = buffer.readUInt32BE(offset + OFFSETS.outGroup);
    object.flags = buffer.readUInt16BE(offset + OFFSETS.flags);
    object.match = _match.default.unpack(buffer, offset + OFFSETS.match);
    object.instructions = [];
    let instructionOffset = offset + OFFSETS.instructions;
    while (instructionOffset < header.length) {
      const instruction = _instruction.default.unpack(buffer, instructionOffset);
      object.instructions.push(instruction);
      instructionOffset += instruction.len;
    }
    return object;
  }
};
exports.default = _default;
//# sourceMappingURL=flow-mod.js.map