import { TokenBucket } from 'limiter';
import { Throttle } from './throttle';
export interface ThrottleOptions {
    rate: number;
    burst?: number;
    chunkSize?: number;
}
export declare class ThrottleGroup {
    private _rate;
    private _burst;
    private _bucket;
    private _chunkSize;
    get rate(): number;
    get burst(): number;
    get bucket(): TokenBucket;
    get chunkSize(): number;
    constructor(options: ThrottleOptions);
    /**
     * Update the sustained rate of the throttle.
     * Calling this method will recalculate the chunk size.
     * In case the the burst rate is less than the sustained rate,
     * the burst rate will be set to the updated sustained rate.
     * @param rate The rate of the throttle in bytes per second
     */
    setRate(rate: number): void;
    /**
     * Update the burst rate (also known as bucket size) of the throttle.
     * @param burst The burst value of the throttle in bytes per second
     * @throws {Error} If the burst value is less than the rate
     */
    setBurst(burst: number): void;
    /**
     * Update the processing chunk size of the throttle.
     * @param chunkSize The chunk size in bytes that received chunks will be split into
     */
    setChunkSize(chunkSize: number): void;
    createThrottle(): Throttle;
    private calculateChunkSize;
}
