"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Throttle = void 0;
const node_stream_1 = require("node:stream");
const group_1 = require("./group");
class Throttle extends node_stream_1.Transform {
    constructor(opts) {
        super();
        if (opts instanceof group_1.ThrottleGroup) {
            this._group = opts;
        }
        else {
            this._group = new group_1.ThrottleGroup(opts);
        }
    }
    get group() {
        return this._group;
    }
    _transform(chunk, _encoding, callback) {
        this._process(chunk, 0, callback);
    }
    _process(chunk, position, callback) {
        const newPosition = position + this._group.chunkSize;
        const slice = chunk.slice(position, newPosition);
        if (!slice.length) {
            callback();
            return;
        }
        this.group.bucket
            .removeTokens(slice.length)
            .then(() => {
            this.push(slice);
            this._process(chunk, newPosition, callback);
        })
            .catch((err) => {
            callback(err);
        });
    }
}
exports.Throttle = Throttle;
//# sourceMappingURL=throttle.js.map