#!/usr/bin/env node
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _blocked = _interopRequireDefault(require("blocked"));
var _debug = _interopRequireDefault(require("debug"));
var _filter = _interopRequireDefault(require("lodash/filter"));
var _find = _interopRequireDefault(require("lodash/find"));
var _lodash = _interopRequireDefault(require("lodash"));
var _minimist = _interopRequireDefault(require("minimist"));
var _pw = _interopRequireDefault(require("pw"));
var _promiseToolbox = require("promise-toolbox");
var _jestDiff = require("jest-diff");
var _bindPropertyDescriptor = require("bind-property-descriptor");
var _repl = require("repl");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function askPassword(prompt = 'Password: ') {
  if (prompt) {
    process.stdout.write(prompt);
  }
  return new Promise(resolve => {
    (0, _pw.default)(resolve);
  });
}
const {
  getPrototypeOf,
  ownKeys
} = Reflect;
function getAllBoundDescriptors(object) {
  const descriptors = {
    __proto__: null
  };
  let current = object;
  do {
    ownKeys(current).forEach(key => {
      if (!(key in descriptors)) {
        descriptors[key] = (0, _bindPropertyDescriptor.getBoundPropertyDescriptor)(current, key, object);
      }
    });
  } while ((current = getPrototypeOf(current)) !== null);
  return descriptors;
}
const usage = 'Usage: xen-api <url> [<user> [<password>]]';
async function main(createClient) {
  const opts = (0, _minimist.default)(process.argv.slice(2), {
    string: ['proxy', 'session-id'],
    boolean: ['allow-unauthorized', 'help', 'read-only', 'verbose'],
    alias: {
      'allow-unauthorized': 'au',
      debounce: 'd',
      help: 'h',
      proxy: 'p',
      'read-only': 'ro',
      verbose: 'v'
    }
  });
  if (opts.help) {
    return usage;
  }
  if (opts.verbose) {
    _debug.default.enable('xen-api,xen-api:*');
  }
  let auth;
  if (opts._.length > 1) {
    const [, user, password = await askPassword()] = opts._;
    auth = {
      user,
      password
    };
  } else if (opts['session-id'] !== undefined) {
    auth = {
      sessionId: opts['session-id']
    };
  }
  {
    const debug = (0, _debug.default)('xen-api:perf');
    (0, _blocked.default)(ms => {
      debug('blocked for %sms', ms | 0);
    });
  }
  const xapi = createClient({
    url: opts._[0],
    allowUnauthorized: opts.au,
    auth,
    debounce: opts.debounce != null ? +opts.debounce : null,
    httpProxy: opts.proxy,
    readOnly: opts.ro,
    syncStackTraces: true
  });
  await xapi.connect();
  const repl = (0, _repl.start)({
    prompt: `${xapi._humanId}> `
  });
  {
    const ctx = repl.context;
    ctx.xapi = xapi;
    ctx.diff = (a, b) => console.log('%s', (0, _jestDiff.diff)(a, b));
    ctx.find = predicate => (0, _find.default)(xapi.objects.all, predicate);
    ctx.findAll = predicate => (0, _filter.default)(xapi.objects.all, predicate);
    ctx.L = _lodash.default;
    Object.defineProperties(ctx, getAllBoundDescriptors(xapi));
  }
  repl.eval = (evaluate => (cmd, context, filename, cb) => {
    _promiseToolbox.asCallback.call((0, _promiseToolbox.fromCallback)(cb => {
      evaluate.call(repl, cmd, context, filename, cb);
    }).then(value => Array.isArray(value) ? Promise.all(value) : value), cb);
  })(repl.eval);
  await (0, _promiseToolbox.fromEvent)(repl, 'exit');
  try {
    await xapi.disconnect();
  } catch (error) {}
}
var _default = main;
exports.default = _default;
if (!module.parent) {
  main(require('./').createClient).catch(console.error.bind(console, 'FATAL'));
}
//# sourceMappingURL=cli.js.map