"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "Ref", {
  enumerable: true,
  get: function () {
    return _Ref.default;
  }
});
exports.createClient = exports.Xapi = void 0;
var _assert = _interopRequireDefault(require("assert"));
var _dns = _interopRequireDefault(require("dns"));
var _kindof = _interopRequireDefault(require("kindof"));
var _ms = _interopRequireDefault(require("ms"));
var _httpRequestPlus = _interopRequireDefault(require("http-request-plus"));
var _map = _interopRequireDefault(require("lodash/map"));
var _noop = _interopRequireDefault(require("lodash/noop"));
var _proxyAgent = _interopRequireDefault(require("proxy-agent"));
var _coalesceCalls = require("@vates/coalesce-calls");
var _xoCollection = require("xo-collection");
var _events = require("events");
var _index = require("xo-collection/index");
var _promiseToolbox = require("promise-toolbox");
var _limitConcurrencyDecorator = require("limit-concurrency-decorator");
var _debug = _interopRequireDefault(require("./_debug"));
var _getTaskResult = _interopRequireDefault(require("./_getTaskResult"));
var _isGetAllRecordsMethod = _interopRequireDefault(require("./_isGetAllRecordsMethod"));
var _isReadOnlyCall = _interopRequireDefault(require("./_isReadOnlyCall"));
var _makeCallSetting = _interopRequireDefault(require("./_makeCallSetting"));
var _parseUrl = _interopRequireDefault(require("./_parseUrl"));
var _Ref = _interopRequireDefault(require("./_Ref"));
var _replaceSensitiveValues = _interopRequireDefault(require("./_replaceSensitiveValues"));
var _transports = _interopRequireDefault(require("./transports"));
var _class;
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) { var desc = {}; Object.keys(descriptor).forEach(function (key) { desc[key] = descriptor[key]; }); desc.enumerable = !!desc.enumerable; desc.configurable = !!desc.configurable; if ('value' in desc || desc.initializer) { desc.writable = true; } desc = decorators.slice().reverse().reduce(function (desc, decorator) { return decorator(target, property, desc) || desc; }, desc); if (context && desc.initializer !== void 0) { desc.value = desc.initializer ? desc.initializer.call(context) : void 0; desc.initializer = undefined; } if (desc.initializer === void 0) { Object.defineProperty(target, property, desc); desc = null; } return desc; }
const EVENT_TIMEOUT = 60;
const {
  defineProperties,
  defineProperty,
  freeze,
  keys: getKeys
} = Object;
const RESERVED_FIELDS = {
  id: true,
  pool: true,
  ref: true,
  type: true,
  xapi: true
};
function getPool() {
  return this.$xapi.pool;
}
const CONNECTED = 'connected';
const CONNECTING = 'connecting';
const DISCONNECTED = 'disconnected';
const BARRIER_PREFIX = 'xo:barrier:';
const BARRIER_MAX_AGE = 60 * 60 * 1e3;
const identity = value => value;
const addSyncStackTrace = async promise => {
  const stackContainer = new Error();
  try {
    return await promise;
  } catch (error) {
    let {
      stack
    } = stackContainer;
    stack = stack.slice(stack.indexOf('\n') + 1);
    error.stack = [error.stack, 'From:', stack].join('\n');
    throw error;
  }
};
let Xapi = (_class = class Xapi extends _events.EventEmitter {
  constructor(opts) {
    var _opts$syncStackTraces, _opts$httpInactivityT, _opts$eventPollDelay, _opts$ignorePremature, _opts$reverseHostIpAd, _opts$callConcurrency, _opts$debounce;
    super();
    _defineProperty(this, "connect", (0, _coalesceCalls.coalesceCalls)(this.connect));
    _defineProperty(this, "_sessionCallRetryOptions", {
      tries: 2,
      when: error => this._status !== DISCONNECTED && (error === null || error === void 0 ? void 0 : error.code) === 'SESSION_INVALID' && this._auth.password !== undefined,
      onRetry: () => this._sessionOpen()
    });
    _defineProperty(this, "_sessionOpenRetryOptions", {
      tries: 2,
      when: {
        code: 'HOST_IS_SLAVE'
      },
      onRetry: error => {
        this._setUrl({
          ...this._url,
          hostname: error.params[0]
        });
      }
    });
    _defineProperty(this, "_sessionOpen", (0, _coalesceCalls.coalesceCalls)(this._sessionOpen));
    _defineProperty(this, "_watchEvents", (0, _coalesceCalls.coalesceCalls)(this._watchEvents));
    const {
      transport = 'auto'
    } = opts;
    const createTransport = _transports.default[transport];
    if (createTransport === undefined) {
      throw new Error('invalid transport: ' + transport);
    }
    this._createTransport = createTransport;
    this._addSyncStackTrace = ((_opts$syncStackTraces = opts.syncStackTraces) !== null && _opts$syncStackTraces !== void 0 ? _opts$syncStackTraces : process.env.NODE_ENV === 'development') ? addSyncStackTrace : identity;
    this._callTimeout = (0, _makeCallSetting.default)(opts.callTimeout, 60 * 60 * 1e3);
    this._httpInactivityTimeout = (_opts$httpInactivityT = opts.httpInactivityTimeout) !== null && _opts$httpInactivityT !== void 0 ? _opts$httpInactivityT : 5 * 60 * 1e3;
    this._eventPollDelay = (_opts$eventPollDelay = opts.eventPollDelay) !== null && _opts$eventPollDelay !== void 0 ? _opts$eventPollDelay : 60 * 1e3;
    this._ignorePrematureClose = (_opts$ignorePremature = opts.ignorePrematureClose) !== null && _opts$ignorePremature !== void 0 ? _opts$ignorePremature : true;
    this._pool = null;
    this._readOnly = Boolean(opts.readOnly);
    this._RecordsByType = {
      __proto__: null
    };
    this._reverseHostIpAddresses = (_opts$reverseHostIpAd = opts.reverseHostIpAddresses) !== null && _opts$reverseHostIpAd !== void 0 ? _opts$reverseHostIpAd : false;
    this._call = (0, _limitConcurrencyDecorator.limitConcurrency)((_opts$callConcurrency = opts.callConcurrency) !== null && _opts$callConcurrency !== void 0 ? _opts$callConcurrency : 20)(this._call);
    this._roCallRetryOptions = {
      delay: 1e3,
      tries: 10,
      ...opts.roCallRetryOptions,
      when: {
        code: 'ECONNRESET'
      }
    };
    this._auth = opts.auth;
    const url = (0, _parseUrl.default)(opts.url);
    if (this._auth === undefined) {
      const user = url.username;
      if (user === undefined) {
        throw new TypeError('missing credentials');
      }
      this._auth = {
        user,
        password: url.password
      };
      delete url.username;
      delete url.password;
    }
    this._allowUnauthorized = opts.allowUnauthorized;
    let {
      httpProxy
    } = opts;
    if (httpProxy !== undefined) {
      if (httpProxy.startsWith('https:')) {
        httpProxy = (0, _parseUrl.default)(httpProxy);
        httpProxy.rejectUnauthorized = !opts.allowUnauthorized;
      }
      this._httpAgent = new _proxyAgent.default(httpProxy);
    }
    this._setUrl(url);
    this._connected = new Promise(resolve => {
      this._resolveConnected = resolve;
    });
    this._disconnected = Promise.resolve();
    this._sessionId = undefined;
    this._status = DISCONNECTED;
    this._watchEventsError = undefined;
    this._lastEventFetchedTimestamp = undefined;
    const objects = new _xoCollection.Collection();
    objects.createIndex('type', new _index.Index('$type'));
    this._objects = objects;
    this._debounce = (_opts$debounce = opts.debounce) !== null && _opts$debounce !== void 0 ? _opts$debounce : 200;
    this._objectsByRef = {
      __proto__: null
    };
    this._objectsFetched = new Promise(resolve => {
      this._resolveObjectsFetched = resolve;
    });
    this._eventWatchers = {
      __proto__: null
    };
    this._taskWatchers = undefined;
    this._watchedTypes = undefined;
    const {
      watchEvents
    } = opts;
    if (watchEvents !== false) {
      if (Array.isArray(watchEvents)) {
        this._watchedTypes = watchEvents;
      }
      this.watchEvents();
    }
  }
  get httpAgent() {
    return this._httpAgent;
  }
  get readOnly() {
    return this._readOnly;
  }
  set readOnly(ro) {
    this._readOnly = Boolean(ro);
  }
  get connected() {
    return this._connected;
  }
  get disconnected() {
    return this._disconnected;
  }
  get pool() {
    return this._pool;
  }
  get sessionId() {
    _assert.default.strictEqual(this._status, CONNECTED);
    return this._sessionId;
  }
  get status() {
    return this._status;
  }
  async connect() {
    const status = this._status;
    if (status === CONNECTED) {
      return;
    }
    _assert.default.strictEqual(status, DISCONNECTED);
    this._status = CONNECTING;
    this._disconnected = new Promise(resolve => {
      this._resolveDisconnected = resolve;
    });
    try {
      await this._sessionOpen();
      (0, _debug.default)('%s: connected', this._humanId);
      this._status = CONNECTED;
      this._resolveConnected();
      this._resolveConnected = undefined;
      this.emit(CONNECTED);
    } catch (error) {
      _promiseToolbox.ignoreErrors.call(this.disconnect());
      throw error;
    }
  }
  async disconnect() {
    const status = this._status;
    if (status === DISCONNECTED) {
      return;
    }
    if (status === CONNECTED) {
      this._connected = new Promise(resolve => {
        this._resolveConnected = resolve;
      });
    } else {
      _assert.default.strictEqual(status, CONNECTING);
    }
    const sessionId = this._sessionId;
    if (sessionId !== undefined) {
      this._sessionId = undefined;
      _promiseToolbox.ignoreErrors.call(this._call('session.logout', [sessionId]));
    }
    (0, _debug.default)('%s: disconnected', this._humanId);
    this._status = DISCONNECTED;
    this._resolveDisconnected();
    this._resolveDisconnected = undefined;
    this.emit(DISCONNECTED);
  }
  call(method, ...args) {
    return (0, _isReadOnlyCall.default)(method, args) ? this._roCall(method, args) : this._readOnly ? Promise.reject(new Error(`cannot call ${method}() in read only mode`)) : this._sessionCall(method, args);
  }
  async callAsync($cancelToken, method, ...args) {
    if (this._readOnly && !(0, _isReadOnlyCall.default)(method, args)) {
      throw new Error(`cannot call ${method}() in read only mode`);
    }
    const taskRef = await this._sessionCall(`Async.${method}`, args);
    $cancelToken.promise.then(() => _promiseToolbox.ignoreErrors.call(this._sessionCall('task.cancel', [taskRef])));
    const promise = this.watchTask(taskRef);
    const destroyTask = () => _promiseToolbox.ignoreErrors.call(this._sessionCall('task.destroy', [taskRef]));
    promise.then(destroyTask, destroyTask);
    return promise;
  }
  async getAllRecords(type) {
    return (0, _map.default)(await this._roCall(`${type}.get_all_records`), (record, ref) => this._wrapRecord(type, ref, record));
  }
  async getRecord(type, ref) {
    return this._wrapRecord(type, ref, await this._roCall(`${type}.get_record`, [ref]));
  }
  async getRecordByUuid(type, uuid) {
    return this.getRecord(type, await this._roCall(`${type}.get_by_uuid`, [uuid]));
  }
  getRecords(type, refs) {
    return Promise.all(refs.map(ref => this.getRecord(type, ref)));
  }
  getField(type, ref, field) {
    return this._roCall(`${type}.get_${field}`, [ref]);
  }
  setField(type, ref, field, value) {
    return this.call(`${type}.set_${field}`, ref, value).then(_noop.default);
  }
  setFields(type, ref, fields) {
    return Promise.all(getKeys(fields).map(field => {
      const value = fields[field];
      if (value !== undefined) {
        return this.call(`${type}.set_${field}`, ref, value);
      }
    })).then(_noop.default);
  }
  setFieldEntries(type, ref, field, entries) {
    return Promise.all(getKeys(entries).map(entry => {
      const value = entries[entry];
      if (value !== undefined) {
        return this.setFieldEntry(type, ref, field, entry, value);
      }
    })).then(_noop.default);
  }
  async setFieldEntry(type, ref, field, entry, value) {
    if (value === null) {
      return this.call(`${type}.remove_from_${field}`, ref, entry).then(_noop.default);
    }
    while (true) {
      try {
        await this.call(`${type}.add_to_${field}`, ref, entry, value);
        return;
      } catch (error) {
        if ((error === null || error === void 0 ? void 0 : error.code) !== 'MAP_DUPLICATE_KEY') {
          throw error;
        }
      }
      await this.call(`${type}.remove_from_${field}`, ref, entry);
    }
  }
  async getResource($cancelToken, pathname, {
    host,
    query,
    task
  } = {}) {
    const taskRef = await this._autoTask(task, `Xapi#getResource ${pathname}`);
    query = {
      ...query,
      session_id: this.sessionId
    };
    let pTaskResult;
    if (taskRef !== undefined) {
      query.task_id = taskRef;
      pTaskResult = this.watchTask(taskRef);
      if (typeof $cancelToken.addHandler === 'function') {
        $cancelToken.addHandler(() => pTaskResult);
      }
    }
    let url = new URL('http://localhost');
    url.protocol = this._url.protocol;
    url.pathname = pathname;
    url.search = new URLSearchParams(query);
    await this._setHostAddressInUrl(url, host);
    const response = await this._addSyncStackTrace((0, _promiseToolbox.pRetry)(async () => (0, _httpRequestPlus.default)(url, {
      rejectUnauthorized: !this._allowUnauthorized,
      timeout: this._httpInactivityTimeout,
      maxRedirects: 0,
      minVersion: 'TLSv1',
      agent: this.httpAgent,
      signal: $cancelToken
    }), {
      when: error => error.response.statusCode === 302,
      onRetry: async error => {
        const response = error.response;
        if (response === undefined) {
          throw error;
        }
        response.destroy();
        url = await this._replaceHostAddressInUrl(new URL(response.headers.location, url));
      }
    }));
    if (pTaskResult !== undefined) {
      response.task = pTaskResult;
    }
    return response;
  }
  async putResource($cancelToken, body, pathname, {
    host,
    query,
    task
  } = {}) {
    if (this._readOnly) {
      throw new Error('cannot put resource in read only mode');
    }
    const taskRef = await this._autoTask(task, `Xapi#putResource ${pathname}`);
    query = {
      ...query,
      session_id: this.sessionId
    };
    let pTaskResult;
    if (taskRef !== undefined) {
      query.task_id = taskRef;
      pTaskResult = this.watchTask(taskRef);
      if (typeof $cancelToken.addHandler === 'function') {
        $cancelToken.addHandler(() => pTaskResult);
      }
    }
    const headers = {};
    const isStream = typeof body.pipe === 'function';
    const useHack = isStream && body.length === undefined;
    if (useHack) {
      console.warn(this._humanId, 'Xapi#putResource', pathname, 'missing length');
      headers['content-length'] = '1125899906842624';
    }
    const url = new URL('http://localhost');
    url.protocol = this._url.protocol;
    url.pathname = pathname;
    url.search = new URLSearchParams(query);
    await this._setHostAddressInUrl(url, host);
    const doRequest = (url, opts) => (0, _httpRequestPlus.default)(url, {
      agent: this.httpAgent,
      body,
      headers,
      method: 'PUT',
      rejectUnauthorized: !this._allowUnauthorized,
      signal: $cancelToken,
      timeout: this._httpInactivityTimeout,
      minVersion: 'TLSv1',
      ...opts
    });
    const dummyUrl = new URL(url);
    dummyUrl.searchParams.delete('task_id');
    const response = await this._addSyncStackTrace(isStream ? doRequest(dummyUrl, {
      body: '',
      maxRedirects: 0
    }).then(response => {
      response.destroy();
      return doRequest(url);
    }, async error => {
      let response;
      if (error != null && (response = error.response) != null) {
        response.destroy();
        const {
          headers: {
            location
          },
          statusCode
        } = response;
        if (statusCode === 302 && location !== undefined) {
          const newUrl = new URL(location, url);
          newUrl.searchParams.set('task_id', query.task_id);
          return doRequest(await this._replaceHostAddressInUrl(newUrl));
        }
      }
      throw error;
    }) : doRequest(url));
    if (pTaskResult !== undefined) {
      if (useHack) {
        pTaskResult = pTaskResult.catch(error => {
          console.warn(this._humanId, 'Xapi#putResource', pathname, error);
        });
      } else {
        pTaskResult = pTaskResult.catch(error => {
          error.url = response.url;
          throw error;
        });
        pTaskResult.catch(_noop.default);
      }
    }
    try {
      const {
        req
      } = response;
      if (!req.finished) {
        await new Promise((resolve, reject) => {
          req.on('finish', resolve);
          response.on('error', reject);
        });
      }
      if (useHack) {
        response.destroy();
      } else {
        response.resume();
        await new Promise((resolve, reject) => {
          response.on('end', resolve).on('error', reject);
        });
      }
    } catch (error) {
      if (this._ignorePrematureClose && error.code === 'ERR_STREAM_PREMATURE_CLOSE') {
        console.warn(this._humanId, 'Xapi#putResource', pathname, error);
      } else {
        throw error;
      }
    }
    return pTaskResult;
  }
  get objects() {
    return this._objects;
  }
  get objectsFetched() {
    return this._objectsFetched;
  }
  get lastEventFetchedTimestamp() {
    return this._lastEventFetchedTimestamp;
  }
  get watchEventsError() {
    return this._watchEventsError;
  }
  async barrier(ref) {
    const eventWatchers = this._eventWatchers;
    if (eventWatchers === undefined) {
      throw new Error('Xapi#barrier() requires events watching');
    }
    const key = BARRIER_PREFIX + Math.random().toString(36).slice(2);
    const {
      $ref: poolRef,
      other_config
    } = this._pool;
    const {
      promise,
      resolve
    } = (0, _promiseToolbox.defer)();
    eventWatchers[key] = resolve;
    const now = Date.now();
    for (const key of Object.keys(other_config)) {
      if (key.startsWith(BARRIER_PREFIX) && now - other_config[key] > BARRIER_MAX_AGE) {
        _promiseToolbox.ignoreErrors.call(this._sessionCall('pool.remove_from_other_config', [poolRef, key]));
      }
    }
    await this._sessionCall('pool.add_to_other_config', [poolRef, key, String(now)]);
    await this._addSyncStackTrace(promise);
    _promiseToolbox.ignoreErrors.call(this._sessionCall('pool.remove_from_other_config', [poolRef, key]));
    if (ref !== undefined) {
      return this.getObjectByRef(ref);
    }
  }
  async createTask(nameLabel, nameDescription = '') {
    const taskRef = await this._sessionCall('task.create', [nameLabel, nameDescription]);
    const destroyTask = () => _promiseToolbox.ignoreErrors.call(this._sessionCall('task.destroy', [taskRef]));
    this.watchTask(taskRef).then(destroyTask, destroyTask);
    return taskRef;
  }
  getObject(idOrUuidOrRef, defaultValue) {
    if (typeof idOrUuidOrRef === 'object') {
      idOrUuidOrRef = idOrUuidOrRef.$id;
    }
    const object = this._objects.all[idOrUuidOrRef] || this._objectsByRef[idOrUuidOrRef];
    if (object !== undefined) return object;
    if (arguments.length > 1) return defaultValue;
    throw new Error('no object with UUID or opaque ref: ' + idOrUuidOrRef);
  }
  getObjectByRef(ref, defaultValue) {
    const object = this._objectsByRef[ref];
    if (object !== undefined) return object;
    if (arguments.length > 1) return defaultValue;
    throw new Error('no object with opaque ref: ' + ref);
  }
  getObjectByUuid(uuid, defaultValue) {
    const object = this._objects.all[uuid];
    if (object !== undefined) return object;
    if (arguments.length > 1) return defaultValue;
    throw new Error('no object with UUID: ' + uuid);
  }
  watchEvents() {
    _promiseToolbox.ignoreErrors.call(this._watchEvents());
  }
  watchTask(ref) {
    const watchers = this._taskWatchers;
    if (watchers === undefined) {
      throw new Error('Xapi#watchTask() requires events watching');
    }
    if (ref.$ref !== undefined) ref = ref.$ref;
    let watcher = watchers[ref];
    if (watcher === undefined) {
      const task = this._objectsByRef[ref];
      if (task !== undefined) {
        const result = (0, _getTaskResult.default)(task);
        if (result !== undefined) {
          return result;
        }
      }
      watcher = watchers[ref] = (0, _promiseToolbox.defer)();
    }
    return this._addSyncStackTrace(watcher.promise);
  }
  async _call(method, args = [], timeout = this._callTimeout(method, args)) {
    const startTime = Date.now();
    try {
      const result = await _promiseToolbox.pTimeout.call(this._addSyncStackTrace(this._transport(method, args)), timeout);
      (0, _debug.default)('%s: %s(...) [%s] ==> %s', this._humanId, method, (0, _ms.default)(Date.now() - startTime), (0, _kindof.default)(result));
      return result;
    } catch (error) {
      const params = args[0] === this._sessionId ? args.slice(1) : args;
      error.call = {
        method,
        params: method === 'session.login_with_password' ? '* obfuscated *' : (0, _replaceSensitiveValues.default)(params, '* obfuscated *')
      };
      (0, _debug.default)('%s: %s(...) [%s] =!> %s', this._humanId, method, (0, _ms.default)(Date.now() - startTime), error);
      throw error;
    }
  }
  _autoTask(task = this._taskWatchers !== undefined, name) {
    if (task === false) {
      return Promise.resolve();
    }
    if (task === true) {
      return this.createTask(name);
    }
    return Promise.resolve(task);
  }
  _interruptOnDisconnect(promise) {
    let listener;
    const pWrapper = new Promise((resolve, reject) => {
      promise.then(resolve, reject);
      this.on(DISCONNECTED, listener = () => {
        reject(new Error('disconnected'));
      });
    });
    const clean = () => {
      this.removeListener(DISCONNECTED, listener);
    };
    pWrapper.then(clean, clean);
    return pWrapper;
  }
  async _sessionCall(method, args, timeout) {
    if (method.startsWith('session.')) {
      return Promise.reject(new Error('session.*() methods are disabled from this interface'));
    }
    try {
      return await (0, _promiseToolbox.pRetry)(() => {
        const sessionId = this._sessionId;
        _assert.default.notStrictEqual(sessionId, undefined);
        const newArgs = [sessionId];
        if (args !== undefined) {
          newArgs.push.apply(newArgs, args);
        }
        return this._call(method, newArgs, timeout);
      }, this._sessionCallRetryOptions);
    } catch (error) {
      if ((error === null || error === void 0 ? void 0 : error.code) === 'SESSION_INVALID') {
        await _promiseToolbox.ignoreErrors.call(this.disconnect());
      }
      throw error;
    }
  }
  async _sessionOpen() {
    var _this$_pool;
    const {
      user,
      password,
      sessionId
    } = this._auth;
    this._sessionId = sessionId;
    if (sessionId === undefined) {
      const params = [user, password];
      this._sessionId = await (0, _promiseToolbox.pRetry)(() => this._interruptOnDisconnect(this._call('session.login_with_password', params)), this._sessionOpenRetryOptions);
    }
    const oldPoolRef = (_this$_pool = this._pool) === null || _this$_pool === void 0 ? void 0 : _this$_pool.$ref;
    const pools = await (0, _promiseToolbox.pRetry)(() => this._call('pool.get_all_records', [this._sessionId]), this._sessionOpenRetryOptions);
    const poolRef = Object.keys(pools)[0];
    this._pool = this._wrapRecord('pool', poolRef, pools[poolRef]);
    this.emit('sessionId', this._sessionId);
    if (this._pool.$ref !== oldPoolRef) {
      const types = this._types = (await this._interruptOnDisconnect(this._call('system.listMethods'))).filter(_isGetAllRecordsMethod.default).map(method => method.slice(0, method.indexOf('.')));
      this._lcToTypes = {
        __proto__: null
      };
      types.forEach(type => {
        const lcType = type.toLowerCase();
        if (lcType !== type) {
          this._lcToTypes[lcType] = type;
        }
      });
    }
  }
  async _getHostBackupAddress(host) {
    if (host === undefined) {
      host = await this.getRecord('host', this._pool.master);
    }
    let {
      address
    } = host;
    const poolBackupNetwork = this._pool.other_config['xo:backupNetwork'];
    if (poolBackupNetwork !== undefined) {
      const hostPifs = new Set(host.PIFs);
      try {
        const networkRef = await this._roCall('network.get_by_uuid', [poolBackupNetwork]);
        const networkPifs = await this.getField('network', networkRef, 'PIFs');
        const backupNetworkPifRef = networkPifs.find(hostPifs.has, hostPifs);
        address = await this.getField('PIF', backupNetworkPifRef, 'IP');
      } catch (error) {
        console.warn('unable to get the host address linked to the pool backup network', poolBackupNetwork, error);
      }
    }
    if (this._reverseHostIpAddresses) {
      try {
        ;
        [address] = await (0, _promiseToolbox.fromCallback)(_dns.default.reverse, address);
      } catch (error) {
        console.warn('reversing host address', address, error);
      }
    }
    if (host.$ref === this._pool.master && address === host.address) {
      return this._url.hostname;
    }
    return address;
  }
  async getHostBackupUrl(host) {
    return Object.assign(new URL('http://localhost'), {
      ...this._url,
      hostname: await this._getHostBackupAddress(host)
    });
  }
  async _setHostAddressInUrl(url, host) {
    const poolBackupNetwork = this._pool.other_config['xo:backupNetwork'];
    if (host === undefined && poolBackupNetwork === undefined) {
      const xapiUrl = this._url;
      url.hostname = xapiUrl.hostname;
      url.port = xapiUrl.port;
      return;
    }
    url.hostname = await this._getHostBackupAddress(host);
  }
  _setUrl(url) {
    var _this$_auth$user;
    this._humanId = `${(_this$_auth$user = this._auth.user) !== null && _this$_auth$user !== void 0 ? _this$_auth$user : 'unknown'}@${url.hostname}`;
    this._transport = this._createTransport({
      secureOptions: {
        minVersion: 'TLSv1',
        rejectUnauthorized: !this._allowUnauthorized
      },
      url,
      agent: this.httpAgent
    });
    this._url = url;
  }
  _addRecordToCache(type, ref, object) {
    object = this._wrapRecord(type, ref, object);
    freeze(object);
    const objects = this._objects;
    const objectsByRef = this._objectsByRef;
    const prev = objectsByRef[ref];
    let prevUuid;
    if (prev !== undefined && (prevUuid = prev.uuid) !== undefined && prevUuid !== object.uuid) {
      objects.remove(prevUuid);
    }
    this._objects.set(object.$id, object);
    objectsByRef[ref] = object;
    if (type === 'pool') {
      this._pool = object;
      const eventWatchers = this._eventWatchers;
      getKeys(object.other_config).forEach(key => {
        const eventWatcher = eventWatchers[key];
        if (eventWatcher !== undefined) {
          delete eventWatchers[key];
          eventWatcher(object);
        }
      });
    } else if (type === 'task') {
      if (prev === undefined) {
        ++this._nTasks;
      }
      const taskWatchers = this._taskWatchers;
      const taskWatcher = taskWatchers === null || taskWatchers === void 0 ? void 0 : taskWatchers[ref];
      if (taskWatcher !== undefined) {
        const result = (0, _getTaskResult.default)(object);
        if (result !== undefined) {
          taskWatcher.resolve(result);
          delete taskWatchers[ref];
        }
      }
    }
  }
  async _replaceHostAddressInUrl(url) {
    try {
      const host = (await this.getAllRecords('host')).find(host => host.address === url.hostname);
      if (host !== undefined) {
        await this._setHostAddressInUrl(url, host);
      }
    } catch (error) {
      console.warn('_replaceHostAddressInUrl', url, error);
    }
    return url;
  }
  _processEvents(events) {
    const flush = this._objects.bufferEvents();
    events.forEach(event => {
      let type = event.class;
      const lcToTypes = this._lcToTypes;
      if (type in lcToTypes) {
        type = lcToTypes[type];
      }
      const {
        ref
      } = event;
      if (event.operation === 'del') {
        this._removeRecordFromCache(type, ref);
      } else {
        this._addRecordToCache(type, ref, event.snapshot);
      }
    });
    flush();
  }
  async _refreshCachedRecords(types) {
    const toRemoveByType = {
      __proto__: null
    };
    types.forEach(type => {
      toRemoveByType[type] = new Set();
    });
    const byRefs = this._objectsByRef;
    getKeys(byRefs).forEach(ref => {
      const {
        $type
      } = byRefs[ref];
      const toRemove = toRemoveByType[$type];
      if (toRemove !== undefined) {
        toRemove.add(ref);
      }
    });
    const flush = this._objects.bufferEvents();
    await Promise.all(types.map(async type => {
      try {
        const toRemove = toRemoveByType[type];
        const records = await this._roCall(`${type}.get_all_records`);
        const refs = getKeys(records);
        refs.forEach(ref => {
          toRemove.delete(ref);
          this._addRecordToCache(type, ref, records[ref]);
        });
        toRemove.forEach(ref => {
          this._removeRecordFromCache(type, ref);
        });
        if (type === 'task') {
          this._nTasks = refs.length;
        }
      } catch (error) {
        if ((error === null || error === void 0 ? void 0 : error.code) !== 'MESSAGE_REMOVED') {
          console.warn('_refreshCachedRecords', type, error);
        }
      }
    }));
    flush();
  }
  _removeRecordFromCache(type, ref) {
    const byRefs = this._objectsByRef;
    const object = byRefs[ref];
    if (object !== undefined) {
      this._objects.unset(object.$id);
      delete byRefs[ref];
      if (type === 'task') {
        --this._nTasks;
      }
    }
    const taskWatchers = this._taskWatchers;
    const taskWatcher = taskWatchers === null || taskWatchers === void 0 ? void 0 : taskWatchers[ref];
    if (taskWatcher !== undefined) {
      const error = new Error('task has been destroyed before completion');
      error.task = object;
      error.taskRef = ref;
      taskWatcher.reject(error);
      delete taskWatchers[ref];
    }
  }
  _roCall(method, args) {
    return (0, _promiseToolbox.pRetry)(() => this._sessionCall(method, args), this._roCallRetryOptions);
  }
  async _watchEvents() {
    {
      const watchedTypes = this._watchedTypes;
      if (this._taskWatchers === undefined && (watchedTypes === undefined || watchedTypes.includes('task'))) {
        this._taskWatchers = {
          __proto__: null
        };
      }
    }
    mainLoop: while (true) {
      var _this$_watchedTypes;
      if (this._resolveObjectsFetched === undefined) {
        this._objectsFetched = new Promise(resolve => {
          this._resolveObjectsFetched = resolve;
        });
      }
      await this._connected;
      let fromToken;
      try {
        fromToken = await this._sessionCall('event.inject', ['pool', this._pool.$ref]);
      } catch (error) {
        if ((error === null || error === void 0 ? void 0 : error.code) === 'MESSAGE_METHOD_UNKNOWN') {
          return this._watchEventsLegacy();
        }
        console.warn('_watchEvents', error);
        await (0, _promiseToolbox.pDelay)(this._eventPollDelay);
        continue;
      }
      const types = (_this$_watchedTypes = this._watchedTypes) !== null && _this$_watchedTypes !== void 0 ? _this$_watchedTypes : this._types;
      await this._refreshCachedRecords(types);
      this._resolveObjectsFetched();
      this._resolveObjectsFetched = undefined;
      const debounce = this._debounce;
      while (true) {
        await (0, _promiseToolbox.pDelay)(debounce);
        await this._connected;
        let result;
        try {
          result = await this._call('event.from', [this._sessionId, types, fromToken, EVENT_TIMEOUT + 0.1], EVENT_TIMEOUT * 1e3 * 1.1);
          this._lastEventFetchedTimestamp = Date.now();
          this._watchEventsError = undefined;
          this.emit('eventFetchingSuccess');
        } catch (error) {
          const code = error === null || error === void 0 ? void 0 : error.code;
          if (code === 'EVENTS_LOST' || code === 'SESSION_INVALID') {
            continue mainLoop;
          }
          this.emit('eventFetchingError', error);
          this._watchEventsError = error;
          console.warn('_watchEvents', error);
          await (0, _promiseToolbox.pDelay)(this._eventPollDelay);
          continue;
        }
        fromToken = result.token;
        this._processEvents(result.events);
        if (result.valid_ref_counts.task !== this._nTasks) {
          await this._refreshCachedRecords(['task']);
        }
      }
    }
  }
  async _watchEventsLegacy() {
    var _this$_watchedTypes2;
    if (this._resolveObjectsFetched === undefined) {
      this._objectsFetched = new Promise(resolve => {
        this._resolveObjectsFetched = resolve;
      });
    }
    await this._connected;
    const types = (_this$_watchedTypes2 = this._watchedTypes) !== null && _this$_watchedTypes2 !== void 0 ? _this$_watchedTypes2 : this._types;
    await this._refreshCachedRecords(types);
    this._resolveObjectsFetched();
    this._resolveObjectsFetched = undefined;
    await this._sessionCall('event.register', [types]);
    const debounce = this._debounce;
    while (true) {
      await (0, _promiseToolbox.pDelay)(debounce);
      try {
        await this._connected;
        this._processEvents(await this._roCall('event.next', undefined, EVENT_TIMEOUT * 1e3));
      } catch (error) {
        if ((error === null || error === void 0 ? void 0 : error.code) === 'EVENTS_LOST') {
          await _promiseToolbox.ignoreErrors.call(this._sessionCall('event.unregister', [types]));
          return this._watchEventsLegacy();
        }
        console.warn('_watchEventsLegacy', error);
        await (0, _promiseToolbox.pDelay)(this._eventPollDelay);
      }
    }
  }
  _wrapRecord(type, ref, data) {
    const RecordsByType = this._RecordsByType;
    let Record = RecordsByType[type];
    if (Record === undefined) {
      const fields = getKeys(data);
      const nFields = fields.length;
      const xapi = this;
      const getObjectByRef = ref => this._objectsByRef[ref];
      Record = defineProperty(function (ref, data) {
        var _data$uuid;
        defineProperties(this, {
          $id: {
            value: (_data$uuid = data.uuid) !== null && _data$uuid !== void 0 ? _data$uuid : ref
          },
          $ref: {
            value: ref
          }
        });
        for (let i = 0; i < nFields; ++i) {
          const field = fields[i];
          this[field] = data[field];
        }
      }, 'name', {
        value: type
      });
      const getters = {
        $pool: getPool
      };
      const props = {
        $call: function (method, ...args) {
          return xapi.call(`${type}.${method}`, this.$ref, ...args);
        },
        $callAsync: function (method, ...args) {
          return xapi.callAsync(`${type}.${method}`, this.$ref, ...args);
        },
        $type: type,
        $xapi: xapi
      };
      (function addMethods(object) {
        Object.getOwnPropertyNames(object).forEach(name => {
          const fn = Object.getOwnPropertyDescriptor(object, name).value;
          if (typeof fn === 'function' && name.startsWith(type + '_')) {
            props['$' + name.slice(type.length + 1)] = function (...args) {
              return xapi[name](this.$ref, ...args);
            };
          }
        });
        const proto = Object.getPrototypeOf(object);
        if (proto !== null) {
          addMethods(proto);
        }
      })(xapi);
      fields.forEach(field => {
        props[`set_${field}`] = function (value) {
          return xapi.setField(this.$type, this.$ref, field, value);
        };
        const $field = (field in RESERVED_FIELDS ? '$$' : '$') + field;
        const value = data[field];
        if (Array.isArray(value)) {
          if (value.length === 0 || _Ref.default.is(value[0])) {
            getters[$field] = function () {
              const value = this[field];
              return value.length === 0 ? value : value.map(getObjectByRef);
            };
          }
          props[`add_${field}`] = function (value) {
            return xapi.call(`${type}.add_${field}`, this.$ref, value).then(_noop.default);
          };
          props[`remove_${field}`] = function (value) {
            return xapi.call(`${type}.remove_${field}`, this.$ref, value).then(_noop.default);
          };
        } else if (value !== null && typeof value === 'object') {
          getters[$field] = function () {
            const value = this[field];
            const result = {};
            getKeys(value).forEach(key => {
              result[key] = xapi._objectsByRef[value[key]];
            });
            return result;
          };
          props[`update_${field}`] = function (entries, value) {
            return typeof entries === 'string' ? xapi.setFieldEntry(this.$type, this.$ref, field, entries, value) : xapi.setFieldEntries(this.$type, this.$ref, field, entries);
          };
        } else if (_Ref.default.is(value)) {
          getters[$field] = function () {
            return xapi._objectsByRef[this[field]];
          };
        }
      });
      const descriptors = {};
      getKeys(getters).forEach(key => {
        descriptors[key] = {
          configurable: true,
          get: getters[key]
        };
      });
      getKeys(props).forEach(key => {
        descriptors[key] = {
          configurable: true,
          value: props[key],
          writable: true
        };
      });
      defineProperties(Record.prototype, descriptors);
      RecordsByType[type] = Record;
    }
    return new Record(ref, data);
  }
}, (_applyDecoratedDescriptor(_class.prototype, "callAsync", [_promiseToolbox.cancelable], Object.getOwnPropertyDescriptor(_class.prototype, "callAsync"), _class.prototype), _applyDecoratedDescriptor(_class.prototype, "getResource", [_promiseToolbox.cancelable], Object.getOwnPropertyDescriptor(_class.prototype, "getResource"), _class.prototype), _applyDecoratedDescriptor(_class.prototype, "putResource", [_promiseToolbox.cancelable], Object.getOwnPropertyDescriptor(_class.prototype, "putResource"), _class.prototype)), _class);
exports.Xapi = Xapi;
const createClient = opts => new Xapi(opts);
exports.createClient = createClient;
//# sourceMappingURL=index.js.map