"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _xmlrpc = require("xmlrpc");
var _promiseToolbox = require("promise-toolbox");
var _XapiError = _interopRequireDefault(require("../_XapiError"));
var _prepareXmlRpcParams = _interopRequireDefault(require("./_prepareXmlRpcParams"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const logError = error => {
  if (error.res) {
    console.error('XML-RPC Error: %s (response status %s)', error.message, error.res.statusCode);
    console.error('%s', error.body);
  }
  throw error;
};
const parseResult = result => {
  const status = result.Status;
  if (status === undefined) {
    return result;
  }
  if (status !== 'Success') {
    throw _XapiError.default.wrap(result.ErrorDescription);
  }
  return result.Value;
};
var _default = ({
  secureOptions,
  url: {
    hostname,
    pathname,
    port,
    protocol
  },
  agent
}) => {
  const secure = protocol === 'https:';
  const client = (secure ? _xmlrpc.createSecureClient : _xmlrpc.createClient)({
    ...(secure ? secureOptions : undefined),
    agent,
    host: hostname,
    pathname,
    port
  });
  const call = (0, _promiseToolbox.promisify)(client.methodCall, client);
  return (method, args) => call(method, (0, _prepareXmlRpcParams.default)(args)).then(parseResult, logError);
};
exports.default = _default;
//# sourceMappingURL=xml-rpc.js.map