"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parse = exports.format = void 0;
var _filter = _interopRequireDefault(require("lodash/filter"));
var _map = _interopRequireDefault(require("lodash/map"));
var _trim = _interopRequireDefault(require("lodash/trim"));
var _trimStart = _interopRequireDefault(require("lodash/trimStart"));
var _querystring = _interopRequireDefault(require("querystring"));
var _urlParse = _interopRequireDefault(require("url-parse"));
const _excluded = ["type", "host", "path", "port", "username", "password", "domain", "protocol", "region"];
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }
function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
function _iterableToArrayLimit(arr, i) { var _i = null == arr ? null : "undefined" != typeof Symbol && arr[Symbol.iterator] || arr["@@iterator"]; if (null != _i) { var _s, _e, _x, _r, _arr = [], _n = !0, _d = !1; try { if (_x = (_i = _i.call(arr)).next, 0 === i) { if (Object(_i) !== _i) return; _n = !1; } else for (; !(_n = (_s = _x.call(_i)).done) && (_arr.push(_s.value), _arr.length !== i); _n = !0); } catch (err) { _d = !0, _e = err; } finally { try { if (!_n && null != _i.return && (_r = _i.return(), Object(_r) !== _r)) return; } finally { if (_d) throw _e; } } return _arr; } }
function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }
const NFS_RE = /^([^:]+):(?:(\d+):)?([^:?]+)(\?[^?]*)?$/;
const SMB_RE = /^([^:]+):(.+)@([^@]+)\\\\([^\0?]+)(?:\0([^?]*))?(\?[^?]*)?$/;
const sanitizePath = (...paths) => (0, _filter.default)((0, _map.default)(paths, s => s && (0, _filter.default)((0, _map.default)(s.split('/'), _trim.default)).join('/'))).join('/');
const parseOptionList = (optionList = '') => {
  if (optionList.startsWith('?')) {
    optionList = optionList.substring(1);
  }
  const parsed = _querystring.default.parse(optionList);
  Object.keys(parsed).forEach(key => {
    const val = parsed[key];
    parsed[key] = val === '' ? false : JSON.parse(val);
  });
  return parsed;
};
const makeOptionList = options => {
  const encoded = {};
  Object.keys(options).filter(key => options[key] !== undefined).forEach(key => {
    const val = options[key];
    encoded[key] = JSON.stringify(val);
  });
  return _querystring.default.stringify(encoded);
};
const parse = string => {
  let object = {};
  let _string$split = string.split('://'),
    _string$split2 = _slicedToArray(_string$split, 2),
    type = _string$split2[0],
    rest = _string$split2[1];
  if (type === 'file') {
    object.type = 'file';
    let optionList;
    var _rest$split = rest.split('?');
    var _rest$split2 = _slicedToArray(_rest$split, 2);
    rest = _rest$split2[0];
    optionList = _rest$split2[1];
    object.path = `/${(0, _trimStart.default)(rest, '/')}`;
    object = _objectSpread(_objectSpread({}, parseOptionList(optionList)), object);
  } else if (type === 'nfs') {
    object.type = 'nfs';
    let host, port, path, optionList;
    try {
      ;
      var _NFS_RE$exec = NFS_RE.exec(rest);
      var _NFS_RE$exec2 = _slicedToArray(_NFS_RE$exec, 5);
      host = _NFS_RE$exec2[1];
      port = _NFS_RE$exec2[2];
      path = _NFS_RE$exec2[3];
      optionList = _NFS_RE$exec2[4];
    } catch (err) {
      ;
      var _rest$split3 = rest.split(':');
      var _rest$split4 = _slicedToArray(_rest$split3, 2);
      host = _rest$split4[0];
      path = _rest$split4[1];
      object.invalidUrl = true;
    }
    object.host = host;
    object.port = port;
    object.path = `/${(0, _trimStart.default)(path, '/')}`;
    object = _objectSpread(_objectSpread({}, parseOptionList(optionList)), object);
  } else if (type === 'smb') {
    object.type = 'smb';
    const _SMB_RE$exec = SMB_RE.exec(rest),
      _SMB_RE$exec2 = _slicedToArray(_SMB_RE$exec, 7),
      username = _SMB_RE$exec2[1],
      password = _SMB_RE$exec2[2],
      domain = _SMB_RE$exec2[3],
      host = _SMB_RE$exec2[4],
      _SMB_RE$exec2$ = _SMB_RE$exec2[5],
      path = _SMB_RE$exec2$ === void 0 ? '' : _SMB_RE$exec2$,
      optionList = _SMB_RE$exec2[6];
    object.host = host;
    object.path = path;
    object.domain = domain;
    object.username = username;
    object.password = password;
    object = _objectSpread(_objectSpread({}, parseOptionList(optionList)), object);
  } else if (type === 's3' || type === 's3+http') {
    const parsed = (0, _urlParse.default)(string, false);
    object.protocol = parsed.protocol === 's3:' ? 'https' : 'http';
    object.type = 's3';
    object.region = parsed.hash.length < 2 ? undefined : parsed.hash.slice(1);
    object.host = parsed.host;
    object.path = parsed.pathname;
    object.username = parsed.username;
    object.password = decodeURIComponent(parsed.password);
    object = _objectSpread(_objectSpread({}, parseOptionList(parsed.query)), object);
  }
  return object;
};
exports.parse = parse;
const format = _ref => {
  let type = _ref.type,
    host = _ref.host,
    path = _ref.path,
    port = _ref.port,
    username = _ref.username,
    password = _ref.password,
    domain = _ref.domain,
    _ref$protocol = _ref.protocol,
    protocol = _ref$protocol === void 0 ? type : _ref$protocol,
    region = _ref.region,
    options = _objectWithoutProperties(_ref, _excluded);
  type === 'local' && (type = 'file');
  let string = `${type}://`;
  if (type === 'nfs') {
    string += `${host}:${port !== undefined ? port + ':' : ''}`;
  }
  if (type === 'smb') {
    string += `${username}:${password}@${domain}\\\\${host}`;
  }
  if (type === 's3') {
    string = protocol === 'https' ? 's3://' : 's3+http://';
    string += `${username}:${encodeURIComponent(password)}@${host}`;
  }
  path = sanitizePath(path);
  if (type === 'smb') {
    path = path.split('/');
    path = '\0' + path.join('\\');
  } else {
    path = `/${path}`;
  }
  string += path;
  const optionsList = makeOptionList(options);
  if (optionsList !== '') {
    string += '?' + optionsList;
  }
  if (type === 's3' && region !== undefined) {
    string += `#${region}`;
  }
  return string;
};
exports.format = format;
//# sourceMappingURL=index.js.map