"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.testSchema = exports.default = exports.configurationSchema = void 0;
var _humanFormat = _interopRequireDefault(require("human-format"));
var _momentTimezone = _interopRequireDefault(require("moment-timezone"));
var _log = require("@xen-orchestra/log");
var _lodash = require("lodash");
var _defined = require("@xen-orchestra/defined");
var _package = _interopRequireDefault(require("../package"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e2) { throw _e2; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e3) { didErr = true; err = _e3; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }
function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
function _iterableToArrayLimit(arr, i) { var _i = null == arr ? null : "undefined" != typeof Symbol && arr[Symbol.iterator] || arr["@@iterator"]; if (null != _i) { var _s, _e, _x, _r, _arr = [], _n = !0, _d = !1; try { if (_x = (_i = _i.call(arr)).next, 0 === i) { if (Object(_i) !== _i) return; _n = !1; } else for (; !(_n = (_s = _x.call(_i)).done) && (_arr.push(_s.value), _arr.length !== i); _n = !0); } catch (err) { _d = !0, _e = err; } finally { try { if (!_n && null != _i.return && (_r = _i.return(), Object(_r) !== _r)) return; } finally { if (_d) throw _e; } } return _arr; } }
function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }
function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }
const logger = (0, _log.createLogger)('xo:xo-server-backup-reports');
const configurationSchema = {
  type: 'object',
  properties: {
    toMails: {
      type: 'array',
      title: 'mails',
      description: 'an array of recipients (mails)',
      items: {
        type: 'string'
      },
      minItems: 1
    },
    toXmpp: {
      type: 'array',
      title: 'xmpp address',
      description: 'an array of recipients (xmpp)',
      items: {
        type: 'string'
      },
      minItems: 1
    }
  }
};
exports.configurationSchema = configurationSchema;
const testSchema = {
  type: 'object',
  properties: {
    runId: {
      type: 'string',
      description: `<a href="https://xen-orchestra.com/docs/backups.html#backups-execution" rel="noopener noreferrer" target="_blank">job's runId</a>`
    }
  },
  additionalProperties: false,
  required: ['runId']
};
exports.testSchema = testSchema;
const INDENT = '  ';
const UNKNOWN_ITEM = 'Unknown';
const ICON_FAILURE = '🚨';
const ICON_INTERRUPTED = '⚠️';
const ICON_SKIPPED = '⏩';
const ICON_SUCCESS = '✔';
const ICON_WARNING = '⚠️';
const STATUS_ICON = {
  failure: ICON_FAILURE,
  interrupted: ICON_INTERRUPTED,
  skipped: ICON_SKIPPED,
  success: ICON_SUCCESS
};
const DATE_FORMAT = 'dddd, MMMM Do YYYY, h:mm:ss a';
const createDateFormatter = timezone => timezone !== undefined ? timestamp => (0, _momentTimezone.default)(timestamp).tz(timezone).format(DATE_FORMAT) : timestamp => (0, _momentTimezone.default)(timestamp).format(DATE_FORMAT);
const formatDuration = milliseconds => _momentTimezone.default.duration(milliseconds).humanize();
const formatMethod = method => (0, _lodash.startCase)(method.slice(method.indexOf('.') + 1));
const formatSize = bytes => (0, _humanFormat.default)(bytes, {
  scale: 'binary',
  unit: 'B'
});
const formatSpeed = (bytes, milliseconds) => milliseconds > 0 ? (0, _humanFormat.default)(bytes * 1e3 / milliseconds, {
  scale: 'binary',
  unit: 'B/s'
}) : 'N/A';
const NO_VMS_MATCH_THIS_PATTERN = 'no VMs match this pattern';
const NO_SUCH_OBJECT_ERROR = 'no such object';
const UNHEALTHY_VDI_CHAIN_ERROR = 'unhealthy VDI chain';
const UNHEALTHY_VDI_CHAIN_MESSAGE = '[(unhealthy VDI chain) Job canceled to protect the VDI chain](https://xen-orchestra.com/docs/backup_troubleshooting.html#vdi-chain-protection)';
const isSkippedError = error => error.message === UNHEALTHY_VDI_CHAIN_ERROR || error.message === NO_SUCH_OBJECT_ERROR;
const STATUS = ['failure', 'interrupted', 'skipped', 'success'];
const TITLE_BY_STATUS = {
  failure: n => `## ${n} Failure${n === 1 ? '' : 's'}`,
  interrupted: n => `## ${n} Interrupted`,
  skipped: n => `## ${n} Skipped`,
  success: n => `## ${n} Success${n === 1 ? '' : 'es'}`
};
const getTemporalDataMarkdown = (end, start, formatDate) => {
  const markdown = [`- **Start time**: ${formatDate(start)}`];
  if (end !== undefined) {
    markdown.push(`- **End time**: ${formatDate(end)}`);
    const duration = end - start;
    if (duration >= 1) {
      markdown.push(`- **Duration**: ${formatDuration(duration)}`);
    }
  }
  return markdown;
};
const getWarningsMarkdown = (warnings = []) => warnings.map(({
  message
}) => `- **${ICON_WARNING} ${message}**`);
const getErrorMarkdown = task => {
  var _task$result$message, _task$result, _task$result2;
  let message;
  if (task.status === 'success' || (message = (_task$result$message = (_task$result = task.result) === null || _task$result === void 0 ? void 0 : _task$result.message) !== null && _task$result$message !== void 0 ? _task$result$message : (_task$result2 = task.result) === null || _task$result2 === void 0 ? void 0 : _task$result2.code) === undefined) {
    return;
  }
  const label = task.status === 'skipped' ? 'Reason' : 'Error';
  return `- **${label}**: ${message}`;
};
const MARKDOWN_BY_TYPE = {
  pool(task, {
    formatDate
  }) {
    const _task$data = task.data,
      id = _task$data.id,
      _task$data$pool = _task$data.pool,
      pool = _task$data$pool === void 0 ? {} : _task$data$pool,
      _task$data$poolMaster = _task$data.poolMaster,
      poolMaster = _task$data$poolMaster === void 0 ? {} : _task$data$poolMaster;
    const name = pool.name_label || poolMaster.name_label || UNKNOWN_ITEM;
    return {
      body: [pool.uuid !== undefined ? `- **UUID**: ${pool.uuid}` : `- **ID**: ${id}`, ...getTemporalDataMarkdown(task.end, task.start, formatDate), getErrorMarkdown(task)],
      title: `[pool] ${name}`
    };
  },
  xo(task, {
    formatDate,
    jobName
  }) {
    return {
      body: [...getTemporalDataMarkdown(task.end, task.start, formatDate), getErrorMarkdown(task)],
      title: `[XO] ${jobName}`
    };
  },
  remote(task, {
    formatDate,
    xo
  }) {
    return _asyncToGenerator(function* () {
      const id = task.data.id;
      const name = yield xo.getRemote(id).then(({
        name
      }) => name, error => {
        logger.warn(error);
        return UNKNOWN_ITEM;
      });
      return {
        body: [`- **ID**: ${id}`, ...getTemporalDataMarkdown(task.end, task.start, formatDate), getErrorMarkdown(task)],
        title: `[remote] ${name}`
      };
    })();
  }
};
const getMarkdown = (task, props) => {
  var _MARKDOWN_BY_TYPE$tas, _task$data2;
  return (_MARKDOWN_BY_TYPE$tas = MARKDOWN_BY_TYPE[(_task$data2 = task.data) === null || _task$data2 === void 0 ? void 0 : _task$data2.type]) === null || _MARKDOWN_BY_TYPE$tas === void 0 ? void 0 : _MARKDOWN_BY_TYPE$tas.call(MARKDOWN_BY_TYPE, task, props);
};
const toMarkdown = parts => {
  const lines = [];
  let indentLevel = -1;
  const helper = part => {
    if (typeof part === 'string') {
      lines.push(`${INDENT.repeat(indentLevel)}${part}`);
    } else if (Array.isArray(part)) {
      ++indentLevel;
      part.forEach(helper);
      --indentLevel;
    }
  };
  helper(parts);
  return lines.join('\n');
};
class BackupReportsXoPlugin {
  constructor(xo) {
    var _this = this;
    this._xo = xo;
    this._eventListener = _asyncToGenerator(function* (...args) {
      try {
        yield _this._report(...args);
      } catch (error) {
        logger.warn(error);
      }
    });
  }
  configure({
    toMails,
    toXmpp
  }) {
    this._mailsReceivers = toMails;
    this._xmppReceivers = toXmpp;
  }
  load() {
    this._xo.on('job:terminated', this._eventListener);
  }
  test({
    runId
  }) {
    return this._report(runId, undefined, true);
  }
  unload() {
    this._xo.removeListener('job:terminated', this._eventListener);
  }
  _report(runJobId, {
    type,
    status
  } = {}, force) {
    var _this2 = this;
    return _asyncToGenerator(function* () {
      const xo = _this2._xo;
      if (type === 'call') {
        return _this2._legacyVmHandler(status);
      }
      const log = yield xo.getBackupNgLogs(runJobId);
      if (log === undefined) {
        throw new Error(`no log found with runId=${JSON.stringify(runJobId)}`);
      }
      const reportWhen = log.data.reportWhen;
      if (!force && (reportWhen === 'never' || reportWhen === 'Never' || reportWhen === 'failure' && log.status === 'success')) {
        return;
      }
      const _yield$Promise$all = yield Promise.all([yield xo.getJob(log.jobId), yield xo.getSchedule(log.scheduleId).catch(error => {
          logger.warn(error);
        })]),
        _yield$Promise$all2 = _slicedToArray(_yield$Promise$all, 2),
        job = _yield$Promise$all2[0],
        schedule = _yield$Promise$all2[1];
      if (job.type === 'backup') {
        return _this2._ngVmHandler(log, job, schedule, force);
      } else if (job.type === 'metadataBackup') {
        return _this2._metadataHandler(log, job, schedule, force);
      }
      throw new Error(`Unknown backup job type: ${job.type}`);
    })();
  }
  _metadataHandler(log, {
    name: jobName
  }, schedule, force) {
    var _this3 = this;
    return _asyncToGenerator(function* () {
      var _log$tasks$length, _log$tasks, _tasksByStatus$succes, _tasksByStatus$succes2;
      const xo = _this3._xo;
      const formatDate = createDateFormatter(schedule === null || schedule === void 0 ? void 0 : schedule.timezone);
      const tasksByStatus = (0, _lodash.groupBy)(log.tasks, 'status');
      const n = (_log$tasks$length = (_log$tasks = log.tasks) === null || _log$tasks === void 0 ? void 0 : _log$tasks.length) !== null && _log$tasks$length !== void 0 ? _log$tasks$length : 0;
      const nSuccesses = (_tasksByStatus$succes = (_tasksByStatus$succes2 = tasksByStatus.success) === null || _tasksByStatus$succes2 === void 0 ? void 0 : _tasksByStatus$succes2.length) !== null && _tasksByStatus$succes !== void 0 ? _tasksByStatus$succes : 0;
      if (!force && log.data.reportWhen === 'failure') {
        delete tasksByStatus.success;
      }
      const markdown = [`##  Global status: ${log.status}`, '', `- **Job ID**: ${log.jobId}`, `- **Job name**: ${jobName}`, `- **Run ID**: ${log.id}`, ...getTemporalDataMarkdown(log.end, log.start, formatDate), n !== 0 && `- **Successes**: ${nSuccesses} / ${n}`, ...getWarningsMarkdown(log.warnings), getErrorMarkdown(log)];
      for (var _i2 = 0, _STATUS = STATUS; _i2 < _STATUS.length; _i2++) {
        const status = _STATUS[_i2];
        const tasks = tasksByStatus[status];
        if (tasks === undefined) {
          continue;
        }
        markdown.push('---', '', TITLE_BY_STATUS[status](tasks.length));
        var _iterator = _createForOfIteratorHelper(tasks),
          _step;
        try {
          for (_iterator.s(); !(_step = _iterator.n()).done;) {
            var _task$tasks;
            const task = _step.value;
            const taskMarkdown = yield getMarkdown(task, {
              formatDate,
              jobName: log.jobName,
              xo
            });
            if (taskMarkdown === undefined) {
              continue;
            }
            const title = taskMarkdown.title,
              body = taskMarkdown.body;
            const subMarkdown = [...body, ...getWarningsMarkdown(task.warnings)];
            var _iterator2 = _createForOfIteratorHelper((_task$tasks = task.tasks) !== null && _task$tasks !== void 0 ? _task$tasks : []),
              _step2;
            try {
              for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
                const subTask = _step2.value;
                const taskMarkdown = yield getMarkdown(subTask, {
                  formatDate,
                  xo
                });
                if (taskMarkdown === undefined) {
                  continue;
                }
                const icon = STATUS_ICON[subTask.status];
                const title = taskMarkdown.title,
                  body = taskMarkdown.body;
                subMarkdown.push([`- **${title}** ${icon}`, [...body, ...getWarningsMarkdown(subTask.warnings)]]);
              }
            } catch (err) {
              _iterator2.e(err);
            } finally {
              _iterator2.f();
            }
            markdown.push('', '', `### ${title}`, ...subMarkdown);
          }
        } catch (err) {
          _iterator.e(err);
        } finally {
          _iterator.f();
        }
      }
      markdown.push('---', '', `*${_package.default.name} v${_package.default.version}*`);
      return _this3._sendReport({
        subject: `[Xen Orchestra] ${log.status} − Metadata backup report for ${log.jobName} ${STATUS_ICON[log.status]}`,
        markdown: toMarkdown(markdown),
        success: log.status === 'success'
      });
    })();
  }
  _ngVmHandler(log, {
    name: jobName,
    settings
  }, schedule, force) {
    var _this4 = this;
    return _asyncToGenerator(function* () {
      const xo = _this4._xo;
      const mailReceivers = (0, _defined.get)(() => settings[''].reportRecipients);
      const _ref2 = log.data || {},
        reportWhen = _ref2.reportWhen,
        mode = _ref2.mode;
      const formatDate = createDateFormatter(schedule === null || schedule === void 0 ? void 0 : schedule.timezone);
      if (log.tasks === undefined) {
        const markdown = [`##  Global status: ${log.status}`, '', `- **Job ID**: ${log.jobId}`, `- **Run ID**: ${log.id}`, `- **mode**: ${mode}`, ...getTemporalDataMarkdown(log.end, log.start, formatDate), getErrorMarkdown(log), ...getWarningsMarkdown(log.warnings), '---', '', `*${_package.default.name} v${_package.default.version}*`];
        return _this4._sendReport({
          subject: `[Xen Orchestra] ${log.status} − Backup report for ${jobName} ${STATUS_ICON[log.status]}`,
          mailReceivers,
          markdown: toMarkdown(markdown),
          success: false
        });
      }
      const failedTasksText = [];
      const skippedVmsText = [];
      const successfulVmsText = [];
      const interruptedVmsText = [];
      let globalMergeSize = 0;
      let globalTransferSize = 0;
      let nFailures = 0;
      let nSkipped = 0;
      let nSuccesses = 0;
      let nInterrupted = 0;
      var _iterator3 = _createForOfIteratorHelper(log.tasks),
        _step3;
      try {
        for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
          var _taskLog$data, _taskLog$tasks;
          const taskLog = _step3.value;
          const _ref3 = (_taskLog$data = taskLog.data) !== null && _taskLog$data !== void 0 ? _taskLog$data : {},
            type = _ref3.type,
            id = _ref3.id;
          if (taskLog.message === 'get SR record' || taskLog.message === 'get remote adapter') {
            ++nFailures;
            failedTasksText.push('');
            try {
              if (type === 'SR') {
                const _xo$getObject = xo.getObject(id),
                  name = _xo$getObject.name_label,
                  uuid = _xo$getObject.uuid;
                failedTasksText.push(`### ${name}`, '', `- **UUID**: ${uuid}`);
              } else {
                const _yield$xo$getRemote = yield xo.getRemote(id),
                  name = _yield$xo$getRemote.name;
                failedTasksText.push(`### ${name}`, '', `- **UUID**: ${id}`);
              }
            } catch (error) {
              logger.warn(error);
              failedTasksText.push(`### ${UNKNOWN_ITEM}`, '', `- **UUID**: ${id}`);
            }
            failedTasksText.push(`- **Type**: ${type}`, ...getTemporalDataMarkdown(taskLog.end, taskLog.start, formatDate), ...getWarningsMarkdown(taskLog.warnings), `- **Error**: ${taskLog.result.message}`);
            continue;
          }
          if (type !== 'VM') {
            continue;
          }
          if (!force && taskLog.status === 'success' && reportWhen === 'failure') {
            ++nSuccesses;
            continue;
          }
          let vm;
          try {
            vm = xo.getObject(id);
          } catch (e) {}
          const text = ['', `### ${vm !== undefined ? vm.name_label : 'VM not found'}`, '', `- **UUID**: ${vm !== undefined ? vm.uuid : id}`, ...getTemporalDataMarkdown(taskLog.end, taskLog.start, formatDate), ...getWarningsMarkdown(taskLog.warnings)];
          const failedSubTasks = [];
          const snapshotText = [];
          const srsText = [];
          const remotesText = [];
          var _iterator4 = _createForOfIteratorHelper((_taskLog$tasks = taskLog.tasks) !== null && _taskLog$tasks !== void 0 ? _taskLog$tasks : []),
            _step4;
          try {
            for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
              var _subTaskLog$data;
              const subTaskLog = _step4.value;
              if (subTaskLog.message !== 'export' && subTaskLog.message !== 'snapshot') {
                continue;
              }
              const icon = STATUS_ICON[subTaskLog.status];
              const type = (_subTaskLog$data = subTaskLog.data) === null || _subTaskLog$data === void 0 ? void 0 : _subTaskLog$data.type;
              const errorMarkdown = getErrorMarkdown(subTaskLog);
              if (subTaskLog.message === 'snapshot') {
                snapshotText.push(`- **Snapshot** ${icon}`, [...getTemporalDataMarkdown(subTaskLog.end, subTaskLog.start, formatDate)]);
              } else if (type === 'remote') {
                const id = subTaskLog.data.id;
                const remote = yield xo.getRemote(id).catch(error => {
                  logger.warn(error);
                });
                const title = remote !== undefined ? remote.name : `Remote Not found`;
                remotesText.push(`- **${title}** (${id}) ${icon}`, [...getTemporalDataMarkdown(subTaskLog.end, subTaskLog.start, formatDate), ...getWarningsMarkdown(subTaskLog.warnings), errorMarkdown]);
                if (subTaskLog.status === 'failure') {
                  failedSubTasks.push(remote !== undefined ? remote.name : id);
                }
              } else {
                const id = subTaskLog.data.id;
                let sr;
                try {
                  sr = xo.getObject(id);
                } catch (e) {}
                const _ref4 = sr !== undefined ? [sr.name_label, sr.uuid] : [`SR Not found`, id],
                  _ref5 = _slicedToArray(_ref4, 2),
                  srName = _ref5[0],
                  srUuid = _ref5[1];
                srsText.push(`- **${srName}** (${srUuid}) ${icon}`, [...getTemporalDataMarkdown(subTaskLog.end, subTaskLog.start, formatDate), ...getWarningsMarkdown(subTaskLog.warnings), errorMarkdown]);
                if (subTaskLog.status === 'failure') {
                  failedSubTasks.push(sr !== undefined ? sr.name_label : id);
                }
              }
              (0, _lodash.forEach)(subTaskLog.tasks, operationLog => {
                var _operationLog$result;
                if (operationLog.message !== 'merge' && operationLog.message !== 'transfer' && operationLog.message !== 'health check') {
                  return;
                }
                const size = (_operationLog$result = operationLog.result) === null || _operationLog$result === void 0 ? void 0 : _operationLog$result.size;
                if (size > 0) {
                  if (operationLog.message === 'merge') {
                    globalMergeSize += size;
                  } else {
                    globalTransferSize += size;
                  }
                } else if (operationLog.status === 'success' && operationLog.message !== 'health check') {
                  return;
                }
                const operationText = [`- **${operationLog.message}** ${STATUS_ICON[operationLog.status]}`, [...getTemporalDataMarkdown(operationLog.end, operationLog.start, formatDate), size > 0 && `- **Size**: ${formatSize(size)}`, size > 0 && `- **Speed**: ${formatSpeed(size, operationLog.end - operationLog.start)}`, ...getWarningsMarkdown(operationLog.warnings), getErrorMarkdown(operationLog)]];
                if (type === 'remote') {
                  remotesText.push(operationText);
                } else if (type === 'SR') {
                  srsText.push(operationText);
                }
              });
            }
          } catch (err) {
            _iterator4.e(err);
          } finally {
            _iterator4.f();
          }
          const subText = [...snapshotText, srsText.length !== 0 && `- **SRs**`, srsText, remotesText.length !== 0 && `- **Remotes**`, remotesText];
          if (taskLog.result !== undefined) {
            if (taskLog.status === 'skipped') {
              ++nSkipped;
              skippedVmsText.push(...text, `- **Reason**: ${taskLog.result.message === UNHEALTHY_VDI_CHAIN_ERROR ? UNHEALTHY_VDI_CHAIN_MESSAGE : taskLog.result.message}`);
            } else {
              ++nFailures;
              failedTasksText.push(...text, `- **Error**: ${taskLog.result.message}`);
            }
          } else {
            if (taskLog.status === 'failure') {
              ++nFailures;
              failedTasksText.push(...text, ...subText);
            } else if (taskLog.status === 'interrupted') {
              ++nInterrupted;
              interruptedVmsText.push(...text, ...subText);
            } else {
              ++nSuccesses;
              successfulVmsText.push(...text, ...subText);
            }
          }
        }
      } catch (err) {
        _iterator3.e(err);
      } finally {
        _iterator3.f();
      }
      const nVmTasks = nSuccesses + nFailures + nSkipped + nInterrupted;
      const markdown = [`##  Global status: ${log.status}`, '', `- **Job ID**: ${log.jobId}`, `- **Run ID**: ${log.id}`, `- **mode**: ${mode}`, ...getTemporalDataMarkdown(log.end, log.start, formatDate), `- **Successes**: ${nSuccesses} / ${nVmTasks}`, globalTransferSize !== 0 && `- **Transfer size**: ${formatSize(globalTransferSize)}`, globalMergeSize !== 0 && `- **Merge size**: ${formatSize(globalMergeSize)}`, ...getWarningsMarkdown(log.warnings), ''];
      if (nFailures !== 0) {
        markdown.push('---', '', `## ${nFailures} Failure${nFailures === 1 ? '' : 's'}`, '', ...failedTasksText);
      }
      if (nSkipped !== 0) {
        markdown.push('---', '', `## ${nSkipped} Skipped`, '', ...skippedVmsText);
      }
      if (nInterrupted !== 0) {
        markdown.push('---', '', `## ${nInterrupted} Interrupted`, '', ...interruptedVmsText);
      }
      if (nSuccesses !== 0 && (force || reportWhen !== 'failure')) {
        markdown.push('---', '', `## ${nSuccesses} Success${nSuccesses === 1 ? '' : 'es'}`, '', ...successfulVmsText);
      }
      markdown.push('---', '', `*${_package.default.name} v${_package.default.version}*`);
      return _this4._sendReport({
        mailReceivers,
        markdown: toMarkdown(markdown),
        subject: `[Xen Orchestra] ${log.status} − Backup report for ${jobName} ${STATUS_ICON[log.status]}`,
        success: log.status === 'success'
      });
    })();
  }
  _sendReport({
    mailReceivers,
    markdown,
    subject,
    success
  }) {
    if (mailReceivers === undefined || mailReceivers.length === 0) {
      mailReceivers = this._mailsReceivers;
    }
    const xo = this._xo;
    return Promise.all([xo.sendEmail !== undefined && xo.sendEmail({
      to: mailReceivers,
      subject,
      markdown
    }), xo.sendToXmppClient !== undefined && xo.sendToXmppClient({
      to: this._xmppReceivers,
      message: markdown
    }), xo.sendSlackMessage !== undefined && xo.sendSlackMessage({
      message: markdown
    }), xo.sendIcinga2Status !== undefined && xo.sendIcinga2Status({
      status: success ? 'OK' : 'CRITICAL',
      message: markdown
    })]);
  }
  _legacyVmHandler(status) {
    const calls = status.calls,
      timezone = status.timezone,
      error = status.error;
    const formatDate = createDateFormatter(timezone);
    if (status.error !== undefined) {
      const _ref6 = error.message === NO_VMS_MATCH_THIS_PATTERN ? ['Skipped', ICON_SKIPPED] : ['Failure', ICON_FAILURE],
        _ref7 = _slicedToArray(_ref6, 2),
        globalStatus = _ref7[0],
        icon = _ref7[1];
      let markdown = [`##  Global status: ${globalStatus}`, '', `- **Start time**: ${formatDate(status.start)}`, `- **End time**: ${formatDate(status.end)}`, `- **Duration**: ${formatDuration(status.end - status.start)}`, `- **Error**: ${error.message}`, '---', '', `*${_package.default.name} v${_package.default.version}*`];
      markdown = markdown.join('\n');
      return this._sendReport({
        subject: `[Xen Orchestra] ${globalStatus} ${icon}`,
        markdown,
        success: false
      });
    }
    const callIds = Object.keys(calls);
    const nCalls = callIds.length;
    if (nCalls === 0) {
      return;
    }
    const oneCall = calls[callIds[0]];
    const _oneCall$params$_repo = oneCall.params._reportWhen,
      reportWhen = _oneCall$params$_repo === void 0 ? 'failure' : _oneCall$params$_repo;
    if (reportWhen === 'never') {
      return;
    }
    const method = oneCall.method;
    if (method !== 'vm.deltaCopy' && method !== 'vm.rollingBackup' && method !== 'vm.rollingDeltaBackup' && method !== 'vm.rollingDrCopy' && method !== 'vm.rollingSnapshot') {
      return;
    }
    const reportOnFailure = reportWhen === 'fail' || reportWhen === 'failure';
    let globalMergeSize = 0;
    let globalTransferSize = 0;
    let nFailures = 0;
    let nSkipped = 0;
    const failedBackupsText = [];
    const skippedBackupsText = [];
    const successfulBackupText = [];
    (0, _lodash.forEach)(calls, call => {
      const _call$params$id = call.params.id,
        id = _call$params$id === void 0 ? call.params.vm : _call$params$id;
      let vm;
      try {
        vm = this._xo.getObject(id);
      } catch (e) {}
      const end = call.end,
        start = call.start;
      const duration = end - start;
      const text = [`### ${vm !== undefined ? vm.name_label : 'VM not found'}`, '', `- **UUID**: ${vm !== undefined ? vm.uuid : id}`, `- **Start time**: ${formatDate(start)}`, `- **End time**: ${formatDate(end)}`, `- **Duration**: ${formatDuration(duration)}`];
      const error = call.error;
      if (error !== undefined) {
        const message = error.message;
        if (isSkippedError(error)) {
          ++nSkipped;
          skippedBackupsText.push(...text, `- **Reason**: ${message === UNHEALTHY_VDI_CHAIN_ERROR ? UNHEALTHY_VDI_CHAIN_MESSAGE : message}`, '');
        } else {
          ++nFailures;
          failedBackupsText.push(...text, `- **Error**: ${message}`, '');
        }
      } else if (!reportOnFailure) {
        const returnedValue = call.returnedValue;
        if (returnedValue != null) {
          const mergeSize = returnedValue.mergeSize,
            transferSize = returnedValue.transferSize;
          if (transferSize !== undefined) {
            globalTransferSize += transferSize;
            text.push(`- **Transfer size**: ${formatSize(transferSize)}`, `- **Transfer speed**: ${formatSpeed(transferSize, returnedValue.transferDuration)}`);
          }
          if (mergeSize !== undefined) {
            globalMergeSize += mergeSize;
            text.push(`- **Merge size**: ${formatSize(mergeSize)}`, `- **Merge speed**: ${formatSpeed(mergeSize, returnedValue.mergeDuration)}`);
          }
        }
        successfulBackupText.push(...text, '');
      }
    });
    const globalSuccess = nFailures === 0 && nSkipped === 0;
    if (reportOnFailure && globalSuccess) {
      return;
    }
    const tag = oneCall.params.tag;
    const duration = status.end - status.start;
    const nSuccesses = nCalls - nFailures - nSkipped;
    const globalStatus = globalSuccess ? `Success` : nFailures !== 0 ? `Failure` : `Skipped`;
    let markdown = [`##  Global status: ${globalStatus}`, '', `- **Type**: ${formatMethod(method)}`, `- **Start time**: ${formatDate(status.start)}`, `- **End time**: ${formatDate(status.end)}`, `- **Duration**: ${formatDuration(duration)}`, `- **Successes**: ${nSuccesses} / ${nCalls}`];
    if (globalTransferSize !== 0) {
      markdown.push(`- **Transfer size**: ${formatSize(globalTransferSize)}`);
    }
    if (globalMergeSize !== 0) {
      markdown.push(`- **Merge size**: ${formatSize(globalMergeSize)}`);
    }
    markdown.push('');
    if (nFailures !== 0) {
      markdown.push('---', '', `## ${nFailures} Failure${nFailures === 1 ? '' : 's'}`, '', ...failedBackupsText);
    }
    if (nSkipped !== 0) {
      markdown.push('---', '', `## ${nSkipped} Skipped`, '', ...skippedBackupsText);
    }
    if (nSuccesses !== 0 && !reportOnFailure) {
      markdown.push('---', '', `## ${nSuccesses} Success${nSuccesses === 1 ? '' : 'es'}`, '', ...successfulBackupText);
    }
    markdown.push('---', '', `*${_package.default.name} v${_package.default.version}*`);
    markdown = markdown.join('\n');
    return this._sendReport({
      markdown,
      subject: `[Xen Orchestra] ${globalStatus} − Backup report for ${tag} ${globalSuccess ? ICON_SUCCESS : nFailures !== 0 ? ICON_FAILURE : ICON_SKIPPED}`,
      success: globalSuccess
    });
  }
}
var _default = ({
  xo
}) => new BackupReportsXoPlugin(xo);
exports.default = _default;
//# sourceMappingURL=index.js.map