"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.debug = void 0;
var _filter = _interopRequireDefault(require("lodash/filter.js"));
var _plan = _interopRequireDefault(require("./plan"));
var _utils = require("./utils");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }
function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }
const debug = str => (0, _utils.debug)(`performance: ${str}`);
exports.debug = debug;
function epsiEqual(a, b, epsi = 0.001) {
  const absA = Math.abs(a);
  const absB = Math.abs(b);
  return Math.abs(a - b) <= Math.min(absA, absB) * epsi || absA <= epsi && absB <= epsi;
}
class PerformancePlan extends _plan.default {
  _checkResourcesThresholds(objects, averages) {
    return (0, _filter.default)(objects, object => {
      const objectAverages = averages[object.id];
      return objectAverages.cpu >= this._thresholds.cpu.high || objectAverages.memoryFree <= this._thresholds.memoryFree.high;
    });
  }
  execute() {
    var _this = this;
    return _asyncToGenerator(function* () {
      try {
        yield Promise.all((0, _filter.default)(_this._getHosts({
          powerState: 'Halted'
        }), host => host.powerOnMode !== '').map(host => {
          const {
            id
          } = host;
          return _this.xo.getXapi(id).powerOnHost(id);
        }));
      } catch (error) {
        console.error(error);
      }
      yield _this._processAntiAffinity();
      const hosts = _this._getHosts();
      const results = yield _this._getHostStatsAverages({
        hosts,
        toOptimizeOnly: true
      });
      if (!results) {
        return;
      }
      const {
        averages,
        toOptimize
      } = results;
      toOptimize.sort((a, b) => -_this._sortHosts(a, b));
      for (const exceededHost of toOptimize) {
        const {
          id
        } = exceededHost;
        debug(`Try to optimize Host (${exceededHost.id}).`);
        const availableHosts = (0, _filter.default)(hosts, host => host.id !== id);
        debug(`Available destinations: ${availableHosts.map(host => host.id)}.`);
        yield _this._optimize({
          exceededHost,
          hosts: availableHosts,
          hostsAverages: averages
        });
      }
    })();
  }
  _getThresholdState(averages) {
    return {
      cpu: averages.cpu >= this._thresholds.cpu.high,
      mem: averages.memoryFree <= this._thresholds.memoryFree.high
    };
  }
  _sortHosts(aAverages, bAverages) {
    const aState = this._getThresholdState(aAverages);
    const bState = this._getThresholdState(bAverages);
    if (aState.mem === bState.mem && aState.cpu === bState.cpu) {
      if (epsiEqual(aAverages.cpu, bAverages.cpu)) {
        return bAverages.memoryFree - aAverages.memoryFree;
      }
      return aAverages.cpu - bAverages.cpu;
    }
    if (!aState.mem && !aState.cpu || bState.mem && bState.cpu) {
      return -1;
    }
    if (!bState.mem && !bState.cpu || aState.mem && aState.cpu) {
      return 1;
    }
    return !aState.cpu ? -1 : 1;
  }
  _optimize({
    exceededHost,
    hosts,
    hostsAverages
  }) {
    var _this2 = this;
    return _asyncToGenerator(function* () {
      const vms = (0, _filter.default)(_this2._getAllRunningVms(), vm => vm.$container === exceededHost.id);
      const vmsAverages = yield _this2._getVmsAverages(vms, {
        [exceededHost.id]: exceededHost
      });
      vms.sort((a, b) => {
        const aAverages = vmsAverages[a.id];
        const bAverages = vmsAverages[b.id];
        if (epsiEqual(aAverages.cpu, bAverages.cpu, 3)) {
          return bAverages.memory - aAverages.memory;
        }
        return bAverages.cpu - aAverages.cpu;
      });
      const exceededAverages = hostsAverages[exceededHost.id];
      const promises = [];
      const xapiSrc = _this2.xo.getXapi(exceededHost);
      let optimizationCount = 0;
      const fmtSrcHost = `${exceededHost.id} "${exceededHost.name_label}"`;
      for (const vm of vms) {
        if (exceededAverages.cpu <= _this2._thresholds.cpu.low && exceededAverages.memoryFree >= _this2._thresholds.memoryFree.low) {
          return;
        }
        if (!vm.xenTools) {
          debug(`VM (${vm.id}) of Host (${exceededHost.id}) does not support pool migration.`);
          continue;
        }
        for (const tag of vm.tags) {
          if (_this2._antiAffinityTags.includes(tag)) {
            debug(`VM (${vm.id}) of Host (${exceededHost.id}) cannot be migrated. It contains anti-affinity tag '${tag}'.`);
            continue;
          }
        }
        hosts.sort((a, b) => {
          if (a.$poolId !== b.$poolId) {
            if (a.$poolId === vm.$poolId) {
              return -1;
            }
            if (b.$poolId === vm.$poolId) {
              return 1;
            }
          }
          return _this2._sortHosts(hostsAverages[a.id], hostsAverages[b.id]);
        });
        const destination = hosts[0];
        const destinationAverages = hostsAverages[destination.id];
        const vmAverages = vmsAverages[vm.id];
        const state = _this2._getThresholdState(exceededAverages);
        if (destinationAverages.cpu + vmAverages.cpu >= _this2._thresholds.cpu.low || destinationAverages.memoryFree - vmAverages.memory <= _this2._thresholds.cpu.high || !state.cpu && !state.memory && (exceededAverages.cpu - vmAverages.cpu < destinationAverages.cpu + vmAverages.cpu || exceededAverages.memoryFree + vmAverages.memory > destinationAverages.memoryFree - vmAverages.memory)) {
          debug(`Cannot migrate VM (${vm.id}) to Host (${destination.id}).`);
          debug(`Src Host CPU=${exceededAverages.cpu}, Dest Host CPU=${destinationAverages.cpu}, VM CPU=${vmAverages.cpu}`);
          debug(`Src Host free RAM=${exceededAverages.memoryFree}, Dest Host free RAM=${destinationAverages.memoryFree}, VM used RAM=${vmAverages.memory})`);
          continue;
        }
        exceededAverages.cpu -= vmAverages.cpu;
        destinationAverages.cpu += vmAverages.cpu;
        exceededAverages.memoryFree += vmAverages.memory;
        destinationAverages.memoryFree -= vmAverages.memory;
        debug(`Migrate VM (${vm.id} "${vm.name_label}") to Host (${destination.id} "${destination.name_label}") from Host (${fmtSrcHost}).`);
        optimizationCount++;
        promises.push(xapiSrc.migrateVm(vm._xapiId, _this2.xo.getXapi(destination), destination._xapiId));
      }
      yield Promise.all(promises);
      debug(`Performance mode: ${optimizationCount} optimizations for Host (${fmtSrcHost}).`);
    })();
  }
}
exports.default = PerformancePlan;
//# sourceMappingURL=performance-plan.js.map