"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.configurationSchema = void 0;
var _json = _interopRequireDefault(require("json5"));
var _cron = require("@xen-orchestra/cron");
var _log = require("@xen-orchestra/log");
var _lodash = require("lodash");
var _d3TimeFormat = require("d3-time-format");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e2) { throw _e2; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e3) { didErr = true; err = _e3; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }
function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
function _iterableToArrayLimit(arr, i) { var _i = null == arr ? null : "undefined" != typeof Symbol && arr[Symbol.iterator] || arr["@@iterator"]; if (null != _i) { var _s, _e, _x, _r, _arr = [], _n = !0, _d = !1; try { if (_x = (_i = _i.call(arr)).next, 0 === i) { if (Object(_i) !== _i) return; _n = !1; } else for (; !(_n = (_s = _x.call(_i)).done) && (_arr.push(_s.value), _arr.length !== i); _n = !0); } catch (err) { _d = !0, _e = err; } finally { try { if (!_n && null != _i.return && (_r = _i.return(), Object(_r) !== _r)) return; } finally { if (_d) throw _e; } } return _arr; } }
function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }
function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }
const logger = (0, _log.createLogger)('xo:xo-server-perf-alert');
const XAPI_TO_XENCENTER = {
  cpuUsage: 'cpu_usage',
  memoryUsage: 'mem_usage',
  storageUsage: 'physical_utilisation'
};
const COMPARATOR_FN = {
  '>': (a, b) => a > b,
  '<': (a, b) => a < b
};
const VM_FUNCTIONS = {
  cpuUsage: {
    name: 'VM CPU usage',
    description: 'Raises an alarm when the average usage of any CPU is higher/lower than the threshold',
    unit: '%',
    createParser: (comparator, legend, threshold) => {
      const regex = /cpu[0-9]+/;
      const filteredLegends = legend.filter(l => l.name.match(regex));
      const accumulator = Object.assign(...filteredLegends.map(l => ({
        [l.name]: []
      })));
      const getDisplayableValue = () => {
        const means = Object.keys(accumulator).map(l => (0, _lodash.mean)(accumulator[l]));
        return Math.max(...means) * 100;
      };
      return {
        parseRow: data => {
          filteredLegends.forEach(l => {
            accumulator[l.name].push(data.values[l.index]);
          });
        },
        getDisplayableValue,
        shouldAlarm: () => COMPARATOR_FN[comparator](getDisplayableValue(), threshold),
        threshold
      };
    }
  },
  memoryUsage: {
    name: 'VM memory usage',
    description: 'Raises an alarm when the used memory % is higher/lower than the threshold',
    unit: '% used',
    createParser: (comparator, legend, threshold) => {
      const memoryBytesLegend = legend.find(l => l.name === 'memory');
      const memoryKBytesFreeLegend = legend.find(l => l.name === 'memory_internal_free');
      const usedMemoryRatio = [];
      const getDisplayableValue = () => (0, _lodash.mean)(usedMemoryRatio) * 100;
      return {
        parseRow: data => {
          const memory = data.values[memoryBytesLegend.index];
          usedMemoryRatio.push((memory - 1024 * data.values[memoryKBytesFreeLegend.index]) / memory);
        },
        getDisplayableValue,
        shouldAlarm: () => COMPARATOR_FN[comparator](getDisplayableValue(), threshold),
        threshold
      };
    }
  }
};
const HOST_FUNCTIONS = {
  cpuUsage: {
    name: 'host CPU usage',
    description: 'Raises an alarm when the average usage of any CPU is higher/lower than the threshold',
    unit: '%',
    createParser: (comparator, legend, threshold) => {
      const regex = /^cpu[0-9]+$/;
      const filteredLegends = legend.filter(l => l.name.match(regex));
      const accumulator = Object.assign(...filteredLegends.map(l => ({
        [l.name]: []
      })));
      const getDisplayableValue = () => {
        const means = Object.keys(accumulator).map(l => (0, _lodash.mean)(accumulator[l]));
        return Math.max(...means) * 100;
      };
      return {
        parseRow: data => {
          filteredLegends.forEach(l => {
            accumulator[l.name].push(data.values[l.index]);
          });
        },
        getDisplayableValue,
        shouldAlarm: () => COMPARATOR_FN[comparator](getDisplayableValue(), threshold),
        threshold
      };
    }
  },
  memoryUsage: {
    name: 'host memory usage',
    description: 'Raises an alarm when the used memory % is higher/lower than the threshold',
    unit: '% used',
    createParser: (comparator, legend, threshold) => {
      const memoryKBytesLegend = legend.find(l => l.name === 'memory_total_kib');
      const memoryKBytesFreeLegend = legend.find(l => l.name === 'memory_free_kib');
      const usedMemoryRatio = [];
      const getDisplayableValue = () => (0, _lodash.mean)(usedMemoryRatio) * 100;
      return {
        parseRow: data => {
          const memory = data.values[memoryKBytesLegend.index];
          usedMemoryRatio.push((memory - data.values[memoryKBytesFreeLegend.index]) / memory);
        },
        getDisplayableValue,
        shouldAlarm: () => COMPARATOR_FN[comparator](getDisplayableValue(), threshold),
        threshold
      };
    }
  }
};
const SR_FUNCTIONS = {
  storageUsage: {
    name: 'SR storage usage',
    description: 'Raises an alarm when the used disk space % is higher/lower than the threshold',
    unit: '% used',
    createGetter: (comparator, threshold) => sr => {
      const getDisplayableValue = () => sr.physical_utilisation * 100 / sr.physical_size;
      return {
        getDisplayableValue,
        shouldAlarm: () => COMPARATOR_FN[comparator](getDisplayableValue(), threshold),
        threshold
      };
    }
  }
};
const TYPE_FUNCTION_MAP = {
  vm: VM_FUNCTIONS,
  host: HOST_FUNCTIONS,
  sr: SR_FUNCTIONS
};
const COMPARATOR_ENTRY = {
  title: 'Comparator',
  type: 'string',
  default: Object.keys(COMPARATOR_FN)[0],
  enum: Object.keys(COMPARATOR_FN)
};
const currentAlarms = {};
const configurationSchema = {
  type: 'object',
  properties: {
    baseUrl: {
      type: 'string',
      title: 'Xen Orchestra URL',
      description: 'URL used in alert messages to quickly get to the VMs (ex: https://xoa.company.tld/ )'
    },
    hostMonitors: {
      type: 'array',
      title: 'Host Monitors',
      description: 'Alarms checking hosts on all pools. The selected performance counter is sampled regularly and averaged. ' + 'The Average is compared to the threshold and an alarm is raised upon crossing',
      items: {
        type: 'object',
        properties: {
          smartMode: {
            title: 'All running hosts',
            type: 'boolean',
            description: 'When enabled, all running hosts will be considered for the alert.',
            default: false
          },
          uuids: {
            title: 'Hosts',
            type: 'array',
            items: {
              type: 'string',
              $type: 'Host'
            }
          },
          variableName: {
            title: 'Alarm Type',
            description: Object.keys(HOST_FUNCTIONS).map(k => `  * ${k} (${HOST_FUNCTIONS[k].unit}): ${HOST_FUNCTIONS[k].description}`).join('\n'),
            type: 'string',
            default: Object.keys(HOST_FUNCTIONS)[0],
            enum: Object.keys(HOST_FUNCTIONS)
          },
          comparator: COMPARATOR_ENTRY,
          alarmTriggerLevel: {
            title: 'Threshold',
            description: 'The direction of the crossing is given by the comparator type',
            type: 'number',
            default: 40
          },
          alarmTriggerPeriod: {
            title: 'Average Length (s)',
            description: 'The points are averaged this number of seconds then the average is compared with the threshold',
            type: 'number',
            default: 60,
            enum: [60, 600]
          }
        },
        oneOf: [{
          properties: {
            uuids: {}
          },
          required: ['uuids']
        }, {
          properties: {
            smartMode: {
              const: true
            }
          },
          required: ['smartMode']
        }]
      }
    },
    vmMonitors: {
      type: 'array',
      title: 'VM Monitors',
      description: 'Alarms checking all VMs on all pools. The selected performance counter is sampled regularly and averaged. ' + 'The Average is compared to the threshold and an alarm is raised upon crossing',
      items: {
        type: 'object',
        properties: {
          smartMode: {
            title: 'All running VMs',
            type: 'boolean',
            description: 'When enabled, all running VMs will be considered for the alert.',
            default: false
          },
          uuids: {
            title: 'Virtual Machines',
            type: 'array',
            items: {
              type: 'string',
              $type: 'VM'
            }
          },
          variableName: {
            title: 'Alarm Type',
            description: Object.keys(VM_FUNCTIONS).map(k => `  * ${k} (${VM_FUNCTIONS[k].unit}): ${VM_FUNCTIONS[k].description}`).join('\n'),
            type: 'string',
            default: Object.keys(VM_FUNCTIONS)[0],
            enum: Object.keys(VM_FUNCTIONS)
          },
          comparator: COMPARATOR_ENTRY,
          alarmTriggerLevel: {
            title: 'Threshold',
            description: 'The direction of the crossing is given by the comparator type',
            type: 'number',
            default: 40
          },
          alarmTriggerPeriod: {
            title: 'Average Length (s)',
            description: 'The points are averaged this number of seconds then the average is compared with the threshold',
            type: 'number',
            default: 60,
            enum: [60, 600]
          }
        },
        oneOf: [{
          properties: {
            uuids: {}
          },
          required: ['uuids']
        }, {
          properties: {
            smartMode: {
              const: true
            }
          },
          required: ['smartMode']
        }]
      }
    },
    srMonitors: {
      type: 'array',
      title: 'SR Monitors',
      description: 'Alarms checking all SRs on all pools. The selected performance counter is sampled regularly and averaged. ' + 'The Average is compared to the threshold and an alarm is raised upon crossing',
      items: {
        type: 'object',
        properties: {
          smartMode: {
            title: 'All SRs',
            type: 'boolean',
            description: 'When enabled, all SRs will be considered for the alert.',
            default: false
          },
          uuids: {
            title: 'SRs',
            type: 'array',
            items: {
              type: 'string',
              $type: 'SR'
            }
          },
          variableName: {
            title: 'Alarm Type',
            description: Object.keys(SR_FUNCTIONS).map(k => `  * ${k} (${SR_FUNCTIONS[k].unit}): ${SR_FUNCTIONS[k].description}`).join('\n'),
            type: 'string',
            default: Object.keys(SR_FUNCTIONS)[0],
            enum: Object.keys(SR_FUNCTIONS)
          },
          comparator: COMPARATOR_ENTRY,
          alarmTriggerLevel: {
            title: 'Threshold',
            description: 'The direction of the crossing is given by the comparator type',
            type: 'number',
            default: 80
          }
        },
        oneOf: [{
          properties: {
            uuids: {}
          },
          required: ['uuids']
        }, {
          properties: {
            smartMode: {
              const: true
            }
          },
          required: ['smartMode']
        }]
      }
    },
    toEmails: {
      type: 'array',
      title: 'Email addresses',
      description: 'Email addresses of the alert recipients',
      items: {
        type: 'string'
      },
      minItems: 1
    }
  }
};
exports.configurationSchema = configurationSchema;
const clearCurrentAlarms = () => (0, _lodash.forOwn)(currentAlarms, (v, k) => {
  delete currentAlarms[k];
});
const raiseOrLowerAlarm = (alarmId, shouldRaise, raiseCallback, lowerCallback) => {
  const current = currentAlarms[alarmId];
  if (shouldRaise) {
    if (!current) {
      currentAlarms[alarmId] = true;
      raiseCallback(alarmId);
    }
  } else if (current) {
    try {
      lowerCallback(alarmId);
    } finally {
      delete currentAlarms[alarmId];
    }
  }
};
function getServerTimestamp(_x, _x2) {
  return _getServerTimestamp.apply(this, arguments);
}
function _getServerTimestamp() {
  _getServerTimestamp = _asyncToGenerator(function* (xapi, host) {
    const serverLocalTime = yield xapi.call('host.get_servertime', host.$ref);
    return Math.floor((0, _d3TimeFormat.utcParse)('%Y%m%dT%H:%M:%SZ')(serverLocalTime).getTime() / 1000);
  });
  return _getServerTimestamp.apply(this, arguments);
}
class PerfAlertXoPlugin {
  constructor(xo) {
    var _this = this;
    this._xo = xo;
    this._job = (0, _cron.createSchedule)('* * * * *').createJob(_asyncToGenerator(function* () {
      try {
        yield _this._checkMonitors();
      } catch (error) {
        console.error('[WARN] scheduled function:', error && error.stack || error);
      }
    }));
  }
  configure(configuration) {
    var _this2 = this;
    return _asyncToGenerator(function* () {
      _this2._configuration = configuration;
      clearCurrentAlarms();
    })();
  }
  load() {
    this._job.start();
  }
  unload() {
    this._job.stop();
  }
  _generateUrl(type, object) {
    const baseUrl = this._configuration.baseUrl;
    const uuid = object.uuid;
    switch (type) {
      case 'vm':
        return `${baseUrl}#/vms/${uuid}/stats`;
      case 'host':
        return `${baseUrl}#/hosts/${uuid}/stats`;
      case 'sr':
        return `${baseUrl}#/srs/${uuid}/general`;
      default:
        return 'unknown type';
    }
  }
  test() {
    var _this3 = this;
    return _asyncToGenerator(function* () {
      const monitorBodies = yield Promise.all((0, _lodash.map)(_this3._getMonitors(), function () {
        var _ref2 = _asyncToGenerator(function* (m) {
          return `
## Monitor for ${m.title}

${(yield m.snapshot()).map(entry => entry.listItem).join('')}`;
        });
        return function (_x3) {
          return _ref2.apply(this, arguments);
        };
      }()));
      _this3._sendAlertEmail('TEST', `
# Performance Alert Test
Your alarms and their current status:
${monitorBodies.join('\n')}`);
    })();
  }
  _parseDefinition(definition) {
    var _this4 = this;
    const objectType = definition.objectType;
    const lcObjectType = objectType.toLowerCase();
    const alarmId = `${lcObjectType}|${definition.variableName}|${definition.alarmTriggerLevel}`;
    const typeFunction = TYPE_FUNCTION_MAP[lcObjectType][definition.variableName];
    const parseData = (result, uuid) => {
      const parsedLegend = result.meta.legend.map((l, index) => {
        const _l$split = l.split(':'),
          _l$split2 = _slicedToArray(_l$split, 4),
          operation = _l$split2[0],
          type = _l$split2[1],
          uuid = _l$split2[2],
          name = _l$split2[3];
        const parsedName = name.split('_');
        const lastComponent = parsedName[parsedName.length - 1];
        const relatedEntity = parsedName.length > 1 && lastComponent.match(/^[0-9a-f]{8}$/) ? lastComponent : null;
        return {
          operation,
          type,
          uuid,
          name,
          relatedEntity,
          parsedName,
          index
        };
      });
      const legendTree = {};
      const getNode = (element, name, defaultValue = {}) => {
        const child = element[name];
        if (child === undefined) {
          element[name] = defaultValue;
          return defaultValue;
        }
        return child;
      };
      parsedLegend.forEach(l => {
        const root = getNode(legendTree, l.uuid);
        const relatedNode = getNode(root, l.relatedEntity);
        relatedNode[l.name] = l;
      });
      const parser = typeFunction.createParser(definition.comparator, parsedLegend.filter(l => l.uuid === uuid), definition.alarmTriggerLevel);
      result.data.forEach(d => parser.parseRow(d));
      return parser;
    };
    const observationPeriod = definition.alarmTriggerPeriod !== undefined ? definition.alarmTriggerPeriod : 60;
    return _objectSpread(_objectSpread({}, definition), {}, {
      alarmId,
      vmFunction: typeFunction,
      title: `${typeFunction.name} ${definition.comparator} ${definition.alarmTriggerLevel}${typeFunction.unit}`,
      snapshot: function () {
        var _snapshot = _asyncToGenerator(function* () {
          return Promise.all((0, _lodash.map)(definition.smartMode ? (0, _lodash.filter)(_this4._xo.getObjects(), obj => obj.type === objectType && (objectType !== 'VM' && objectType !== 'host' || obj.power_state === 'Running')).map(obj => obj.uuid) : definition.uuids, function () {
            var _ref3 = _asyncToGenerator(function* (uuid) {
              try {
                const result = {
                  uuid,
                  object: _this4._xo.getXapi(uuid).getObject(uuid)
                };
                if (result.object === undefined) {
                  throw new Error('object not found');
                }
                result.objectLink = `[${result.object.name_label}](${_this4._generateUrl(lcObjectType, result.object)})`;
                if (typeFunction.createGetter === undefined) {
                  result.rrd = yield _this4.getRrd(result.object, observationPeriod);
                  if (result.rrd !== null) {
                    const data = parseData(result.rrd, result.object.uuid);
                    Object.assign(result, {
                      data,
                      value: data.getDisplayableValue(),
                      shouldAlarm: data.shouldAlarm(),
                      threshold: data.threshold,
                      observationPeriod
                    });
                  }
                } else {
                  const getter = typeFunction.createGetter(definition.comparator, definition.alarmTriggerLevel);
                  const data = getter(result.object);
                  Object.assign(result, {
                    value: data.getDisplayableValue(),
                    shouldAlarm: data.shouldAlarm(),
                    threshold: data.threshold,
                    observationPeriod
                  });
                }
                result.listItem = `  * ${result.objectLink}: ${result.value === undefined ? "**Can't read performance counters**" : result.value.toFixed(1) + typeFunction.unit}\n`;
                return result;
              } catch (error) {
                logger.warn(error);
                return {
                  uuid,
                  object: null,
                  objectLink: `cannot find object ${uuid}`,
                  listItem: `  * ${uuid}: **Can't read performance counters**\n`
                };
              }
            });
            return function (_x4) {
              return _ref3.apply(this, arguments);
            };
          }()));
        });
        function snapshot() {
          return _snapshot.apply(this, arguments);
        }
        return snapshot;
      }()
    });
  }
  _getMonitors() {
    return (0, _lodash.map)(this._configuration.hostMonitors, def => this._parseDefinition(_objectSpread(_objectSpread({}, def), {}, {
      objectType: 'host'
    }))).concat((0, _lodash.map)(this._configuration.vmMonitors, def => this._parseDefinition(_objectSpread(_objectSpread({}, def), {}, {
      objectType: 'VM'
    })))).concat((0, _lodash.map)(this._configuration.srMonitors, def => this._parseDefinition(_objectSpread(_objectSpread({}, def), {}, {
      objectType: 'SR'
    }))));
  }
  _checkMonitors() {
    var _this5 = this;
    return _asyncToGenerator(function* () {
      const monitors = _this5._getMonitors();
      var _iterator = _createForOfIteratorHelper(monitors),
        _step;
      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          const monitor = _step.value;
          const snapshot = yield monitor.snapshot();
          const entriesWithMissingStats = [];
          var _iterator2 = _createForOfIteratorHelper(snapshot),
            _step2;
          try {
            for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
              const entry = _step2.value;
              if (entry.object.physical_size <= 0 && entry.object.content_type === 'iso') continue;
              if (entry.value === undefined) {
                entriesWithMissingStats.push(entry);
                continue;
              }
              const raiseAlarm = _alarmId => {
                _this5._xo.getXapi(entry.object.uuid).call('message.create', 'ALARM', 3, entry.object.$type, entry.object.uuid, `value: ${(entry.value / 100).toFixed(1)}
config:
<variable>
<name value="${XAPI_TO_XENCENTER[monitor.variableName]}"/>
<alarm_trigger_level value="${entry.threshold / 100}"/>
<alarm_trigger_period value ="${entry.observationPeriod}"/>
</variable>`);
                _this5._sendAlertEmail('', `
## ALERT: ${monitor.title}
${entry.listItem}
### Description
  ${monitor.vmFunction.description}`);
              };
              const lowerAlarm = _alarmId => {
                _this5._sendAlertEmail('END OF ALERT', `
## END OF ALERT: ${monitor.title}
${entry.listItem}
### Description
  ${monitor.vmFunction.description}`);
              };
              raiseOrLowerAlarm(`${monitor.alarmId}|${entry.uuid}`, entry.shouldAlarm, raiseAlarm, lowerAlarm);
            }
          } catch (err) {
            _iterator2.e(err);
          } finally {
            _iterator2.f();
          }
          raiseOrLowerAlarm(`${monitor.alarmId}|${entriesWithMissingStats.map(({
            uuid
          }) => uuid).sort().join('|')}|RRD`, entriesWithMissingStats.length !== 0, () => {
            _this5._sendAlertEmail('Secondary Issue', `
## There was an issue when trying to check ${monitor.title}
${entriesWithMissingStats.map(({
              listItem
            }) => listItem).join('\n')}`);
          }, () => {});
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
    })();
  }
  _sendAlertEmail(subjectSuffix, markdownBody) {
    if (this._configuration.toEmails !== undefined && this._xo.sendEmail !== undefined) {
      this._xo.sendEmail({
        to: this._configuration.toEmails,
        subject: `[Xen Orchestra] − Performance Alert ${subjectSuffix}`,
        markdown: markdownBody + `\n\n\nSent from Xen Orchestra [perf-alert plugin](${this._configuration.baseUrl}#/settings/plugins)\n`
      });
    } else {
      throw new Error('The email alert system has a configuration issue.');
    }
  }
  getRrd(xapiObject, secondsAgo) {
    var _this6 = this;
    return _asyncToGenerator(function* () {
      const host = xapiObject.$type === 'host' ? xapiObject : xapiObject.$resident_on;
      if (host == null) {
        return null;
      }
      const xapi = _this6._xo.getXapi(host.uuid);
      const serverTimestamp = yield getServerTimestamp(xapi, host);
      const payload = {
        host,
        query: {
          cf: 'AVERAGE',
          host: (xapiObject.$type === 'host').toString(),
          json: 'true',
          start: serverTimestamp - secondsAgo
        }
      };
      if (xapiObject.$type === 'VM') {
        payload.vm_uuid = xapiObject.uuid;
      }
      return _json.default.parse(yield (yield xapi.getResource('/rrd_updates', payload)).text());
    })();
  }
}
exports.default = function ({
  xo
}) {
  return new PerfAlertXoPlugin(xo);
};
//# sourceMappingURL=index.js.map