"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TlsHelper = void 0;
var _fromEvent = _interopRequireDefault(require("promise-toolbox/fromEvent"));
var _tls = require("tls");
var _log = require("@xen-orchestra/log");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const log = (0, _log.createLogger)('xo:xo-server:sdn-controller:tls-connect');
class TlsHelper {
  updateCertificates(clientKey, clientCert, caCert) {
    this._clientKey = clientKey;
    this._clientCert = clientCert;
    this._caCert = caCert;
    log.debug('Certificates have been updated');
  }
  async connect(address, port) {
    const options = {
      ca: this._caCert,
      cert: this._clientCert,
      ciphers: 'DEFAULT:!DH',
      host: address,
      key: this._clientKey,
      port,
      rejectUnauthorized: false,
      requestCert: false
    };
    const socket = (0, _tls.connect)(options);
    try {
      await (0, _fromEvent.default)(socket, 'secureConnect');
    } catch (error) {
      log.error('TLS connection failed', {
        error,
        address,
        port
      });
      throw error;
    }
    socket.on('error', error => {
      log.error('Socket error', {
        error,
        address,
        port
      });
    });
    return socket;
  }
}
exports.TlsHelper = TlsHelper;
//# sourceMappingURL=tls-helper.js.map