"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.testSchema = exports.default = exports.configurationSchema = void 0;
var _nodemailer = require("nodemailer");
var _nodemailerMarkdown = require("nodemailer-markdown");
var _promiseToolbox = require("promise-toolbox");
const _excluded = ["ignoreUnauthorized", "password", "secure", "user"];
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
const removeUndefined = obj => {
  Object.keys(obj).forEach(key => {
    if (obj[key] === undefined) {
      delete obj[key];
    }
  });
  return obj;
};
const markdownCompiler = (0, _nodemailerMarkdown.markdown)();
const logAndRethrow = error => {
  console.error('[WARN] plugin transport-email:', error && error.stack || error);
  throw error;
};
const configurationSchema = {
  type: 'object',
  properties: {
    from: {
      type: 'object',
      properties: {
        name: {
          type: 'string',
          description: 'human readable name of the sender'
        },
        address: {
          type: 'string',
          description: 'email address of the sender'
        }
      },
      additionalProperties: false,
      required: ['address']
    },
    transport: {
      type: 'object',
      properties: {
        host: {
          type: 'string',
          description: 'hostname or IP address of the SMTP server'
        },
        port: {
          type: 'integer',
          description: 'port of the SMTP server (defaults to 25 or 465 for TLS)'
        },
        secure: {
          default: false,
          enum: [false, 'force', 'disabled', true],
          enumNames: ['auto (uses STARTTLS if available)', 'force (requires STARTTLS or fail)', 'disabled (never use STARTTLS)', 'TLS'],
          description: 'whether the connection should use TLS'
        },
        ignoreUnauthorized: {
          type: 'boolean',
          description: 'ignore certificates error (e.g. self-signed certificate)'
        },
        user: {
          type: 'string',
          description: 'name to use to authenticate'
        },
        password: {
          type: 'string',
          description: 'password to use to authenticate'
        }
      },
      additionalProperties: false,
      required: ['host']
    }
  },
  additionalProperties: false,
  required: ['from', 'transport']
};
exports.configurationSchema = configurationSchema;
const testSchema = {
  type: 'object',
  properties: {
    to: {
      type: 'string',
      description: 'recipient of the test mail'
    }
  },
  additionalProperties: false,
  required: ['to']
};
exports.testSchema = testSchema;
class TransportEmailPlugin {
  constructor({
    staticConfig,
    xo
  }) {
    this._staticConfig = staticConfig;
    this._xo = xo;
    this._unset = null;
    this._send = null;
  }
  configure(_ref) {
    let from = _ref.from,
      _ref$transport = _ref.transport,
      ignoreUnauthorized = _ref$transport.ignoreUnauthorized,
      password = _ref$transport.password,
      secure = _ref$transport.secure,
      user = _ref$transport.user,
      transportConf = _objectWithoutProperties(_ref$transport, _excluded);
    if (ignoreUnauthorized != null) {
      ;
      (transportConf.tls || (transportConf.tls = {})).rejectUnauthorized = !ignoreUnauthorized;
    }
    if (user != null && password != null) {
      transportConf.auth = {
        user,
        pass: password
      };
    }
    switch (secure) {
      case true:
        transportConf.secure = true;
        break;
      case 'disabled':
        transportConf.ignoreTLS = true;
        break;
      case 'required':
        transportConf.requireTLS = true;
        break;
    }
    const transport = (0, _nodemailer.createTransport)(_objectSpread(_objectSpread({}, transportConf), this._staticConfig.transport), {
      from
    });
    transport.use('compile', markdownCompiler);
    this._send = (0, _promiseToolbox.promisify)(transport.sendMail, transport);
  }
  load() {
    this._unset = this._xo.defineProperty('sendEmail', this._sendEmail, this);
  }
  unload() {
    this._unset();
  }
  test({
    to
  }) {
    return this._sendEmail({
      to,
      subject: '[Xen Orchestra] Test of transport-email plugin',
      markdown: `Hi there,

The transport-email plugin for Xen Orchestra server seems to be working fine, nicely done :)
`,
      attachments: [{
        filename: 'example.txt',
        content: 'Attachments are working too, great!\n'
      }]
    });
  }
  _sendEmail({
    from,
    to,
    cc,
    bcc,
    subject,
    markdown,
    attachments
  }) {
    return this._send(removeUndefined({
      from,
      to,
      cc,
      bcc,
      subject,
      markdown,
      attachments
    })).catch(logAndRethrow);
  }
}
var _default = opts => new TransportEmailPlugin(opts);
exports.default = _default;
//# sourceMappingURL=index.js.map