"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.testSchema = exports.default = exports.configurationSchema = exports.WARNING = exports.OK = exports.CRITICAL = void 0;
var _bufferCrc = _interopRequireDefault(require("buffer-crc32"));
var _net = _interopRequireDefault(require("net"));
var _buffer = require("buffer");
var _log = require("@xen-orchestra/log");
var _template = require("@xen-orchestra/template");
const _excluded = ["host", "service"];
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
const _createLogger = (0, _log.createLogger)('xo:server:transport:nagios'),
  debug = _createLogger.debug,
  warn = _createLogger.warn;
const hostDescription = `Host name on Nagios.

Leave empty if the host name equals the vm name (the default configuration).

Otherwise, you could choose a custom name but the template \`{vm.name_label}\` must  be included. For example: \`xo-backup-{vm.name_label}\`.`;
const serviceDescription = `Service name on Nagios.

Leave empty if the host name equals the backup job name (the default configuration).

Otherwise, you could choose a custom name but the template \`{job.name}\` must e included. For example: \`{job.name}-Xen Orchestra\`.`;
const configurationSchema = {
  type: 'object',
  properties: {
    server: {
      type: 'string',
      description: 'The nagios server adress'
    },
    port: {
      type: 'integer',
      description: 'The NSCA port'
    },
    key: {
      type: 'string',
      description: 'The encryption key'
    },
    host: {
      default: '{vm.name_label}',
      description: hostDescription,
      type: 'string'
    },
    service: {
      default: '{job.name}',
      description: serviceDescription,
      type: 'string'
    }
  },
  additionalProperties: false,
  required: ['server', 'port', 'key']
};
exports.configurationSchema = configurationSchema;
const testSchema = {
  type: 'object',
  properties: {
    VmNameLabel: {
      title: 'VM Name Label',
      description: 'Name of a VM',
      type: 'string'
    },
    jobName: {
      title: 'Job Name',
      description: 'Name of a backup job',
      type: 'string'
    }
  },
  required: ['VmNameLabel', 'jobName']
};
exports.testSchema = testSchema;
function nscaPacketBuilder({
  host,
  iv,
  message,
  service,
  status,
  timestamp
}) {
  const SIZE = 720;
  const packet = _buffer.Buffer.alloc(SIZE);
  packet.writeInt16BE(VERSION, 0);
  packet.fill('h', 2, 3);
  packet.writeUInt32BE(0, 4);
  packet.writeUInt32BE(timestamp, 8);
  packet.writeInt16BE(status, 12);
  packet.write(host, 14, 77, ENCODING);
  packet.write(service, 78, 206, ENCODING);
  packet.write(message, 206, SIZE, ENCODING);
  packet.writeUInt32BE(_bufferCrc.default.unsigned(packet), 4);
  return packet;
}
function xor(data, mask) {
  const dataSize = data.length;
  const maskSize = mask.length;
  const result = _buffer.Buffer.allocUnsafe(dataSize);
  let j = 0;
  for (let i = 0; i < dataSize; i++) {
    if (j === maskSize) {
      j = 0;
    }
    result[i] = data[i] ^ mask[j];
    j++;
  }
  return result;
}
const OK = 0;
exports.OK = OK;
const WARNING = 1;
exports.WARNING = WARNING;
const CRITICAL = 2;
exports.CRITICAL = CRITICAL;
const VERSION = 3;
const ENCODING = 'binary';
class XoServerNagios {
  constructor({
    xo
  }) {
    this._sendPassiveCheck = this._sendPassiveCheck.bind(this);
    this._set = xo.defineProperty.bind(xo);
    this._unset = null;
    this._conf = null;
    this._key = null;
  }
  configure(_ref) {
    let host = _ref.host,
      service = _ref.service,
      configuration = _objectWithoutProperties(_ref, _excluded);
    this._conf = configuration;
    this._key = _buffer.Buffer.from(configuration.key, ENCODING);
    const templateRules = {
      '{vm.name_label}': vmNameLabel => vmNameLabel,
      '{job.name}': (vmNameLabel, _jobName) => _jobName
    };
    this._getHost = (0, _template.compileTemplate)(host, templateRules);
    this._getService = (0, _template.compileTemplate)(service, templateRules);
  }
  load() {
    this._unset = this._set('sendPassiveCheck', this._sendPassiveCheck);
  }
  unload() {
    this._unset();
  }
  test({
    VmNameLabel,
    jobName
  }) {
    return this._sendPassiveCheck({
      message: 'The server-nagios plugin for Xen Orchestra server seems to be working fine, nicely done :)',
      status: OK
    }, VmNameLabel, jobName);
  }
  _sendPassiveCheck({
    message,
    status
  }, vmNameLabel, jobName) {
    return new Promise((resolve, reject) => {
      const conf = _objectSpread(_objectSpread({}, this._conf), {}, {
        host: this._getHost(vmNameLabel, jobName),
        service: this._getService(vmNameLabel, jobName)
      });
      if (/\r|\n/.test(message)) {
        warn('the message must not contain a line break', {
          message
        });
        for (let i = 0, n = message.length; i < n; ++i) {
          const c = message[i];
          if (c === '\n') {
            message[i] = '\\n';
          } else if (c === '\r') {
            message[i] = '\\r';
          }
        }
      }
      const client = new _net.default.Socket();
      client.connect(conf.port, conf.server, () => {
        debug('Successful connection');
      });
      client.on('data', data => {
        const timestamp = data.readInt32BE(128);
        const iv = data.slice(0, 128);
        const packet = nscaPacketBuilder(_objectSpread(_objectSpread({}, conf), {}, {
          iv,
          message,
          status,
          timestamp
        }));
        const xorPacketBuffer = xor(xor(packet, iv), this._key);
        client.write(xorPacketBuffer, res => {
          client.destroy();
          resolve(res);
        });
      });
      client.on('error', reject);
    });
  }
}
var _default = opts => new XoServerNagios(opts);
exports.default = _default;
//# sourceMappingURL=index.js.map