"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.configurationSchema = void 0;
var _slackNode = _interopRequireDefault(require("slack-node"));
var _promiseToolbox = require("promise-toolbox");
const _excluded = ["webhookUri"];
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
const logAndRethrow = error => {
  console.error('[WARN] plugin transport-slack:', error != null && error.stack || error);
  throw error;
};
const configurationSchema = {
  type: 'object',
  properties: {
    webhookUri: {
      type: 'string',
      description: 'The Mattermost or Slack webhook URL.'
    },
    channel: {
      type: 'string',
      description: 'Channel, private group, or IM channel to send message to.'
    },
    username: {
      type: 'string',
      description: 'Bot username.'
    },
    icon_emoji: {
      type: 'string',
      description: 'The bot icon. It can be a slack emoji or a URL image.'
    }
  },
  additionalProperties: false,
  required: ['webhookUri', 'channel']
};
exports.configurationSchema = configurationSchema;
class XoServerTransportSlack {
  constructor({
    xo
  }) {
    this._sendSlack = this._sendSlack.bind(this);
    this._set = xo.defineProperty.bind(xo);
    this._unset = null;
    this._conf = null;
    this._send = null;
  }
  configure(_ref) {
    let webhookUri = _ref.webhookUri,
      conf = _objectWithoutProperties(_ref, _excluded);
    const slack = new _slackNode.default();
    slack.setWebhook(webhookUri);
    this._conf = conf;
    this._send = (0, _promiseToolbox.promisify)(slack.webhook);
  }
  load() {
    this._unset = this._set('sendSlackMessage', this._sendSlack);
  }
  unload() {
    this._unset();
  }
  test() {
    return this._sendSlack({
      message: `Hi there,

The transport-slack plugin for Xen Orchestra server seems to be working fine, nicely done :)`
    });
  }
  _sendSlack({
    message
  }) {
    return this._send(_objectSpread(_objectSpread({}, this._conf), {}, {
      text: message
    })).catch(logAndRethrow);
  }
}
var _default = opts => new XoServerTransportSlack(opts);
exports.default = _default;
//# sourceMappingURL=index.js.map