"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.configurationSchema = void 0;
var _legacy = _interopRequireDefault(require("@xen-orchestra/async-map/legacy"));
var _handlebars = _interopRequireDefault(require("handlebars"));
var _humanFormat = _interopRequireDefault(require("human-format"));
var _csvStringify = require("csv-stringify");
var _log = require("@xen-orchestra/log");
var _cron = require("@xen-orchestra/cron");
var _path = require("path");
var _htmlMinifier = require("html-minifier");
var _lodash = require("lodash");
var _promiseToolbox = require("promise-toolbox");
var _fs = require("fs");
const _excluded = ["storedStatsPath"];
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }
function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }
const log = (0, _log.createLogger)('xo:xo-server-usage-report');
const GRANULARITY = 'days';
const pReadFile = (0, _promiseToolbox.promisify)(_fs.readFile);
const pWriteFile = (0, _promiseToolbox.promisify)(_fs.writeFile);
const compareOperators = {
  '>': (l, r) => l > r
};
const mathOperators = {
  '+': (l, r) => l + r
};
const gibPower = Math.pow(2, 30);
const mibPower = Math.pow(2, 20);
const kibPower = Math.pow(2, 10);
let template = null;
pReadFile((0, _path.join)(__dirname, '../report.html.tpl'), 'utf8').then(tpl => {
  template = _handlebars.default.compile((0, _htmlMinifier.minify)(tpl, {
    collapseBooleanAttributes: true,
    collapseWhitespace: true,
    minifyCSS: true,
    removeAttributeQuotes: true,
    removeComments: true,
    removeOptionalTags: true,
    removeRedundantAttributes: true
  }));
});
let imgXo = null;
pReadFile((0, _path.join)(__dirname, '../images/xo.png'), 'base64').then(data => {
  imgXo = `data:image/png;base64,${data}`;
});
const configurationSchema = {
  type: 'object',
  properties: {
    emails: {
      type: 'array',
      items: {
        type: 'string'
      }
    },
    all: {
      type: 'boolean',
      description: "It includes all resources' stats if on."
    },
    periodicity: {
      type: 'string',
      enum: ['monthly', 'weekly', 'daily'],
      description: 'If you choose weekly you will receive the report every sunday and if you choose monthly you will receive it every first day of the month.'
    }
  },
  additionalProperties: false,
  required: ['emails', 'periodicity']
};
exports.configurationSchema = configurationSchema;
const shortUuid = uuid => {
  if (typeof uuid === 'string') {
    return uuid.split('-')[0];
  }
};
const formatIops = value => (0, _lodash.isFinite)(value) ? (0, _humanFormat.default)(value, {
  unit: 'IOPS',
  decimals: 2
}) : '-';
const normaliseValue = value => (0, _lodash.isFinite)(value) ? (0, _lodash.round)(value, 2) : '-';
_handlebars.default.registerHelper('compare', function (lvalue, operator, rvalue, options) {
  if (arguments.length < 3) {
    throw new Error('Handlebars Helper "compare" needs 2 parameters');
  }
  if (!compareOperators[operator]) {
    throw new Error(`Handlebars Helper "compare" doesn't know the operator ${operator}`);
  }
  return compareOperators[operator](lvalue, rvalue) ? options.fn(this) : options.inverse(this);
});
_handlebars.default.registerHelper('math', function (lvalue, operator, rvalue, options) {
  if (arguments.length < 3) {
    throw new Error('Handlebars Helper "math" needs 2 parameters');
  }
  if (!mathOperators[operator]) {
    throw new Error(`Handlebars Helper "math" doesn't know the operator ${operator}`);
  }
  return mathOperators[operator](+lvalue, +rvalue);
});
_handlebars.default.registerHelper('shortUUID', shortUuid);
_handlebars.default.registerHelper('formatAddresses', addresses => addresses.length === 0 ? 'No IP record' : addresses.join(', '));
_handlebars.default.registerHelper('normaliseValue', normaliseValue);
_handlebars.default.registerHelper('normaliseEvolution', value => new _handlebars.default.SafeString((0, _lodash.isFinite)(value = (0, _lodash.round)(value, 2)) && value !== 0 ? value > 0 ? `(<b style="color: green;">▲ ${value}%</b>)` : `(<b style="color: red;">▼ ${String(value).slice(1)}%</b>)` : ''));
_handlebars.default.registerHelper('formatIops', formatIops);
const getHeader = (label, size) => `
  <tr>
    <td rowspan='${size + 1}' class="tableHeader">${label}</td>
  </tr>
`;
const getBody = ({
  uuid,
  name,
  value
}, transformValue, unit) => `
  <tr>
    <td>${shortUuid(uuid)}</td>
    <td>${name}</td>
    <td>${transformValue(value)}${unit !== undefined ? ` ${unit}` : ''}</td>
  </tr>
`;
const getTopIops = ({
  iopsRead,
  iopsWrite,
  iopsTotal
}) => `
  ${getHeader('IOPS read', iopsRead.length)}
  ${iopsRead.map(obj => getBody(obj, formatIops)).join('')}
  ${getHeader('IOPS write', iopsWrite.length)}
  ${iopsWrite.map(obj => getBody(obj, formatIops)).join('')}
  ${getHeader('IOPS total', iopsTotal.length)}
  ${iopsTotal.map(obj => getBody(obj, formatIops)).join('')}
`;
_handlebars.default.registerHelper('getTopSrs', ({
  usedSpace,
  iopsRead,
  iopsWrite,
  iopsTotal
}) => new _handlebars.default.SafeString(`
      ${getHeader('Used space', usedSpace.length)}
      ${usedSpace.map(obj => getBody(obj, normaliseValue, 'GiB')).join('')}
      ${getTopIops({
  iopsRead,
  iopsWrite,
  iopsTotal
})}
    `));
_handlebars.default.registerHelper('getTopIops', props => new _handlebars.default.SafeString(getTopIops(props)));
function computeMean(values) {
  let sum = 0;
  let n = 0;
  (0, _lodash.forEach)(values, val => {
    if ((0, _lodash.isFinite)(val)) {
      sum += val;
      n++;
    }
  });
  return sum / n;
}
const computeDoubleMean = val => computeMean((0, _lodash.map)(val, computeMean));
function computeMeans(objects, options) {
  return (0, _lodash.zipObject)(options, (0, _lodash.map)(options, opt => computeMean((0, _lodash.map)(objects, opt)), 2));
}
function getTop(objects, options) {
  return (0, _lodash.zipObject)(options, (0, _lodash.map)(options, opt => (0, _lodash.map)((0, _lodash.orderBy)(objects, object => {
    const value = object[opt];
    return isNaN(value) ? -Infinity : value;
  }, 'desc').slice(0, 3), obj => ({
    uuid: obj.uuid,
    name: obj.name,
    value: obj[opt]
  }))));
}
function computePercentage(curr, prev, options) {
  return (0, _lodash.zipObject)(options, (0, _lodash.map)(options, opt => prev[opt] === 0 || prev[opt] === null ? 'NONE' : `${(curr[opt] - prev[opt]) * 100 / prev[opt]}`));
}
function getDiff(oldElements, newElements) {
  return {
    added: (0, _lodash.differenceBy)(newElements, oldElements, 'uuid'),
    removed: (0, _lodash.differenceBy)(oldElements, newElements, 'uuid')
  };
}
function getMemoryUsedMetric({
  memory,
  memoryFree = memory
}) {
  return (0, _lodash.map)(memory, (value, key) => value - memoryFree[key]);
}
const METRICS_MEAN = {
  cpu: computeDoubleMean,
  disk: value => computeDoubleMean((0, _lodash.values)(value)) / mibPower,
  iops: value => computeDoubleMean((0, _lodash.values)(value)),
  load: computeMean,
  net: value => computeDoubleMean(value) / kibPower,
  ram: value => computeMean(value) / gibPower
};
const DAYS_TO_KEEP = {
  daily: 1,
  weekly: 7,
  monthly: 30
};
function getLastDays(data, periodicity) {
  const daysToKeep = DAYS_TO_KEEP[periodicity];
  const expectedData = {};
  for (const [key, value] of Object.entries(data)) {
    if (Array.isArray(value)) {
      expectedData[key] = value.slice(-daysToKeep);
    } else {
      expectedData[key] = value;
    }
  }
  return expectedData;
}
function getVmsStats(_x) {
  return _getVmsStats.apply(this, arguments);
}
function _getVmsStats() {
  _getVmsStats = _asyncToGenerator(function* ({
    runningVms,
    periodicity,
    xo
  }) {
    return (0, _lodash.orderBy)(yield Promise.all((0, _lodash.map)(runningVms, function () {
      var _ref3 = _asyncToGenerator(function* (vm) {
        const {
          stats
        } = yield xo.getXapiVmStats(vm, GRANULARITY).catch(error => {
          log.warn('Error on fetching VM stats', {
            error,
            vmId: vm.id
          });
          return {
            stats: {}
          };
        });
        const iopsRead = METRICS_MEAN.iops(getLastDays((0, _lodash.get)(stats.iops, 'r'), periodicity));
        const iopsWrite = METRICS_MEAN.iops(getLastDays((0, _lodash.get)(stats.iops, 'w'), periodicity));
        return {
          uuid: vm.uuid,
          name: vm.name_label,
          addresses: Object.values(vm.addresses),
          cpu: METRICS_MEAN.cpu(getLastDays(stats.cpus, periodicity)),
          ram: METRICS_MEAN.ram(getLastDays(getMemoryUsedMetric(stats), periodicity)),
          diskRead: METRICS_MEAN.disk(getLastDays((0, _lodash.get)(stats.xvds, 'r'), periodicity)),
          diskWrite: METRICS_MEAN.disk(getLastDays((0, _lodash.get)(stats.xvds, 'w'), periodicity)),
          iopsRead,
          iopsWrite,
          iopsTotal: iopsRead + iopsWrite,
          netReception: METRICS_MEAN.net(getLastDays((0, _lodash.get)(stats.vifs, 'rx'), periodicity)),
          netTransmission: METRICS_MEAN.net(getLastDays((0, _lodash.get)(stats.vifs, 'tx'), periodicity))
        };
      });
      return function (_x8) {
        return _ref3.apply(this, arguments);
      };
    }())), 'name', 'asc');
  });
  return _getVmsStats.apply(this, arguments);
}
function getHostsStats(_x2) {
  return _getHostsStats.apply(this, arguments);
}
function _getHostsStats() {
  _getHostsStats = _asyncToGenerator(function* ({
    runningHosts,
    periodicity,
    xo
  }) {
    return (0, _lodash.orderBy)(yield Promise.all((0, _lodash.map)(runningHosts, function () {
      var _ref4 = _asyncToGenerator(function* (host) {
        const {
          stats
        } = yield xo.getXapiHostStats(host, GRANULARITY).catch(error => {
          log.warn('Error on fetching host stats', {
            error,
            hostId: host.id
          });
          return {
            stats: {}
          };
        });
        return {
          uuid: host.uuid,
          name: host.name_label,
          cpu: METRICS_MEAN.cpu(getLastDays(stats.cpus, periodicity)),
          ram: METRICS_MEAN.ram(getLastDays(getMemoryUsedMetric(stats), periodicity)),
          load: METRICS_MEAN.load(getLastDays(stats.load, periodicity)),
          netReception: METRICS_MEAN.net(getLastDays((0, _lodash.get)(stats.pifs, 'rx'), periodicity)),
          netTransmission: METRICS_MEAN.net(getLastDays((0, _lodash.get)(stats.pifs, 'tx'), periodicity))
        };
      });
      return function (_x9) {
        return _ref4.apply(this, arguments);
      };
    }())), 'name', 'asc');
  });
  return _getHostsStats.apply(this, arguments);
}
function getSrsStats(_x3) {
  return _getSrsStats.apply(this, arguments);
}
function _getSrsStats() {
  _getSrsStats = _asyncToGenerator(function* ({
    periodicity,
    xo,
    xoObjects
  }) {
    return (0, _lodash.orderBy)(yield (0, _legacy.default)((0, _lodash.filter)(xoObjects, obj => obj.type === 'SR' && obj.size > 0 && obj.$PBDs.length > 0), function () {
      var _ref5 = _asyncToGenerator(function* (sr) {
        const totalSpace = sr.size / gibPower;
        const usedSpace = sr.physical_usage / gibPower;
        let name = sr.name_label;
        let container;
        if (!sr.shared && (container = (0, _lodash.find)(xoObjects, {
          id: sr.$container
        })) !== undefined) {
          name += ` (${container.name_label})`;
        }
        const {
          stats
        } = yield xo.getXapiSrStats(sr.id, GRANULARITY).catch(error => {
          log.warn('Error on fetching SR stats', {
            error,
            srId: sr.id
          });
          return {
            stats: {}
          };
        });
        const iopsRead = computeMean(getLastDays((0, _lodash.get)(stats.iops, 'r'), periodicity));
        const iopsWrite = computeMean(getLastDays((0, _lodash.get)(stats.iops, 'w'), periodicity));
        return {
          uuid: sr.uuid,
          name,
          total: totalSpace,
          usedSpace,
          freeSpace: totalSpace - usedSpace,
          iopsRead,
          iopsWrite,
          iopsTotal: iopsRead + iopsWrite
        };
      });
      return function (_x10) {
        return _ref5.apply(this, arguments);
      };
    }()), 'name', 'desc');
  });
  return _getSrsStats.apply(this, arguments);
}
function computeGlobalVmsStats({
  haltedVms,
  vmsStats,
  xo
}) {
  const allVms = vmsStats.map(vm => ({
    uuid: vm.uuid,
    name: vm.name
  }));
  haltedVms.forEach(vm => {
    const isReplication = 'start' in vm.blockedOperations && vm.tags.some(tag => tag === 'Disaster Recovery' || tag === 'Continuous Replication');
    if (!isReplication) {
      allVms.push({
        uuid: vm.uuid,
        name: vm.name_label
      });
    }
  });
  return Object.assign(computeMeans(vmsStats, ['cpu', 'ram', 'diskRead', 'diskWrite', 'netReception', 'netTransmission']), {
    number: vmsStats.length + haltedVms.length,
    allVms
  });
}
function computeGlobalHostsStats({
  haltedHosts,
  hostsStats,
  xo
}) {
  const allHosts = (0, _lodash.concat)((0, _lodash.map)(hostsStats, host => ({
    uuid: host.uuid,
    name: host.name
  })), (0, _lodash.map)(haltedHosts, host => ({
    uuid: host.uuid,
    name: host.name_label
  })));
  return Object.assign(computeMeans(hostsStats, ['cpu', 'ram', 'load', 'netReception', 'netTransmission']), {
    number: allHosts.length,
    allHosts
  });
}
function getTopVms({
  vmsStats,
  xo
}) {
  return getTop(vmsStats, ['cpu', 'ram', 'diskRead', 'diskWrite', 'iopsRead', 'iopsWrite', 'iopsTotal', 'netReception', 'netTransmission']);
}
function getTopHosts({
  hostsStats,
  xo
}) {
  return getTop(hostsStats, ['cpu', 'ram', 'load', 'netReception', 'netTransmission']);
}
function getTopSrs(srsStats) {
  return getTop(srsStats, ['usedSpace', 'iopsRead', 'iopsWrite', 'iopsTotal']);
}
function getHostsMissingPatches(_x4) {
  return _getHostsMissingPatches.apply(this, arguments);
}
function _getHostsMissingPatches() {
  _getHostsMissingPatches = _asyncToGenerator(function* ({
    runningHosts,
    xo
  }) {
    const hostsMissingPatches = yield Promise.all((0, _lodash.map)(runningHosts, function () {
      var _ref6 = _asyncToGenerator(function* (host) {
        let hostsPatches = yield xo.getXapi(host).listMissingPatches(host._xapiId).catch(error => {
          console.error('[WARN] error on fetching hosts missing patches:', JSON.stringify(error));
          return [];
        });
        if (host.license_params.sku_type === 'free') {
          hostsPatches = (0, _lodash.filter)(hostsPatches, {
            paid: false
          });
        }
        if (hostsPatches.length > 0) {
          return {
            uuid: host.uuid,
            name: host.name_label,
            patches: (0, _lodash.map)(hostsPatches, 'name')
          };
        }
      });
      return function (_x11) {
        return _ref6.apply(this, arguments);
      };
    }()));
    return (0, _lodash.filter)(hostsMissingPatches, host => host !== undefined);
  });
  return _getHostsMissingPatches.apply(this, arguments);
}
function getAllUsersEmail(users) {
  return (0, _lodash.map)(users, 'email');
}
function storeStats(_x5) {
  return _storeStats.apply(this, arguments);
}
function _storeStats() {
  _storeStats = _asyncToGenerator(function* ({
    data,
    storedStatsPath
  }) {
    yield pWriteFile(storedStatsPath, JSON.stringify(data));
  });
  return _storeStats.apply(this, arguments);
}
function computeEvolution(_x6) {
  return _computeEvolution.apply(this, arguments);
}
function _computeEvolution() {
  _computeEvolution = _asyncToGenerator(function* (_ref) {
    let {
        storedStatsPath
      } = _ref,
      newStats = _objectWithoutProperties(_ref, _excluded);
    try {
      const oldStats = JSON.parse(yield pReadFile(storedStatsPath, 'utf8'));
      const newStatsVms = newStats.vms;
      const oldStatsVms = oldStats.global.vms;
      const newStatsHosts = newStats.hosts;
      const oldStatsHosts = oldStats.global.hosts;
      const prevDate = oldStats.style.currDate;
      const resourcesOptions = {
        vms: ['cpu', 'ram', 'diskRead', 'diskWrite', 'iopsRead', 'iopsWrite', 'iopsTotal', 'netReception', 'netTransmission'],
        hosts: ['cpu', 'ram', 'load', 'netReception', 'netTransmission'],
        srs: ['total']
      };
      const vmsEvolution = _objectSpread({
        number: newStatsVms.number - oldStatsVms.number
      }, computePercentage(newStatsVms, oldStatsVms, resourcesOptions.vms));
      const hostsEvolution = _objectSpread({
        number: newStatsHosts.number - oldStatsHosts.number
      }, computePercentage(newStatsHosts, oldStatsHosts, resourcesOptions.hosts));
      const vmsResourcesEvolution = getDiff(oldStatsVms.allVms, newStatsVms.allVms);
      const hostsResourcesEvolution = getDiff(oldStatsHosts.allHosts, newStatsHosts.allHosts);
      const usersEvolution = getDiff(oldStats.users, newStats.users);
      const newAllResourcesStats = newStats.allResources;
      const oldAllResourcesStats = oldStats.allResources;
      if (newAllResourcesStats !== undefined && oldAllResourcesStats !== undefined) {
        (0, _lodash.forEach)(newAllResourcesStats, (resource, key) => {
          const option = resourcesOptions[key];
          if (option !== undefined) {
            (0, _lodash.forEach)(resource, newItem => {
              const oldItem = (0, _lodash.find)(oldAllResourcesStats[key], {
                uuid: newItem.uuid
              });
              if (oldItem !== undefined) {
                newItem.evolution = computePercentage(newItem, oldItem, option);
              }
            });
          }
        });
      }
      return {
        vmsEvolution,
        hostsEvolution,
        prevDate,
        vmsResourcesEvolution,
        hostsResourcesEvolution,
        usersEvolution
      };
    } catch (err) {
      if (err.code !== 'ENOENT') throw err;
    }
  });
  return _computeEvolution.apply(this, arguments);
}
function dataBuilder(_x7) {
  return _dataBuilder.apply(this, arguments);
}
function _dataBuilder() {
  _dataBuilder = _asyncToGenerator(function* ({
    currDate,
    periodicity,
    xo,
    storedStatsPath,
    all
  }) {
    const xoObjects = (0, _lodash.values)(xo.getObjects());
    const runningVms = (0, _lodash.filter)(xoObjects, {
      type: 'VM',
      power_state: 'Running'
    });
    const haltedVms = (0, _lodash.filter)(xoObjects, {
      type: 'VM',
      power_state: 'Halted'
    });
    const runningHosts = (0, _lodash.filter)(xoObjects, {
      type: 'host',
      power_state: 'Running'
    });
    const haltedHosts = (0, _lodash.filter)(xoObjects, {
      type: 'host',
      power_state: 'Halted'
    });
    const [users, vmsStats, hostsStats, srsStats, hostsMissingPatches] = yield Promise.all([xo.getAllUsers(), getVmsStats({
      xo,
      runningVms,
      periodicity
    }), getHostsStats({
      xo,
      runningHosts,
      periodicity
    }), getSrsStats({
      xo,
      xoObjects,
      periodicity
    }), getHostsMissingPatches({
      xo,
      runningHosts
    })]);
    const [globalVmsStats, globalHostsStats, topVms, topHosts, topSrs, usersEmail] = yield Promise.all([computeGlobalVmsStats({
      xo,
      vmsStats,
      haltedVms
    }), computeGlobalHostsStats({
      xo,
      hostsStats,
      haltedHosts
    }), getTopVms({
      xo,
      vmsStats
    }), getTopHosts({
      xo,
      hostsStats
    }), getTopSrs(srsStats), getAllUsersEmail(users)]);
    let allResources;
    if (all) {
      allResources = {
        vms: vmsStats,
        hosts: hostsStats,
        srs: srsStats,
        date: currDate
      };
    }
    const evolution = yield computeEvolution({
      allResources,
      storedStatsPath,
      hosts: globalHostsStats,
      usersEmail,
      vms: globalVmsStats
    });
    return {
      allResources,
      global: {
        vms: globalVmsStats,
        hosts: globalHostsStats,
        vmsEvolution: evolution && evolution.vmsEvolution,
        hostsEvolution: evolution && evolution.hostsEvolution
      },
      topHosts,
      topSrs,
      topVms,
      hostsMissingPatches,
      usersEmail,
      vmsResourcesEvolution: evolution && evolution.vmsResourcesEvolution,
      hostsResourcesEvolution: evolution && evolution.hostsResourcesEvolution,
      usersEvolution: evolution && evolution.usersEvolution,
      style: {
        imgXo,
        currDate,
        prevDate: evolution && evolution.prevDate,
        page: '{{page}}'
      }
    };
  });
  return _dataBuilder.apply(this, arguments);
}
const CRON_BY_PERIODICITY = {
  monthly: '0 6 1 * *',
  weekly: '0 6 * * 0',
  daily: '0 6 * * *'
};
const CSV_CAST = {
  number: value => Number.isNaN(value) ? undefined : String(value),
  string: value => value === 'NONE' ? undefined : value
};
const CSV_COLUMNS = {
  addresses: {
    key: 'addresses',
    header: 'IP addresses'
  },
  cpu: {
    key: 'cpu',
    header: 'CPU (%)'
  },
  cpuEvolution: {
    key: 'evolution.cpu',
    header: 'CPU evolution (%)'
  },
  diskRead: {
    key: 'diskRead',
    header: 'Disk read (MiB)'
  },
  diskReadEvolution: {
    key: 'evolution.diskRead',
    header: 'Disk read evolution (%)'
  },
  diskWrite: {
    key: 'diskWrite',
    header: 'Disk write (MiB)'
  },
  diskWriteEvolution: {
    key: 'evolution.diskWrite',
    header: 'Disk write evolution (%)'
  },
  iopsRead: {
    key: 'iopsRead',
    header: 'IOPS read'
  },
  iopsReadEvolution: {
    key: 'evolution.iopsRead',
    header: 'IOPS read evolution (%)'
  },
  iopsTotal: {
    key: 'iopsTotal',
    header: 'IOPS total'
  },
  iopsTotalEvolution: {
    key: 'evolution.iopsTotal',
    header: 'IOPS total evolution (%)'
  },
  iopsWrite: {
    key: 'iopsWrite',
    header: 'IOPS write'
  },
  iopsWriteEvolution: {
    key: 'evolution.iopsWrite',
    header: 'IOPS write evolution (%)'
  },
  load: {
    key: 'load',
    header: 'Load average'
  },
  loadEvolution: {
    key: 'evolution.load',
    header: 'Load average evolution (%)'
  },
  name: {
    key: 'name',
    header: 'Name'
  },
  netReception: {
    key: 'netReception',
    header: 'Network RX (KiB)'
  },
  netReceptionEvolution: {
    key: 'evolution.netReception',
    header: 'Network RX evolution (%)'
  },
  netTransmission: {
    key: 'netTransmission',
    header: 'Network TX (KiB)'
  },
  netTransmissionEvolution: {
    key: 'evolution.netTransmission',
    header: 'Network TX evolution (%)'
  },
  ram: {
    key: 'ram',
    header: 'RAM (GiB)'
  },
  ramEvolution: {
    key: 'evolution.ram',
    header: 'RAM evolution (%)'
  },
  spaceFree: {
    key: 'freeSpace',
    header: 'Free space (GiB)'
  },
  spaceTotal: {
    key: 'total',
    header: 'Total space (GiB)'
  },
  spaceTotalEvolution: {
    key: 'evolution.total',
    header: 'Total space evolution (%)'
  },
  spaceUsed: {
    key: 'usedSpace',
    header: 'Used space (GiB)'
  },
  uuid: {
    key: 'uuid',
    header: 'UUID'
  }
};
class UsageReportPlugin {
  constructor({
    xo,
    getDataDir
  }) {
    this._xo = xo;
    this._dir = getDataDir;
    this._conf = null;
    this._xo.addApiMethod('plugin.usageReport.send', this._sendReport.bind(this, false));
  }
  configure(configuration, state) {
    var _this = this;
    this._conf = configuration;
    if (this._job !== undefined) {
      this._job.stop();
    }
    this._job = (0, _cron.createSchedule)(CRON_BY_PERIODICITY[configuration.periodicity]).createJob(_asyncToGenerator(function* () {
      try {
        yield _this._sendReport(true);
      } catch (error) {
        console.error('[WARN] scheduled function:', error && error.stack || error);
      }
    }));
    if (state.loaded) {
      this._job.start();
    }
  }
  load() {
    var _this2 = this;
    return _asyncToGenerator(function* () {
      const dir = yield _this2._dir();
      _this2._storedStatsPath = `${dir}/stats.json`;
      _this2._job.start();
    })();
  }
  unload() {
    this._job.stop();
  }
  test() {
    return this._sendReport(true);
  }
  _sendReport(storeData) {
    var _this3 = this;
    return _asyncToGenerator(function* () {
      const xo = _this3._xo;
      if (xo.sendEmail === undefined) {
        _promiseToolbox.ignoreErrors.call(xo.unloadPlugin('usage-report'));
        throw new Error('The plugin usage-report requires the plugin transport-email to be loaded');
      }
      const currDate = new Date().toISOString().slice(0, 10);
      const data = yield dataBuilder({
        currDate,
        periodicity: _this3._conf.periodicity,
        xo,
        storedStatsPath: _this3._storedStatsPath,
        all: _this3._conf.all
      });
      const attachments = [{
        filename: `xoReport_${currDate}.html`,
        content: template(data)
      }];
      if (data.allResources !== undefined) {
        attachments.push({
          filename: `xoReport_${currDate}_vms.csv`,
          content: (0, _csvStringify.stringify)(data.allResources.vms, {
            cast: CSV_CAST,
            header: true,
            columns: [CSV_COLUMNS.uuid, CSV_COLUMNS.name, CSV_COLUMNS.addresses, CSV_COLUMNS.cpu, CSV_COLUMNS.cpuEvolution, CSV_COLUMNS.ram, CSV_COLUMNS.ramEvolution, CSV_COLUMNS.diskRead, CSV_COLUMNS.diskReadEvolution, CSV_COLUMNS.diskWrite, CSV_COLUMNS.diskWriteEvolution, CSV_COLUMNS.iopsRead, CSV_COLUMNS.iopsReadEvolution, CSV_COLUMNS.iopsWrite, CSV_COLUMNS.iopsWriteEvolution, CSV_COLUMNS.iopsTotal, CSV_COLUMNS.iopsTotalEvolution, CSV_COLUMNS.netReception, CSV_COLUMNS.netReceptionEvolution, CSV_COLUMNS.netTransmission, CSV_COLUMNS.netTransmissionEvolution]
          })
        }, {
          filename: `xoReport_${currDate}_hosts.csv`,
          content: (0, _csvStringify.stringify)(data.allResources.hosts, {
            cast: CSV_CAST,
            header: true,
            columns: [CSV_COLUMNS.uuid, CSV_COLUMNS.name, CSV_COLUMNS.cpu, CSV_COLUMNS.cpuEvolution, CSV_COLUMNS.ram, CSV_COLUMNS.ramEvolution, CSV_COLUMNS.load, CSV_COLUMNS.loadEvolution, CSV_COLUMNS.netReception, CSV_COLUMNS.netReceptionEvolution, CSV_COLUMNS.netTransmission, CSV_COLUMNS.netTransmissionEvolution]
          })
        }, {
          filename: `xoReport_${currDate}_srs.csv`,
          content: (0, _csvStringify.stringify)(data.allResources.srs, {
            cast: CSV_CAST,
            header: true,
            columns: [CSV_COLUMNS.uuid, CSV_COLUMNS.name, CSV_COLUMNS.spaceTotal, CSV_COLUMNS.spaceTotalEvolution, CSV_COLUMNS.spaceUsed, CSV_COLUMNS.spaceFree]
          })
        });
      }
      yield Promise.all([xo.sendEmail({
        to: _this3._conf.emails,
        subject: `[Xen Orchestra] Xo Report - ${currDate}`,
        markdown: `Hi there,

  You have chosen to receive your xo report ${_this3._conf.periodicity}.
  Please, find the attached report.

  best regards.`,
        attachments
      }), storeData && storeStats({
        data,
        storedStatsPath: _this3._storedStatsPath
      })]);
    })();
  }
}
var _default = opts => new UsageReportPlugin(opts);
exports.default = _default;
//# sourceMappingURL=index.js.map