function getOrCreate(map, key, Klass) {
  let value = map.get(key);
  if (value === undefined) {
    value = new Klass();
    map.set(key, value);
  }
  return value;
}
function addMaybeMultiple(set, value) {
  if (Array.isArray(value)) {
    for (const item of value) {
      set.add(item);
    }
  } else {
    set.add(value);
  }
}
export default function normalizeVmNetworks(networks) {
  const byDevice = new Map();
  for (const key of Object.keys(networks).sort()) {
    let address = networks[key].trim();
    if (address === '') {
      continue;
    }
    const matches = /^(\d+)\/(?:ip|(ipv[46])\/.+)$/.exec(key);
    if (matches === null) {
      continue;
    }
    let [, device, protocol] = matches;
    if (protocol === undefined) {
      protocol = 'ipv4';
      address = address.split(/\s+/);
    }
    addMaybeMultiple(getOrCreate(getOrCreate(byDevice, device, Map), protocol, Set), address);
  }
  const addressesMap = {};
  for (const [device, byProtocol] of byDevice.entries()) {
    for (const [protocol, addresses] of byProtocol.entries()) {
      let i = 0;
      for (const address of addresses) {
        addressesMap[`${device}/${protocol}/${i++}`] = address;
      }
    }
  }
  return addressesMap;
}
//# sourceMappingURL=_normalizeVmNetworks.mjs.map