import { createPredicate } from 'value-matcher';
import { extractIdsFromSimplePattern } from '@xen-orchestra/backups/extractIdsFromSimplePattern.js';
import { forbiddenOperation } from 'xo-common/api-errors.js';
export default async function backupGuard(poolId) {
  const jobs = await this.getAllJobs('backup');
  const guard = id => {
    if (this.getObject(id).$poolId === poolId) {
      throw forbiddenOperation('Backup is running', `A backup is running on the pool: ${poolId}`);
    }
  };
  jobs.forEach(({
    runId,
    vms
  }) => {
    if (runId !== undefined) {
      if (vms.id !== undefined) {
        extractIdsFromSimplePattern(vms).forEach(guard);
      } else {
        const isPoolSafe = vms.$pool === undefined ? false : !createPredicate(vms.$pool)(poolId);
        if (!isPoolSafe) {
          throw forbiddenOperation('May have running backup', `A backup may run on the pool: ${poolId}`);
        }
      }
    }
  });
}
//# sourceMappingURL=_backupGuard.mjs.map