import { ignoreErrors } from 'promise-toolbox';
export async function add({
  autoConnect = true,
  ...props
}) {
  const server = await this.registerXenServer(props);
  if (autoConnect) {
    var _context;
    (_context = this.connectXenServer(server.id), ignoreErrors).call(_context);
  }
  return server.id;
}
add.description = 'register a new Xen server';
add.permission = 'admin';
add.params = {
  label: {
    optional: true,
    type: 'string'
  },
  host: {
    type: 'string'
  },
  username: {
    type: 'string'
  },
  password: {
    type: 'string'
  },
  autoConnect: {
    optional: true,
    type: 'boolean'
  },
  allowUnauthorized: {
    optional: true,
    type: 'boolean'
  },
  httpProxy: {
    optional: true,
    type: 'string'
  }
};
export async function remove({
  id
}) {
  await this.unregisterXenServer(id);
}
remove.description = 'unregister a Xen server';
remove.permission = 'admin';
remove.params = {
  id: {
    type: 'string'
  }
};
export function getAll() {
  return this.getAllXenServers();
}
getAll.description = 'returns all the registered Xen server';
getAll.permission = 'admin';
export async function set({
  id,
  ...props
}) {
  await this.updateXenServer(id, props);
}
set.description = 'changes the properties of a Xen server';
set.permission = 'admin';
set.params = {
  id: {
    type: 'string'
  },
  label: {
    type: 'string',
    optional: true
  },
  host: {
    type: 'string',
    optional: true
  },
  username: {
    type: 'string',
    optional: true
  },
  password: {
    type: 'string',
    optional: true
  },
  allowUnauthorized: {
    optional: true,
    type: 'boolean'
  },
  readOnly: {
    optional: true,
    type: 'boolean'
  },
  httpProxy: {
    optional: true,
    type: ['string', 'null']
  }
};
export async function enable({
  id
}) {
  var _context2;
  (_context2 = this.updateXenServer(id, {
    enabled: true
  }), ignoreErrors).call(_context2);
  await this.connectXenServer(id);
}
enable.description = 'enable a Xen server';
enable.permission = 'admin';
enable.params = {
  id: {
    type: 'string'
  }
};
export async function disable({
  id
}) {
  var _context3;
  (_context3 = this.updateXenServer(id, {
    enabled: false
  }), ignoreErrors).call(_context3);
  await this.disconnectXenServer(id);
}
disable.description = 'disable a Xen server';
disable.permission = 'admin';
disable.params = {
  id: {
    type: 'string'
  }
};
//# sourceMappingURL=server.mjs.map