import asyncMapSettled from '@xen-orchestra/async-map/legacy.js';
import filter from 'lodash/filter.js';
import isEmpty from 'lodash/isEmpty.js';
import some from 'lodash/some.js';
import ensureArray from '../_ensureArray.mjs';
import { asInteger } from '../xapi/utils.mjs';
import { debounceWithKey } from '../_pDebounceWithKey.mjs';
import { forEach, parseXml } from '../utils.mjs';
export async function set({
  sr,
  name_description: nameDescription,
  name_label: nameLabel
}) {
  sr = this.getXapiObject(sr);
  await Promise.all([nameDescription !== undefined && sr.set_name_description(nameDescription), nameLabel !== undefined && sr.set_name_label(nameLabel)]);
}
set.params = {
  id: {
    type: 'string'
  },
  name_label: {
    type: 'string',
    optional: true
  },
  name_description: {
    type: 'string',
    minLength: 0,
    optional: true
  }
};
set.resolve = {
  sr: ['id', 'SR', 'operate']
};
export async function scan({
  SR
}) {
  await this.getXapi(SR).callAsync('SR.scan', SR._xapiRef);
}
scan.params = {
  id: {
    type: 'string'
  }
};
scan.resolve = {
  SR: ['id', 'SR', 'operate']
};
const srIsBackingHa = sr => sr.$pool.ha_enabled && some(sr.$pool.$ha_statefiles, f => f.$SR === sr);
export async function destroy({
  sr
}) {
  const xapi = this.getXapi(sr);
  if (sr.SR_type !== 'xosan') {
    await xapi.destroySr(sr._xapiId);
    return;
  }
  const xapiSr = xapi.getObject(sr);
  if (srIsBackingHa(xapiSr)) {
    throw new Error('You tried to remove a SR the High Availability is relying on. Please disable HA first.');
  }
  const config = xapi.xo.getData(sr, 'xosan_config');
  await xapi.forgetSr(sr._xapiId);
  await asyncMapSettled(config.nodes, node => this.getXapiObject(node.vm.id).$destroy());
  await xapi.deleteNetwork(config.network);
  if (sr.SR_type === 'xosan') {
    await this.unbindXosanLicense({
      srId: sr.id
    });
  }
}
destroy.params = {
  id: {
    type: 'string'
  }
};
destroy.resolve = {
  sr: ['id', 'SR', 'administrate']
};
export async function forget({
  SR
}) {
  await this.getXapi(SR).forgetSr(SR._xapiId);
}
forget.params = {
  id: {
    type: 'string'
  }
};
forget.resolve = {
  SR: ['id', 'SR', 'administrate']
};
export async function connectAllPbds({
  SR
}) {
  await this.getXapi(SR).connectAllSrPbds(SR._xapiId);
}
connectAllPbds.params = {
  id: {
    type: 'string'
  }
};
connectAllPbds.resolve = {
  SR: ['id', 'SR', 'administrate']
};
export async function disconnectAllPbds({
  SR
}) {
  await this.getXapi(SR).disconnectAllSrPbds(SR._xapiId);
}
disconnectAllPbds.params = {
  id: {
    type: 'string'
  }
};
disconnectAllPbds.resolve = {
  SR: ['id', 'SR', 'administrate']
};
export async function createIso({
  host,
  nameLabel,
  nameDescription,
  path,
  type,
  user,
  password,
  nfsVersion,
  nfsOptions,
  srUuid
}) {
  const xapi = this.getXapi(host);
  const deviceConfig = {};
  if (type === 'local') {
    deviceConfig.legacy_mode = 'true';
  } else if (type === 'smb') {
    path = path.replace(/\\/g, '/');
    deviceConfig.type = 'cifs';
    deviceConfig.username = user;
    deviceConfig.cifspassword = password;
  } else if (type === 'nfs') {
    if (nfsVersion !== undefined) {
      deviceConfig.nfsversion = nfsVersion;
    }
    if (nfsOptions !== undefined) {
      deviceConfig.options = nfsOptions;
    }
  }
  deviceConfig.location = path;
  if (srUuid !== undefined) {
    return xapi.reattachSr({
      uuid: srUuid,
      nameLabel,
      nameDescription,
      type,
      deviceConfig
    });
  }
  const srRef = await xapi.SR_create({
    content_type: 'iso',
    device_config: deviceConfig,
    host: host._xapiRef,
    name_description: nameDescription,
    name_label: nameLabel,
    shared: type !== 'local',
    type: 'iso'
  });
  const sr = await xapi.call('SR.get_record', srRef);
  return sr.uuid;
}
createIso.params = {
  host: {
    type: 'string'
  },
  nameLabel: {
    type: 'string'
  },
  nameDescription: {
    type: 'string',
    minLength: 0
  },
  path: {
    type: 'string'
  },
  type: {
    type: 'string'
  },
  user: {
    type: 'string',
    optional: true
  },
  password: {
    type: 'string',
    optional: true
  },
  nfsVersion: {
    type: 'string',
    optional: true
  },
  nfsOptions: {
    type: 'string',
    optional: true
  },
  srUuid: {
    type: 'string',
    optional: true
  }
};
createIso.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function createNfs({
  host,
  nameLabel,
  nameDescription,
  server,
  serverPath,
  nfsVersion,
  nfsOptions,
  srUuid
}) {
  const xapi = this.getXapi(host);
  const deviceConfig = {
    server,
    serverpath: serverPath
  };
  if (nfsVersion) {
    deviceConfig.nfsversion = nfsVersion;
  }
  if (nfsOptions) {
    deviceConfig.options = nfsOptions;
  }
  if (srUuid !== undefined) {
    return xapi.reattachSr({
      uuid: srUuid,
      nameLabel,
      nameDescription,
      type: 'nfs',
      deviceConfig
    });
  }
  const srRef = await xapi.SR_create({
    device_config: deviceConfig,
    host: host._xapiRef,
    name_description: nameDescription,
    name_label: nameLabel,
    shared: true,
    type: 'nfs'
  });
  const sr = await xapi.call('SR.get_record', srRef);
  return sr.uuid;
}
createNfs.params = {
  host: {
    type: 'string'
  },
  nameLabel: {
    type: 'string'
  },
  nameDescription: {
    type: 'string',
    minLength: 0
  },
  server: {
    type: 'string'
  },
  serverPath: {
    type: 'string'
  },
  nfsVersion: {
    type: 'string',
    optional: true
  },
  nfsOptions: {
    type: 'string',
    optional: true
  },
  srUuid: {
    type: 'string',
    optional: true
  }
};
createNfs.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function createHba({
  host,
  nameLabel,
  nameDescription,
  scsiId,
  srUuid
}) {
  const xapi = this.getXapi(host);
  const deviceConfig = {
    SCSIid: scsiId
  };
  if (srUuid !== undefined) {
    return xapi.reattachSr({
      uuid: srUuid,
      nameLabel,
      nameDescription,
      type: 'hba',
      deviceConfig
    });
  }
  const srRef = await xapi.SR_create({
    device_config: deviceConfig,
    host: host._xapiRef,
    name_description: nameDescription,
    name_label: nameLabel,
    shared: true,
    type: 'lvmohba'
  });
  const sr = await xapi.call('SR.get_record', srRef);
  return sr.uuid;
}
createHba.params = {
  host: {
    type: 'string'
  },
  nameLabel: {
    type: 'string'
  },
  nameDescription: {
    type: 'string',
    minLength: 0
  },
  scsiId: {
    type: 'string'
  },
  srUuid: {
    type: 'string',
    optional: true
  }
};
createHba.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function createLvm({
  host,
  nameLabel,
  nameDescription,
  device
}) {
  const xapi = this.getXapi(host);
  const deviceConfig = {
    device
  };
  const srRef = await xapi.SR_create({
    device_config: deviceConfig,
    host: host._xapiRef,
    name_description: nameDescription,
    name_label: nameLabel,
    shared: false,
    type: 'lvm'
  });
  const sr = await xapi.call('SR.get_record', srRef);
  return sr.uuid;
}
createLvm.params = {
  host: {
    type: 'string'
  },
  nameLabel: {
    type: 'string'
  },
  nameDescription: {
    type: 'string',
    minLength: 0
  },
  device: {
    type: 'string'
  }
};
createLvm.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function createExt({
  host,
  nameLabel,
  nameDescription,
  device
}) {
  const xapi = this.getXapi(host);
  const deviceConfig = {
    device
  };
  const srRef = await xapi.SR_create({
    device_config: deviceConfig,
    host: host._xapiRef,
    name_description: nameDescription,
    name_label: nameLabel,
    shared: false,
    type: 'ext'
  });
  const sr = await xapi.call('SR.get_record', srRef);
  return sr.uuid;
}
createExt.params = {
  host: {
    type: 'string'
  },
  nameLabel: {
    type: 'string'
  },
  nameDescription: {
    type: 'string',
    minLength: 0
  },
  device: {
    type: 'string'
  }
};
createExt.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function probeZfs({
  host
}) {
  const xapi = this.getXapi(host);
  try {
    const result = await xapi.call('host.call_plugin', host._xapiRef, 'zfs.py', 'list_zfs_pools', {});
    return JSON.parse(result);
  } catch (error) {
    if (error.code === 'XENAPI_MISSING_PLUGIN' || error.code === 'UNKNOWN_XENAPI_PLUGIN_FUNCTION') {
      return {};
    } else {
      throw error;
    }
  }
}
probeZfs.params = {
  host: {
    type: 'string'
  }
};
probeZfs.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function createZfs({
  host,
  nameLabel,
  nameDescription,
  location
}) {
  const xapi = this.getXapi(host);
  const types = await xapi.call('SR.get_supported_types');
  return await xapi.getField('SR', await xapi.SR_create({
    device_config: {
      location
    },
    host: host._xapiRef,
    name_description: nameDescription,
    name_label: nameLabel,
    shared: false,
    type: types.includes('zfs') ? 'zfs' : 'file'
  }), 'uuid');
}
createZfs.params = {
  host: {
    type: 'string'
  },
  nameLabel: {
    type: 'string'
  },
  nameDescription: {
    type: 'string',
    minLength: 0
  },
  location: {
    type: 'string'
  }
};
createZfs.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function probeNfs({
  host,
  server
}) {
  const xapi = this.getXapi(host);
  const deviceConfig = {
    server
  };
  let xml;
  try {
    await xapi.call('SR.probe', host._xapiRef, deviceConfig, 'nfs', {});
    throw new Error('the call above should have thrown an error');
  } catch (error) {
    if (error.code !== 'SR_BACKEND_FAILURE_101') {
      throw error;
    }
    xml = parseXml(error.params[2]);
  }
  const nfsExports = [];
  forEach(ensureArray(xml['nfs-exports'].Export), nfsExport => {
    nfsExports.push({
      path: nfsExport.Path.trim(),
      acl: nfsExport.Accesslist.trim()
    });
  });
  return nfsExports;
}
probeNfs.params = {
  host: {
    type: 'string'
  },
  server: {
    type: 'string'
  }
};
probeNfs.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function probeHba({
  host
}) {
  const xapi = this.getXapi(host);
  let xml;
  try {
    await xapi.call('SR.probe', host._xapiRef, {}, 'lvmohba', {});
    throw new Error('the call above should have thrown an error');
  } catch (error) {
    if (error.code !== 'SR_BACKEND_FAILURE_107') {
      throw error;
    }
    xml = parseXml(error.params[2]);
  }
  const hbaDevices = [];
  forEach(ensureArray(xml.Devlist.BlockDevice), hbaDevice => {
    hbaDevices.push({
      hba: hbaDevice.hba.trim(),
      id: hbaDevice.id.trim(),
      path: hbaDevice.path.trim(),
      scsiId: hbaDevice.SCSIid.trim(),
      serial: hbaDevice.serial.trim(),
      size: +hbaDevice.size.trim(),
      vendor: hbaDevice.vendor.trim()
    });
  });
  return hbaDevices;
}
probeHba.params = {
  host: {
    type: 'string'
  }
};
probeHba.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function createIscsi({
  host,
  nameLabel,
  nameDescription,
  size,
  target,
  port,
  targetIqn,
  scsiId,
  chapUser,
  chapPassword,
  srUuid
}) {
  const xapi = this.getXapi(host);
  const deviceConfig = {
    target,
    targetIQN: targetIqn,
    SCSIid: scsiId
  };
  if (chapUser && chapPassword) {
    deviceConfig.chapuser = chapUser;
    deviceConfig.chappassword = chapPassword;
  }
  if (port) {
    deviceConfig.port = asInteger(port);
  }
  if (srUuid !== undefined) {
    return xapi.reattachSr({
      uuid: srUuid,
      nameLabel,
      nameDescription,
      type: 'lvmoiscsi',
      deviceConfig
    });
  }
  const srRef = await xapi.SR_create({
    device_config: deviceConfig,
    host: host._xapiRef,
    name_description: nameDescription,
    name_label: nameLabel,
    shared: true,
    type: 'lvmoiscsi'
  });
  const sr = await xapi.call('SR.get_record', srRef);
  return sr.uuid;
}
createIscsi.params = {
  host: {
    type: 'string'
  },
  nameLabel: {
    type: 'string'
  },
  nameDescription: {
    type: 'string',
    minLength: 0
  },
  target: {
    type: 'string'
  },
  port: {
    type: 'integer',
    optional: true
  },
  targetIqn: {
    type: 'string'
  },
  scsiId: {
    type: 'string'
  },
  chapUser: {
    type: 'string',
    optional: true
  },
  chapPassword: {
    type: 'string',
    optional: true
  },
  srUuid: {
    type: 'string',
    optional: true
  }
};
createIscsi.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function probeIscsiIqns({
  host,
  target: targetIp,
  port,
  chapUser,
  chapPassword
}) {
  const xapi = this.getXapi(host);
  const deviceConfig = {
    target: targetIp
  };
  if (chapUser && chapPassword) {
    deviceConfig.chapuser = chapUser;
    deviceConfig.chappassword = chapPassword;
  }
  if (port) {
    deviceConfig.port = asInteger(port);
  }
  let xml;
  try {
    await xapi.call('SR.probe', host._xapiRef, deviceConfig, 'lvmoiscsi', {});
    throw new Error('the call above should have thrown an error');
  } catch (error) {
    if (error.code === 'SR_BACKEND_FAILURE_141') {
      return [];
    }
    if (error.code !== 'SR_BACKEND_FAILURE_96') {
      throw error;
    }
    xml = parseXml(error.params[2]);
  }
  const targets = [];
  forEach(ensureArray(xml['iscsi-target-iqns'].TGT), target => {
    if (target.IPAddress.trim() === targetIp) {
      targets.push({
        iqn: target.TargetIQN.trim(),
        ip: target.IPAddress.trim()
      });
    }
  });
  return targets;
}
probeIscsiIqns.params = {
  host: {
    type: 'string'
  },
  target: {
    type: 'string'
  },
  port: {
    type: 'integer',
    optional: true
  },
  chapUser: {
    type: 'string',
    optional: true
  },
  chapPassword: {
    type: 'string',
    optional: true
  }
};
probeIscsiIqns.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function probeIscsiLuns({
  host,
  target: targetIp,
  port,
  targetIqn,
  chapUser,
  chapPassword
}) {
  const xapi = this.getXapi(host);
  const deviceConfig = {
    target: targetIp,
    targetIQN: targetIqn
  };
  if (chapUser && chapPassword) {
    deviceConfig.chapuser = chapUser;
    deviceConfig.chappassword = chapPassword;
  }
  if (port) {
    deviceConfig.port = asInteger(port);
  }
  let xml;
  try {
    await xapi.call('SR.probe', host._xapiRef, deviceConfig, 'lvmoiscsi', {});
    throw new Error('the call above should have thrown an error');
  } catch (error) {
    if (error.code !== 'SR_BACKEND_FAILURE_107') {
      throw error;
    }
    xml = parseXml(error.params[2]);
  }
  const luns = [];
  forEach(ensureArray(xml['iscsi-target'].LUN), lun => {
    var _lun$size;
    luns.push({
      id: lun.LUNid.trim(),
      vendor: lun.vendor.trim(),
      serial: lun.serial.trim(),
      size: (_lun$size = lun.size) === null || _lun$size === void 0 ? void 0 : _lun$size.trim(),
      scsiId: lun.SCSIid.trim()
    });
  });
  return luns;
}
probeIscsiLuns.params = {
  host: {
    type: 'string'
  },
  target: {
    type: 'string'
  },
  port: {
    type: 'integer',
    optional: true
  },
  targetIqn: {
    type: 'string'
  },
  chapUser: {
    type: 'string',
    optional: true
  },
  chapPassword: {
    type: 'string',
    optional: true
  }
};
probeIscsiLuns.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function probeIscsiExists({
  host,
  target: targetIp,
  port,
  targetIqn,
  scsiId,
  chapUser,
  chapPassword
}) {
  const xapi = this.getXapi(host);
  const deviceConfig = {
    target: targetIp,
    targetIQN: targetIqn,
    SCSIid: scsiId
  };
  if (chapUser && chapPassword) {
    deviceConfig.chapuser = chapUser;
    deviceConfig.chappassword = chapPassword;
  }
  if (port) {
    deviceConfig.port = asInteger(port);
  }
  const xml = parseXml(await xapi.call('SR.probe', host._xapiRef, deviceConfig, 'lvmoiscsi', {}));
  const srs = [];
  forEach(ensureArray(xml.SRlist.SR), sr => {
    srs.push({
      uuid: sr.UUID.trim()
    });
  });
  return srs;
}
probeIscsiExists.params = {
  host: {
    type: 'string'
  },
  target: {
    type: 'string'
  },
  port: {
    type: 'integer',
    optional: true
  },
  targetIqn: {
    type: 'string'
  },
  scsiId: {
    type: 'string'
  },
  chapUser: {
    type: 'string',
    optional: true
  },
  chapPassword: {
    type: 'string',
    optional: true
  }
};
probeIscsiExists.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function probeHbaExists({
  host,
  scsiId
}) {
  const xapi = this.getXapi(host);
  const deviceConfig = {
    SCSIid: scsiId
  };
  const xml = parseXml(await xapi.call('SR.probe', host._xapiRef, deviceConfig, 'lvmohba', {}));
  return ensureArray(xml.SRlist.SR).map(sr => ({
    uuid: sr.UUID.trim()
  }));
}
probeHbaExists.params = {
  host: {
    type: 'string'
  },
  scsiId: {
    type: 'string'
  }
};
probeHbaExists.resolve = {
  host: ['host', 'host', 'administrate']
};
export async function probeNfsExists({
  host,
  server,
  serverPath
}) {
  const xapi = this.getXapi(host);
  const deviceConfig = {
    server,
    serverpath: serverPath
  };
  const xml = parseXml(await xapi.call('SR.probe', host._xapiRef, deviceConfig, 'nfs', {}));
  const srs = [];
  forEach(ensureArray(xml.SRlist.SR), sr => {
    srs.push({
      uuid: sr.UUID.trim()
    });
  });
  return srs;
}
probeNfsExists.params = {
  host: {
    type: 'string'
  },
  server: {
    type: 'string'
  },
  serverPath: {
    type: 'string'
  }
};
probeNfsExists.resolve = {
  host: ['host', 'host', 'administrate']
};
export const getAllUnhealthyVdiChainsLength = debounceWithKey(function getAllUnhealthyVdiChainsLength() {
  const unhealthyVdiChainsLengthBySr = {};
  filter(this.objects.all, obj => obj.type === 'SR' && obj.content_type !== 'iso' && obj.size > 0).forEach(sr => {
    const unhealthyVdiChainsLengthByVdi = this.getXapi(sr).getVdiChainsInfo(sr);
    if (!isEmpty(unhealthyVdiChainsLengthByVdi)) {
      unhealthyVdiChainsLengthBySr[sr.uuid] = unhealthyVdiChainsLengthByVdi;
    }
  });
  return unhealthyVdiChainsLengthBySr;
}, 60e3);
export function getVdiChainsInfo({
  sr
}) {
  return this.getXapi(sr).getVdiChainsInfo(sr);
}
getVdiChainsInfo.params = {
  id: {
    type: 'string'
  }
};
getVdiChainsInfo.resolve = {
  sr: ['id', 'SR', 'operate']
};
export function stats({
  sr,
  granularity
}) {
  return this.getXapiSrStats(sr._xapiId, granularity);
}
stats.description = 'returns statistic of the sr';
stats.params = {
  id: {
    type: 'string'
  },
  granularity: {
    type: 'string',
    optional: true
  }
};
stats.resolve = {
  sr: ['id', 'SR', 'view']
};
export function enableMaintenanceMode({
  sr,
  vmsToShutdown
}) {
  return this.getXapiObject(sr).$enableMaintenanceMode({
    vmsToShutdown
  });
}
enableMaintenanceMode.description = 'switch the SR into maintenance mode';
enableMaintenanceMode.params = {
  id: {
    type: 'string'
  },
  vmsToShutdown: {
    type: 'array',
    items: {
      type: 'string'
    },
    optional: true
  }
};
enableMaintenanceMode.permission = 'admin';
enableMaintenanceMode.resolve = {
  sr: ['id', 'SR', 'operate']
};
export function disableMaintenanceMode({
  sr
}) {
  return this.getXapiObject(sr).$disableMaintenanceMode();
}
disableMaintenanceMode.description = 'disable the maintenance of the SR';
disableMaintenanceMode.params = {
  id: {
    type: 'string'
  }
};
disableMaintenanceMode.permission = 'admin';
disableMaintenanceMode.resolve = {
  sr: ['id', 'SR', 'operate']
};
//# sourceMappingURL=sr.mjs.map