import Collection from '../collection/redis.mjs';
import { forEach, multiKeyHash } from '../utils.mjs';
const DEFAULT_ACTION = 'admin';
export class Acls extends Collection {
  create(subject, object, action) {
    return multiKeyHash(subject, object, action).then(hash => this.add({
      id: hash,
      subject,
      object,
      action
    }));
  }
  delete(subject, object, action) {
    return multiKeyHash(subject, object, action).then(hash => this.remove(hash));
  }
  aclExists(subject, object, action) {
    return multiKeyHash(subject, object, action).then(hash => this.exists(hash));
  }
  async get(properties) {
    const acls = await super.get(properties);
    const toUpdate = [];
    forEach(acls, acl => {
      if (!acl.action) {
        acl.action = DEFAULT_ACTION;
        toUpdate.push(acl);
      }
    });
    if (toUpdate.length) {
      await this.remove(toUpdate.map(_ => _.id));
      await Promise.all(toUpdate.map(acl => multiKeyHash(acl.subject, acl.object, acl.action).then(id => {
        acl.id = id;
      })));
      await this.add(toUpdate);
    }
    return acls;
  }
}
//# sourceMappingURL=acl.mjs.map