var _dec, _dec2, _obj;
function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) { var desc = {}; Object.keys(descriptor).forEach(function (key) { desc[key] = descriptor[key]; }); desc.enumerable = !!desc.enumerable; desc.configurable = !!desc.configurable; if ('value' in desc || desc.initializer) { desc.writable = true; } desc = decorators.slice().reverse().reduce(function (desc, decorator) { return decorator(target, property, desc) || desc; }, desc); if (context && desc.initializer !== void 0) { desc.value = desc.initializer ? desc.initializer.call(context) : void 0; desc.initializer = undefined; } if (desc.initializer === void 0) { Object.defineProperty(target, property, desc); desc = null; } return desc; }
import filter from 'lodash/filter.js';
import find from 'lodash/find.js';
import groupBy from 'lodash/groupBy.js';
import mapValues from 'lodash/mapValues.js';
import pickBy from 'lodash/pickBy.js';
import some from 'lodash/some.js';
import unzip from 'unzipper';
import { createLogger } from '@xen-orchestra/log';
import { decorateWith } from '@vates/decorate-with';
import { defer as deferrable } from 'golike-defer';
import { incorrectState } from 'xo-common/api-errors.js';
import { parseDateTime } from '@xen-orchestra/xapi';
import { timeout } from 'promise-toolbox';
import ensureArray from '../../_ensureArray.mjs';
import { debounceWithKey } from '../../_pDebounceWithKey.mjs';
import { forEach, mapFilter, parseXml } from '../../utils.mjs';
import { extractOpaqueRef, isHostRunning, useUpdateSystem } from '../utils.mjs';
const log = createLogger('xo:xapi');
const _isXcp = host => host.software_version.product_brand === 'XCP-ng';
const LISTING_DEBOUNCE_TIME_MS = 60000;
async function _listMissingPatches(hostId) {
  const host = this.getObject(hostId);
  return _isXcp(host) ? this._listXcpUpdates(host) : this._listInstallablePatches(host);
}
const listMissingPatches = debounceWithKey(_listMissingPatches, LISTING_DEBOUNCE_TIME_MS, hostId => hostId);
export default (_dec = decorateWith(debounceWithKey, 24 * 60 * 60 * 1000, function () {
  return this;
}), _dec2 = decorateWith(deferrable), (_obj = {
  async _getXenUpdates() {
    const response = await this.xo.httpRequest('https://updates.xensource.com/XenServer/updates.xml');
    const data = parseXml(await response.buffer()).patchdata;
    const patches = {
      __proto__: null
    };
    forEach(data.patches.patch, patch => {
      patches[patch.uuid] = {
        date: patch.timestamp,
        description: patch['name-description'],
        documentationUrl: patch.url,
        guidance: patch['after-apply-guidance'],
        name: patch['name-label'],
        url: patch['patch-url'],
        id: patch.uuid,
        uuid: patch.uuid,
        conflicts: ensureArray(patch.conflictingpatches).map(patch => patch.conflictingpatch.uuid),
        requirements: ensureArray(patch.requiredpatches).map(patch => patch.requiredpatch.uuid),
        paid: patch['update-stream'] === 'premium',
        upgrade: /^(XS|CH)\d{2,}$/.test(patch['name-label'])
      };
      if (patches[patch.uuid].conflicts[0] === undefined) {
        patches[patch.uuid].conflicts.length = 0;
      }
      if (patches[patch.uuid].requirements[0] === undefined) {
        patches[patch.uuid].requirements.length = 0;
      }
    });
    const resolveVersionPatches = function (uuids) {
      const versionPatches = {
        __proto__: null
      };
      forEach(ensureArray(uuids), ({
        uuid
      }) => {
        versionPatches[uuid] = patches[uuid];
      });
      return versionPatches;
    };
    const versions = {
      __proto__: null
    };
    let latestVersion;
    forEach(data.serverversions.version, version => {
      versions[version.value] = {
        date: version.timestamp,
        name: version.name,
        id: version.value,
        documentationUrl: version.url,
        patches: resolveVersionPatches(version.patch)
      };
      if (version.latest) {
        latestVersion = versions[version.value];
      }
    });
    return {
      patches,
      latestVersion,
      versions
    };
  },
  async _ejectToolsIsos(hostRef) {
    return Promise.all(mapFilter(this.objects.all, vm => {
      if (vm.$type !== 'vm' || hostRef && vm.resident_on !== hostRef) {
        return;
      }
      const shouldEjectCd = some(vm.$VBDs, vbd => {
        const vdi = vbd.$VDI;
        return vdi && vdi.is_tools_iso;
      });
      if (shouldEjectCd) {
        return this.ejectCdFromVm(vm.$id);
      }
    }));
  },
  async _listXcpUpdates(host) {
    const result = JSON.parse(await this.call('host.call_plugin', host.$ref, 'updater.py', 'check_update', {}));
    if (result.error != null) {
      throw new Error(result.error);
    }
    return result;
  },
  async _listPatches(host) {
    const versions = (await this._getXenUpdates()).versions;
    const hostVersions = host.software_version;
    const version = versions[hostVersions.product_version] || versions[hostVersions.product_version_text];
    return version ? pickBy(version.patches, patch => !patch.upgrade) : {};
  },
  _listInstalledPatches(host) {
    const installed = {
      __proto__: null
    };
    if (!useUpdateSystem(host)) {
      forEach(host.$patches, hostPatch => {
        installed[hostPatch.$pool_patch.uuid] = true;
      });
      return installed;
    }
    forEach(host.$updates, update => {
      if (update.name_label.startsWith('XS')) {
        installed[update.uuid] = true;
      }
    });
    return installed;
  },
  async _listInstallablePatches(host, requestedPatches) {
    const all = await this._listPatches(host);
    const installed = this._listInstalledPatches(host);
    let getAll = false;
    if (requestedPatches === undefined) {
      getAll = true;
      requestedPatches = Object.keys(all);
    }
    const freeHost = this.pool.$master.license_params.sku_type === 'free';
    const installable = [];
    forEach(requestedPatches, id => {
      const patch = all[id];
      if (patch === undefined) {
        throw new Error(`patch not found: ${id}`);
      }
      if (installed[id] !== undefined) {
        if (getAll) {
          return;
        }
        throw new Error(`patch already installed: ${patch.name} (${id})`);
      }
      if (find(installable, {
        id
      }) !== undefined) {
        return;
      }
      if (patch.paid && freeHost) {
        if (getAll) {
          return;
        }
        throw new Error(`requested patch ${patch.name} (${id}) requires a XenServer license`);
      }
      let conflictId;
      if ((conflictId = find(patch.conflicts, conflictId => installed[conflictId] !== undefined)) !== undefined) {
        if (getAll) {
          log.debug(`patch ${patch.name} (${id}) conflicts with installed patch ${conflictId}`);
          return;
        }
        throw new Error(`patch ${patch.name} (${id}) conflicts with installed patch ${conflictId}`);
      }
      if ((conflictId = find(patch.conflicts, conflictId => find(installable, {
        id: conflictId
      }))) !== undefined) {
        if (getAll) {
          log.debug(`patches ${id} and ${conflictId} conflict with eachother`);
          return;
        }
        throw new Error(`patches ${id} and ${conflictId} conflict with eachother`);
      }
      forEach(patch.requirements, id => {
        const requiredPatch = all[id];
        if (requiredPatch === undefined) {
          throw new Error(`required patch ${id} not found`);
        }
        if (!installed[id] && find(installable, {
          id
        }) === undefined) {
          if (requiredPatch.paid && freeHost) {
            throw new Error(`required patch ${requiredPatch.name} (${id}) requires a XenServer license`);
          }
          installable.push(requiredPatch);
        }
      });
      installable.push(patch);
    });
    return installable;
  },
  listMissingPatches,
  async findPatches(names) {
    const all = await this._listPatches(this.pool.$master);
    return filter(all, patch => names.includes(patch.name)).map(patch => patch.id);
  },
  async _xcpUpdate(hosts) {
    if (hosts === undefined) {
      hosts = Object.values(this.objects.indexes.type.host);
    }
    hosts = hosts.sort(({
      $ref
    }) => $ref === this.pool.master ? -1 : 1);
    for (const host of hosts) {
      const result = JSON.parse(await this.call('host.call_plugin', host.$ref, 'updater.py', 'update', {}));
      const {
        exit
      } = result;
      if (exit !== undefined && exit !== 0) {
        throw new Error(result.stderr);
      }
      log.debug(result.stdout);
      await host.update_other_config('rpm_patch_installation_time', String(Date.now() / 1000));
    }
  },
  async _legacyUploadPatch(uuid) {
    try {
      return this.getObjectByUuid(uuid);
    } catch (e) {}
    log.debug(`legacy downloading patch ${uuid}`);
    const patchInfo = (await this._getXenUpdates()).patches[uuid];
    if (!patchInfo) {
      throw new Error('no such patch ' + uuid);
    }
    let stream = await this.xo.httpRequest(patchInfo.url);
    stream = await new Promise((resolve, reject) => {
      const PATCH_RE = /\.xsupdate$/;
      stream.pipe(unzip.Parse()).on('entry', entry => {
        if (PATCH_RE.test(entry.path)) {
          entry.length = entry.vars.uncompressedSize;
          resolve(entry);
        } else {
          entry.autodrain();
        }
      }).on('error', reject);
    });
    const patchRef = await this.putResource(stream, '/pool_patch_upload', {
      task: this.task_create('Patch upload', patchInfo.name)
    }).then(extractOpaqueRef);
    return this._getOrWaitObject(patchRef);
  },
  async _uploadPatch($defer, uuid) {
    log.debug(`downloading patch ${uuid}`);
    const patchInfo = (await this._getXenUpdates()).patches[uuid];
    if (!patchInfo) {
      throw new Error('no such patch ' + uuid);
    }
    let stream = await this.xo.httpRequest(patchInfo.url);
    stream = await new Promise((resolve, reject) => {
      stream.pipe(unzip.Parse()).on('entry', entry => {
        entry.length = entry.vars.uncompressedSize;
        resolve(entry);
      }).on('error', reject);
    });
    const sr = this.findAvailableSr(stream.length);
    if (sr === undefined) {
      return;
    }
    const vdi = await this.createTemporaryVdiOnSr(stream, sr, '[XO] Patch ISO', 'small temporary VDI to store a patch ISO');
    $defer(() => vdi.$destroy());
    return vdi;
  },
  _poolWideInstall: deferrable(async function ($defer, patches) {
    if (!useUpdateSystem(this.pool.$master)) {
      for (const p of patches) {
        const [patch] = await Promise.all([this._legacyUploadPatch(p.uuid), this._ejectToolsIsos(this.pool.$master.$ref)]);
        await this.call('pool_patch.pool_apply', patch.$ref);
      }
      return;
    }
    for (const p of patches) {
      const [vdi] = await Promise.all([this._uploadPatch($defer, p.uuid), this._ejectToolsIsos()]);
      if (vdi === undefined) {
        throw new Error('patch could not be uploaded');
      }
      log.debug(`installing patch ${p.uuid}`);
      await this.call('pool_update.pool_apply', await this.call('pool_update.introduce', vdi.$ref));
    }
  }),
  async _hostInstall(patches, host) {
    throw new Error('single host install not implemented');
  },
  async installPatches({
    patches,
    hosts
  } = {}) {
    if (_isXcp(this.pool.$master)) {
      return this._xcpUpdate(hosts);
    }
    const poolWide = hosts === undefined;
    if (poolWide) {
      log.debug('patches that were requested to be installed', patches);
      const installablePatches = await this._listInstallablePatches(this.pool.$master, patches);
      log.debug('patches that will actually be installed', installablePatches.map(patch => patch.uuid));
      return this._poolWideInstall(installablePatches);
    }
    throw new Error('non pool-wide install not implemented');
  },
  async rollingPoolUpdate($defer) {
    const isXcp = _isXcp(this.pool.$master);
    if (this.pool.ha_enabled) {
      const haSrs = this.pool.$ha_statefiles.map(vdi => vdi.SR);
      const haConfig = this.pool.ha_configuration;
      await this.call('pool.disable_ha');
      $defer(() => this.call('pool.enable_ha', haSrs, haConfig));
    }
    const hosts = filter(this.objects.all, {
      $type: 'host'
    });
    {
      const deadHost = hosts.find(_ => !isHostRunning(_));
      if (deadHost !== undefined) {
        throw incorrectState({
          actual: 'Halted',
          expected: 'Running',
          object: deadHost.$id,
          property: 'power_state'
        });
      }
    }
    await Promise.all(hosts.map(host => host.$call('assert_can_evacuate')));
    if (!isXcp) {
      log.debug('Install patches');
      await this.installPatches();
    }
    const vmsByHost = mapValues(groupBy(filter(this.objects.all, {
      $type: 'VM',
      power_state: 'Running',
      is_control_domain: false
    }), vm => {
      var _vm$$resident_on;
      const hostId = (_vm$$resident_on = vm.$resident_on) === null || _vm$$resident_on === void 0 ? void 0 : _vm$$resident_on.$id;
      if (hostId === undefined) {
        throw new Error('Could not find host of all running VMs');
      }
      return hostId;
    }), vms => vms.map(vm => vm.$id));
    const indexOfMaster = hosts.findIndex(host => host.$ref === this.pool.master);
    if (indexOfMaster === -1) {
      throw new Error('Could not find pool master');
    }
    ;
    [hosts[0], hosts[indexOfMaster]] = [hosts[indexOfMaster], hosts[0]];
    for (const host of hosts) {
      const hostId = host.uuid;
      const metricsRef = host.metrics;
      await this.barrier(metricsRef);
      await this._waitObjectState(metricsRef, metrics => metrics.live);
      const getServerTime = async () => parseDateTime(await this.call('host.get_servertime', host.$ref)) * 1e3;
      let rebootTime;
      if (isXcp) {
        log.debug(`Evacuate host ${hostId}`);
        await this.clearHost(host);
        log.debug(`Install patches on host ${hostId}`);
        await this.installPatches({
          hosts: [host]
        });
        log.debug(`Restart host ${hostId}`);
        rebootTime = await getServerTime();
        await this.callAsync('host.reboot', host.$ref);
      } else {
        log.debug(`Evacuate and restart host ${hostId}`);
        rebootTime = await getServerTime();
        await this.rebootHost(hostId);
      }
      log.debug(`Wait for host ${hostId} to be up`);
      await timeout.call((async () => {
        await this._waitObjectState(hostId, host => host.enabled && rebootTime < host.other_config.agent_start_time * 1e3);
        await this._waitObjectState(metricsRef, metrics => metrics.live);
      })(), this._restartHostTimeout, new Error(`Host ${hostId} took too long to restart`));
      log.debug(`Host ${hostId} is up`);
    }
    log.debug('Migrate VMs back to where they were');
    [hosts[0], hosts[hosts.length - 1]] = [hosts[hosts.length - 1], hosts[0]];
    let error;
    for (const host of hosts) {
      const hostId = host.uuid;
      const vmIds = vmsByHost[hostId];
      if (vmIds === undefined) {
        continue;
      }
      for (const vmId of vmIds) {
        try {
          await this.migrateVm(vmId, this, hostId);
        } catch (err) {
          log.error(err);
          if (error === undefined) {
            error = err;
          }
        }
      }
    }
    if (error !== undefined) {
      throw error;
    }
  }
}, (_applyDecoratedDescriptor(_obj, "_getXenUpdates", [_dec], Object.getOwnPropertyDescriptor(_obj, "_getXenUpdates"), _obj), _applyDecoratedDescriptor(_obj, "rollingPoolUpdate", [_dec2], Object.getOwnPropertyDescriptor(_obj, "rollingPoolUpdate"), _obj)), _obj));
//# sourceMappingURL=patching.mjs.map