import Disposable from 'promise-toolbox/Disposable';
import { execa } from 'execa';
export default class BackupNgFileRestore {
  constructor(app) {
    this._app = app;
    this._mounts = {
      __proto__: null
    };
  }
  async fetchBackupNgPartitionFiles(remoteId, diskId, partitionId, paths) {
    const app = this._app;
    const remote = await app.getRemoteWithCredentials(remoteId);
    return remote.proxy !== undefined ? app.callProxyMethod(remote.proxy, 'backup.fetchPartitionFiles', {
      disk: diskId,
      remote: {
        url: remote.url,
        options: remote.options
      },
      partition: partitionId,
      paths
    }, {
      assertType: 'stream'
    }) : Disposable.use(app.getBackupsRemoteAdapter(remote), adapter => adapter.fetchPartitionFiles(diskId, partitionId, paths));
  }
  async listBackupNgDiskPartitions(remoteId, diskId) {
    const app = this._app;
    const remote = await app.getRemoteWithCredentials(remoteId);
    if (remote.proxy !== undefined) {
      const stream = await app.callProxyMethod(remote.proxy, 'backup.listDiskPartitions', {
        disk: diskId,
        remote: {
          url: remote.url,
          options: remote.options
        }
      }, {
        assertType: 'iterator'
      });
      const partitions = [];
      for await (const partition of stream) {
        partitions.push(partition);
      }
      return partitions;
    } else {
      return Disposable.use(app.getBackupsRemoteAdapter(remote), adapter => adapter.listPartitions(diskId));
    }
  }
  async listBackupNgPartitionFiles(remoteId, diskId, partitionId, path) {
    const app = this._app;
    const remote = await app.getRemoteWithCredentials(remoteId);
    return remote.proxy !== undefined ? app.callProxyMethod(remote.proxy, 'backup.listPartitionFiles', {
      disk: diskId,
      remote: {
        url: remote.url,
        options: remote.options
      },
      partition: partitionId,
      path
    }) : Disposable.use(app.getBackupsRemoteAdapter(remote), adapter => adapter.listPartitionFiles(diskId, partitionId, path));
  }
}
//# sourceMappingURL=file-restore-ng.mjs.map