var _dec, _class;
function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) { var desc = {}; Object.keys(descriptor).forEach(function (key) { desc[key] = descriptor[key]; }); desc.enumerable = !!desc.enumerable; desc.configurable = !!desc.configurable; if ('value' in desc || desc.initializer) { desc.writable = true; } desc = decorators.slice().reverse().reduce(function (desc, decorator) { return decorator(target, property, desc) || desc; }, desc); if (context && desc.initializer !== void 0) { desc.value = desc.initializer ? desc.initializer.call(context) : void 0; desc.initializer = undefined; } if (desc.initializer === void 0) { Object.defineProperty(target, property, desc); desc = null; } return desc; }
import { Backup } from '@xen-orchestra/backups/Backup.js';
import { decorateWith } from '@vates/decorate-with';
import { defer as deferrable } from 'golike-defer';
import { fromEvent } from 'promise-toolbox';
import { Task } from '@xen-orchestra/mixins/Tasks.mjs';
import { v4 as generateUuid } from 'uuid';
import { VDI_FORMAT_VHD } from '@xen-orchestra/xapi';
import asyncMapSettled from '@xen-orchestra/async-map/legacy.js';
import Esxi from '@xen-orchestra/vmware-explorer/esxi.mjs';
import openDeltaVmdkasVhd from '@xen-orchestra/vmware-explorer/openDeltaVmdkAsVhd.mjs';
import OTHER_CONFIG_TEMPLATE from '../xapi/other-config-template.mjs';
import VhdEsxiRaw from '@xen-orchestra/vmware-explorer/VhdEsxiRaw.mjs';
let MigrateVm = (_dec = decorateWith(deferrable), (_class = class MigrateVm {
  constructor(app) {
    this._app = app;
  }
  #createWarmBackup(sourceVmId, srId, jobId) {
    const app = this._app;
    const config = {
      snapshotNameLabelTpl: '[XO warm migration {job.name}] {vm.name_label}'
    };
    const job = {
      type: 'backup',
      id: jobId,
      mode: 'delta',
      vms: {
        id: sourceVmId
      },
      name: `Warm migration`,
      srs: {
        id: srId
      },
      settings: {
        '': {
          copyRetention: 1,
          _warmMigration: true
        }
      }
    };
    const schedule = {
      id: 'one-time'
    };
    return new Backup({
      config,
      job,
      schedule,
      getAdapter: async remoteId => app.getBackupsRemoteAdapter(await app.getRemoteWithCredentials(remoteId)),
      getConnectedRecord: async (xapiType, uuid) => app.getXapiObject(uuid)
    });
  }
  async warmMigrateVm(sourceVmId, srId, startDestVm = true, deleteSource = false) {
    const jobId = generateUuid();
    const app = this._app;
    const sourceVm = app.getXapiObject(sourceVmId);
    let backup = this.#createWarmBackup(sourceVmId, srId, jobId);
    await backup.run();
    const xapi = sourceVm.$xapi;
    const ref = sourceVm.$ref;
    try {
      await xapi.callAsync('VM.clean_shutdown', ref);
    } catch (error) {
      await xapi.callAsync('VM.hard_shutdown', ref);
    }
    const message = 'This VM has been migrated somewhere else and might not be up to date, check twice before starting it.';
    await sourceVm.update_blocked_operations({
      start: message,
      start_on: message
    });
    backup = this.#createWarmBackup(sourceVmId, srId, jobId);
    await backup.run();
    const targets = Object.keys(app.getObjects({
      filter: obj => {
        return 'other' in obj && obj.other['xo:backup:job'] === jobId && obj.other['xo:backup:sr'] === srId && obj.other['xo:backup:vm'] === sourceVm.uuid && 'start' in obj.blockedOperations;
      }
    }));
    if (targets.length === 0) {
      throw new Error(`Vm target of warm migration not found for ${sourceVmId} on SR ${srId} `);
    }
    if (targets.length > 1) {
      throw new Error(`Multiple target of warm migration found for ${sourceVmId} on SR ${srId} `);
    }
    const targetVm = app.getXapiObject(targets[0]);
    await targetVm.update_blocked_operations({
      start: null,
      start_on: null
    });
    if (startDestVm) {
      await targetVm.$xapi.startVm(targetVm.$ref);
      if (deleteSource) {
        sourceVm.$xapi.VM_destroy(sourceVm.$ref);
      } else {}
    }
  }
  #buildDiskChainByNode(disks, snapshots) {
    let chain = [];
    if (snapshots && snapshots.current) {
      const currentSnapshotId = snapshots.current;
      let currentSnapshot = snapshots.snapshots.find(({
        uid
      }) => uid === currentSnapshotId);
      chain = [currentSnapshot.disks];
      while (currentSnapshot = snapshots.snapshots.find(({
        uid
      }) => uid === currentSnapshot.parent)) {
        chain.push(currentSnapshot.disks);
      }
      chain.reverse();
    }
    chain.push(disks);
    for (const disk of chain) {
      if (disk.capacity > 2 * 1024 * 1024 * 1024 * 1024) {
        throw new Error("Can't migrate disks larger than 2TiB");
      }
    }
    const chainsByNodes = {};
    chain.forEach(disks => {
      disks.forEach(disk => {
        chainsByNodes[disk.node] = chainsByNodes[disk.node] || [];
        chainsByNodes[disk.node].push(disk);
      });
    });
    return chainsByNodes;
  }
  #connectToEsxi(host, user, password, sslVerify) {
    return Task.run({
      data: {
        name: `connecting to ${host}`
      }
    }, async () => {
      const esxi = new Esxi(host, user, password, sslVerify);
      await fromEvent(esxi, 'ready');
      return esxi;
    });
  }
  async connectToEsxiAndList({
    host,
    user,
    password,
    sslVerify
  }) {
    const esxi = await this.#connectToEsxi(host, user, password, sslVerify);
    return esxi.getAllVmMetadata();
  }
  async migrationfromEsxi($defer, {
    host,
    user,
    password,
    sslVerify,
    sr: srId,
    network: networkId,
    vm: vmId,
    thin,
    stopSource
  }) {
    const app = this._app;
    const esxi = await this.#connectToEsxi(host, user, password, sslVerify);
    const esxiVmMetadata = await Task.run({
      data: {
        name: `get metadata of ${vmId}`
      }
    }, async () => {
      return esxi.getTransferableVmMetadata(vmId);
    });
    const {
      disks,
      firmware,
      memory,
      name_label,
      networks,
      nCpus,
      powerState,
      snapshots
    } = esxiVmMetadata;
    const isRunning = powerState !== 'poweredOff';
    const chainsByNodes = await Task.run({
      data: {
        name: `build disks and snapshots chains for ${vmId}`
      }
    }, async () => {
      return this.#buildDiskChainByNode(disks, snapshots);
    });
    const sr = app.getXapiObject(srId);
    const xapi = sr.$xapi;
    const vm = await Task.run({
      data: {
        name: 'creating MV on XCP side'
      }
    }, async () => {
      const vm = await xapi._getOrWaitObject(await xapi.VM_create({
        ...OTHER_CONFIG_TEMPLATE,
        memory_dynamic_max: memory,
        memory_dynamic_min: memory,
        memory_static_max: memory,
        memory_static_min: memory,
        name_description: 'from esxi',
        name_label,
        VCPUs_at_startup: nCpus,
        VCPUs_max: nCpus
      }));
      await Promise.all([vm.update_HVM_boot_params('firmware', firmware), vm.update_platform('device-model', 'qemu-upstream-' + (firmware === 'uefi' ? 'uefi' : 'compat')), asyncMapSettled(['start', 'start_on'], op => vm.update_blocked_operations(op, 'Esxi migration in progress...')), vm.set_name_label(`[Importing...] ${name_label}`)]);
      const vifDevices = await xapi.call('VM.get_allowed_VIF_devices', vm.$ref);
      await Promise.all(networks.map((network, i) => xapi.VIF_create({
        device: vifDevices[i],
        network: xapi.getObject(networkId).$ref,
        VM: vm.$ref
      }, {
        MAC: network.macAddress
      })));
      return vm;
    });
    $defer.onFailure.call(xapi, 'VM_destroy', vm.$ref);
    const vhds = await Promise.all(Object.keys(chainsByNodes).map(async (node, userdevice) => Task.run({
      data: {
        name: `Cold import of disks ${node}`
      }
    }, async () => {
      const chainByNode = chainsByNodes[node];
      const vdi = await xapi._getOrWaitObject(await xapi.VDI_create({
        name_description: 'fromESXI' + chainByNode[0].descriptionLabel,
        name_label: '[ESXI]' + chainByNode[0].nameLabel,
        SR: sr.$ref,
        virtual_size: chainByNode[0].capacity
      }));
      $defer.onFailure.call(xapi, 'VDI_destroy', vdi.$ref);
      await xapi.VBD_create({
        VDI: vdi.$ref,
        VM: vm.$ref
      });
      let parentVhd, vhd;
      const nbColdDisks = isRunning ? chainByNode.length - 1 : chainByNode.length;
      for (let diskIndex = 0; diskIndex < nbColdDisks; diskIndex++) {
        const disk = chainByNode[diskIndex];
        const {
          fileName,
          path,
          datastore,
          isFull
        } = disk;
        if (isFull) {
          vhd = await VhdEsxiRaw.open(esxi, datastore, path + '/' + fileName, {
            thin
          });
          await vhd.readBlockAllocationTable();
        } else {
          vhd = await openDeltaVmdkasVhd(esxi, datastore, path + '/' + fileName, parentVhd);
        }
        parentVhd = vhd;
      }
      if (vhd !== undefined) {
        const stream = vhd.stream();
        await vdi.$importContent(stream, {
          format: VDI_FORMAT_VHD
        });
      }
      return {
        vdi,
        vhd
      };
    })));
    if (isRunning && stopSource) {
      await Task.run({
        data: {
          name: 'powering down source VM'
        }
      }, () => esxi.powerOff(vmId));
      await Promise.all(Object.keys(chainsByNodes).map(async (node, userdevice) => {
        await Task.run({
          data: {
            name: `Transfering deltas of ${userdevice}`
          }
        }, async () => {
          const chainByNode = chainsByNodes[node];
          const disk = chainByNode[chainByNode.length - 1];
          const {
            fileName,
            path,
            datastore,
            isFull
          } = disk;
          const {
            vdi,
            vhd: parentVhd
          } = vhds[userdevice];
          let vhd;
          if (vdi === undefined) {
            throw new Error(`Can't import delta of a running VM without its parent vdi`);
          }
          if (isFull) {
            vhd = await VhdEsxiRaw.open(esxi, datastore, path + '/' + fileName, {
              thin
            });
            await vhd.readBlockAllocationTable();
          } else {
            if (parentVhd === undefined) {
              throw new Error(`Can't import delta of a running VM without its parent VHD`);
            }
            vhd = await openDeltaVmdkasVhd(esxi, datastore, path + '/' + fileName, parentVhd, {
              lookMissingBlockInParent: false
            });
          }
          const stream = vhd.stream();
          await vdi.$importContent(stream, {
            format: VDI_FORMAT_VHD
          });
        });
      }));
    }
    await Task.run({
      data: {
        name: 'Finishing transfer'
      }
    }, async () => {
      await vm.set_name_label(esxiVmMetadata.name_label);
      await asyncMapSettled(['start', 'start_on'], op => vm.update_blocked_operations(op, null));
    });
    return vm.uuid;
  }
}, (_applyDecoratedDescriptor(_class.prototype, "migrationfromEsxi", [_dec], Object.getOwnPropertyDescriptor(_class.prototype, "migrationfromEsxi"), _class.prototype)), _class));
export { MigrateVm as default };
//# sourceMappingURL=migrate-vm.mjs.map