"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "ParsableFile", {
  enumerable: true,
  get: function () {
    return _ovaRead.ParsableFile;
  }
});
exports.computeVmdkLength = computeVmdkLength;
Object.defineProperty(exports, "parseOVAFile", {
  enumerable: true,
  get: function () {
    return _ovaRead.parseOVAFile;
  }
});
Object.defineProperty(exports, "readCapacityAndGrainTable", {
  enumerable: true,
  get: function () {
    return _vmdkReadTable.readCapacityAndGrainTable;
  }
});
Object.defineProperty(exports, "readVmdkGrainTable", {
  enumerable: true,
  get: function () {
    return _vmdkReadTable.default;
  }
});
exports.vhdToVMDK = vhdToVMDK;
exports.vhdToVMDKIterator = vhdToVMDKIterator;
exports.vmdkToVhd = vmdkToVhd;
Object.defineProperty(exports, "writeOvaOn", {
  enumerable: true,
  get: function () {
    return _ovaGenerate.writeOvaOn;
  }
});
var _asyncIteratorToStream = _interopRequireDefault(require("async-iterator-to-stream"));
var _createReadableSparseStream = _interopRequireDefault(require("vhd-lib/createReadableSparseStream.js"));
var _ovaRead = require("./ova-read");
var _vmdkRead = _interopRequireDefault(require("./vmdk-read"));
var _vmdkGenerate = require("./vmdk-generate");
var _ovaGenerate = require("./ova-generate");
var _parseVhdToBlocks = require("./parseVhdToBlocks");
var _vmdkReadTable = _interopRequireWildcard(require("./vmdk-read-table"));
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
async function vmdkToVhd(vmdkReadStream, grainLogicalAddressList, grainFileOffsetList, gzipped = false, length) {
  const parser = new _vmdkRead.default(vmdkReadStream, grainLogicalAddressList, grainFileOffsetList, gzipped, length);
  const header = await parser.readHeader();
  return (0, _createReadableSparseStream.default)(header.capacitySectors * 512, header.grainSizeSectors * 512, grainLogicalAddressList, parser.blockIterator());
}
async function computeVmdkLength(diskName, vhdReadStream) {
  let length = 0;
  const {
    iterator
  } = await vhdToVMDKIterator(diskName, vhdReadStream);
  for await (const b of iterator) {
    length += b.length;
  }
  return length;
}
async function vhdToVMDK(diskName, vhdReadStreamGetter, withLength = false) {
  const {
    iterator,
    size
  } = await vhdToVMDKIterator(diskName, await vhdReadStreamGetter());
  let length;
  const stream = await (0, _asyncIteratorToStream.default)(iterator);
  if (withLength) {
    if (size === undefined) {
      length = await computeVmdkLength(diskName, await vhdReadStreamGetter());
    } else {
      length = size;
    }
    stream.length = length;
  }
  return stream;
}
async function vhdToVMDKIterator(diskName, vhdReadStream) {
  const {
    blockSize,
    blockCount,
    blocks,
    diskSize,
    geometry
  } = await (0, _parseVhdToBlocks.parseVhdToBlocks)(vhdReadStream);
  const dataSize = blockSize * blockCount;
  const {
    iterator,
    metadataSize
  } = await (0, _vmdkGenerate.generateVmdkData)(diskName, diskSize, blockSize, blocks, geometry, dataSize);
  return {
    iterator,
    size: dataSize + metadataSize
  };
}
//# sourceMappingURL=index.js.map